/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "exception.h"

enum class Announcement {
  noannouncement,
  no120, // means re or contra
  no90,
  no60,
  no30,
  no0,
  // a reply or a re/contra as reply to another announcement (no further announcement can be made)
  // this is only announced, when the rule 'announcement_strict_limit' is inactive
  reply,
  // these are only announced, when the rule 'announcement_strict_limit' is active
  no120_reply,
  no90_reply,
  no60_reply,
  no30_reply,
  no0_reply,
}; // enum class Announcement

constexpr Announcement announcement_list[] = {
  Announcement::noannouncement,
  Announcement::no120,
  Announcement::no90,
  Announcement::no60,
  Announcement::no30,
  Announcement::no0,
  Announcement::reply,
  Announcement::no120_reply,
  Announcement::no90_reply,
  Announcement::no60_reply,
  Announcement::no30_reply,
  Announcement::no0_reply,
};
constexpr Announcement real_announcements[] = {
  Announcement::no120,
  Announcement::no90,
  Announcement::no60,
  Announcement::no30,
  Announcement::no0,
};

bool operator<(Announcement lhs, Announcement rhs) noexcept;

string to_string(Announcement announcement) noexcept;
string gettext(Announcement announcement);
ostream& operator<<(ostream& ostr, Announcement announcement);
Announcement announcement_from_string(string const& name);

bool operator!(Announcement announcement) noexcept;
bool operator&&(bool lhs, Announcement rhs) noexcept;
bool operator&&(Announcement lhs, bool rhs) noexcept;
bool operator||(bool lhs, Announcement rhs) noexcept;
bool operator||(Announcement lhs, bool rhs) noexcept;
bool operator<(Announcement lhs, Announcement rhs) noexcept;
bool operator<=(Announcement lhs, Announcement rhs) noexcept;

bool is_real(Announcement announcement) noexcept;
bool is_reply(Announcement announcement) noexcept;
Announcement to_reply(Announcement announcement);
Announcement from_reply(Announcement announcement);
Announcement previous(Announcement announcement);
Announcement next(Announcement announcement);

// the points needed for the announcement
unsigned needed_points(Announcement announcement) noexcept;
// the points needed for the opposite team to win
unsigned point_limit_for_opposite_team(Announcement announcement) noexcept;

struct AnnouncementWithTrickno {
    AnnouncementWithTrickno() noexcept = default;
    AnnouncementWithTrickno(Announcement const announcement,
                            unsigned const trickno) noexcept :
      announcement(announcement), trickno(trickno)
  { }
    operator Announcement const&() const noexcept
    { return this->announcement; }
    Announcement announcement = Announcement::noannouncement;
    unsigned trickno = UINT_MAX;
}; // struct AnnouncementWithTrickno
