/**********************************************************************
 *
 *   freedoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "bock_trigger.h"

/** @return   the name of the bock trigger
 **/
string
to_string(BockTrigger const bock_trigger)
{
  switch (bock_trigger) {
  case BockTrigger::equal_points:
    return "equal points";
    (void)_("BockTrigger::equal points");
  case BockTrigger::solo_lost:
    return "solo lost";
    (void)_("BockTrigger::solo lost");
  case BockTrigger::re_lost:
    return "re lost";
    (void)_("BockTrigger::re lost");
  case BockTrigger::contra_lost:
    return "contra lost";
    (void)_("BockTrigger::contra lost");
  case BockTrigger::heart_trick:
    return "heart trick";
    (void)_("BockTrigger::heart trick");
  case BockTrigger::black:
    return "black";
    (void)_("BockTrigger::black");
  } // switch (bock_trigger)

  DEBUG_ASSERTION(false,
                  "to_string(BockTrigger):\n"
                  "  no name found for " << static_cast<int>(bock_trigger)
                 );

  return "";
} // string to_string(BockTrigger bock_trigger)

/** -> result
 **
 ** @param    bock_trigger   bock trigger
 **
 ** @return   translation of the bock trigger
 **/
string
gettext(BockTrigger const bock_trigger)
{
  return gettext("BockTrigger::" + to_string(bock_trigger));
}

/** -> result
 **
 ** @param    name   bock trigger name
 **
 ** @return   the bock trigger with name 'name'
 **/
BockTrigger
bock_trigger_from_string(string const& name)
{
  for (auto const bt : bock_trigger_list)
    if (name == to_string(bt))
      return bt;

  throw ReadException("unknown bock trigger '" + name + "'");
  return BockTrigger::equal_points;
} // BockTrigger bock_trigger_from_string(string name)

/** writes the bock trigger into the stream
 **
 ** @param    ostr           output stream
 ** @param    bock_trigger   bock trigger
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, BockTrigger const bock_trigger)
{ return (ostr << to_string(bock_trigger)); }

