/**********************************************************************
 *
 *   freedoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "game_type.h"

/** -> result
 **
 ** @param    game_type   game_type
 **
 ** @return   name of the game type
 **/
string
to_string(GameType const game_type) noexcept
{
  switch(game_type) {
  case GameType::normal:
    return "normal";
    (void)_("GameType::normal");
  case GameType::thrown_nines:
    return "thrown nines";
    (void)_("GameType::thrown nines");
  case GameType::thrown_kings:
    return "thrown kings";
    (void)_("GameType::thrown kings");
  case GameType::thrown_nines_and_kings:
    return "thrown nines and kings";
    (void)_("GameType::thrown nines and kings");
  case GameType::thrown_richness:
    return "thrown richness";
    (void)_("GameType::thrown richness");
  case GameType::fox_highest_trump:
    return "fox highest trump";
    (void)_("GameType::fox highest trump");
  case GameType::redistribute:
    return "redistribute";
    (void)_("GameType::redistribute");
  case GameType::poverty:
    return "poverty";
    (void)_("GameType::poverty");
  case GameType::genscher:
    return "genscher";
    (void)_("GameType::genscher");
  case GameType::marriage:
    return "marriage";
    (void)_("GameType::marriage");
  case GameType::marriage_solo:
    return "marriage solo";
    (void)_("GameType::marriage solo");
  case GameType::marriage_silent:
    return "marriage silent";
    (void)_("GameType::marriage silent");
  case GameType::solo_jack:
    return "solo jack";
    (void)_("GameType::solo jack");
  case GameType::solo_queen:
    return "solo queen";
    (void)_("GameType::solo queen");
  case GameType::solo_king:
    return "solo king";
    (void)_("GameType::solo king");
  case GameType::solo_queen_jack:
    return "solo queen-jack";
    (void)_("GameType::solo queen-jack");
  case GameType::solo_king_jack:
    return "solo king-jack";
    (void)_("GameType::solo king-jack");
  case GameType::solo_king_queen:
    return "solo king-queen";
    (void)_("GameType::solo king-queen");
  case GameType::solo_koehler:
    return "solo koehler";
    (void)_("GameType::solo koehler");
  case GameType::solo_club:
    return "solo club";
    (void)_("GameType::solo club");
  case GameType::solo_spade:
    return "solo spade";
    (void)_("GameType::solo spade");
  case GameType::solo_heart:
    return "solo heart";
    (void)_("GameType::solo heart");
  case GameType::solo_diamond:
    return "solo diamond";
    (void)_("GameType::solo diamond");
  case GameType::solo_meatless:
    return "solo meatless";
    (void)_("GameType::solo meatless");
  } // switch(gametype)

  DEBUG_ASSERTION(false,
                  "to_string(GameType):\n"
                  "  game type not found: " << static_cast<int>(game_type));

  return "";
} // string to_string(GameType game_type)

/** -> result
 **
 ** @param    game_type   game type
 **
 ** @return   translation of the game type
 **/
string
gettext(GameType const game_type)
{
  return gettext("GameType::" + to_string(game_type));
}

/** writes the gametype into the stream
 **
 ** @param    ostr        output stream
 ** @param    game_type   game type
 **
 ** @return   output stream
 **/
ostream&
operator<<(ostream& ostr, GameType const game_type)
{ return (ostr << to_string(game_type)); }

/** -> result
 **
 ** @param    name   gametype name
 **
 ** @return   the gametype with name 'name'
 **/
GameType
game_type_from_string(string const& name)
{
  for (auto const g : game_type_list)
    if (name == to_string(g))
      return g;

  throw ReadException("unknown game type '" + name + "'");
  return GameType::normal;
} // GameType game_type_from_string(string name)

/** -> return
 **
 ** @param    game_type   the game type
 **
 ** @return   whether 'game_type' is a game where the teams are not known in the beginning
 **/
bool
is_with_unknown_teams(GameType const game_type) noexcept
{
  switch (game_type) {
  case GameType::normal:
  case GameType::marriage_silent:
  case GameType::genscher:
    return true;
  default:
    return false;
  } // switch(game_type)
} // bool GameType::is_normal(GameType game_type)

/** -> return
 **
 ** @param    game_type   the game type
 **
 ** @return   whether 'game_type' is a 'normal' game
 **/
bool
is_normal(GameType const game_type) noexcept
{
  switch (game_type) {
  case GameType::normal:
  case GameType::poverty:
  case GameType::marriage:
  case GameType::genscher:
    return true;
  default:
    return false;
  } // switch(game_type)
} // bool GameType::is_normal(GameType game_type)

/** -> return
 **
 ** @param    game_type   the game type
 **
 ** @return   whether 'game_type' is a marriage
 **/
bool
is_marriage(GameType const game_type) noexcept
{
  switch (game_type) {
  case GameType::marriage:
  case GameType::marriage_solo:
  case GameType::marriage_silent:
    return true;
  default:
    return false;
  } // switch(game_type)
} // bool GameType::is_marriage(GameType game_type)

/** -> return
 **
 ** @param    game_type   the game type
 **
 ** @return   whether 'game_type' is a poverty
 **/
bool
is_poverty(GameType const game_type) noexcept
{
  switch (game_type) {
  case GameType::thrown_nines:
  case GameType::thrown_kings:
  case GameType::thrown_nines_and_kings:
  case GameType::fox_highest_trump:
  case GameType::redistribute:
  case GameType::poverty:
    return true;
  default:
    return false;
  } // switch(game_type)
} // bool GameType::is_poverty(GameType game_type)

/** -> return
 **
 ** @param    game_type   the game type
 **
 ** @return   whether 'game_type' is a real solo (t.i. no marriage solo)
 **/
bool
is_real_solo(GameType const game_type) noexcept
{
  return (   is_color_solo(game_type)
          || is_picture_solo(game_type)
          || (game_type == GameType::solo_meatless) );
} // bool is_real_solo(GameType game_type)

/** -> return
 **
 ** @param    game_type   the game type
 **
 ** @return   whether 'game_type' is a solo
 **/
bool
is_solo(GameType const game_type) noexcept
{
  return (   is_real_solo(game_type)
          || (game_type == GameType::marriage_solo)
          || (game_type == GameType::marriage_silent) );
} // bool is_solo(GameType game_type)

/** -> return
 **
 ** @param    game_type   the game type
 **
 ** @return   whether 'game_type' is a color solo
 **/
bool
is_color_solo(GameType const game_type) noexcept
{
  switch (game_type) {
  case GameType::solo_club:
  case GameType::solo_spade:
  case GameType::solo_heart:
  case GameType::solo_diamond:
    return true;
  default:
    return false;
  } // switch(game_type)
} // bool is_color_solo(GameType game_type)

/** -> return
 **
 ** @param    game_type   the game type
 **
 ** @return   whether 'game_type' is a picture solo
 **/
bool
is_picture_solo(GameType const game_type) noexcept
{
  return (   is_single_picture_solo(game_type)
          || is_double_picture_solo(game_type)
          || (game_type == GameType::solo_koehler));
} // bool is_picture_solo(GameType game_type)

/** -> return
 **
 ** @param    game_type   the game type
 **
 ** @return   whether 'game_type' is a single picture solo
 **/
bool
is_single_picture_solo(GameType const game_type) noexcept
{
  switch (game_type) {
  case GameType::solo_jack:
  case GameType::solo_queen:
  case GameType::solo_king:
    return true;
  default:
    return false;
  } // switch(game_type)
} // bool is_single_picture_solo(GameType game_type)

/** -> return
 **
 ** @param    game_type   the game type
 **
 ** @return   whether 'game_type' is a double picture solo
 **/
bool
is_double_picture_solo(GameType const game_type) noexcept
{
  switch (game_type) {
  case GameType::solo_queen_jack:
  case GameType::solo_king_jack:
  case GameType::solo_king_queen:
    return true;
  default:
    return false;
  } // switch(game_type)
} // bool is_double_picture_solo(GameType game_type)
