/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#ifndef CONSTANTS_HEADER
#define CONSTANTS_HEADER

// using the microsoft visual C++ compiler
#ifdef _MSC_VER
#define MS_VISUAL_C
#define WINDOWS
#endif

// C++ include

#include <typeinfo>
#include <iostream>
using std::ostream;
using std::istream;
using std::cout;
using std::clog;
using std::cerr;
using std::endl;
using std::flush;
#include <iomanip>
using std::setw;
#include <fstream>
using std::ifstream;
using std::ofstream;
#include <sstream>
using std::stringstream;
using std::istringstream;
using std::ostringstream;
#include <algorithm>
using std::min;
using std::max;

// for temporary output
// ToDo: use null_ostr in the release
#define COUT cout
#define CLOG cout << __FILE__ << '#' << __LINE__ << "  "

#define LOGreturn if(CLOG << "return" << endl, true) return


#define ENUM_WITH_LIST(EnumName, enum_name_list, first_value, type1, ...) \
  enum EnumName { type1 = first_value, __VA_ARGS__ }; \
  constexpr EnumName enum_name_list[] = { type1, __VA_ARGS__ }

#define ENUM_WITH_STATIC_LIST(EnumName, enum_name_list, first_value, type1, ...) \
  enum EnumName { type1 = first_value, __VA_ARGS__ }; \
  static constexpr EnumName enum_name_list[] = { type1, __VA_ARGS__ }


using std::pair;
#include <vector>
using std::vector;
#include <queue>
using std::queue;
#include <map>
using std::map;
#include <set>
using std::set;
#include <string>
using std::string;
using namespace std::literals::string_literals;

#include <memory>
using std::unique_ptr;
using std::make_unique;
#include <functional>
using std::function;
#include "utils/container_algorithm.h"

#include "utils/optional.h"

#include "class/signal.h"
#include "signals.h"


using Seed = std::uint_fast32_t; // same as in utils/random.h

// C functions
#include <cmath>
#include <climits>

#ifdef WINDOWS
#define USE_REGISTRY
#endif

#ifdef WINDOWS
#include "utils/windows.h"
#endif


#ifndef MS_VISUAL_C
#include <dirent.h>
#endif


#include "utils/version.h"


// Whether GTKMM can be used.
// When not defined, FreeDoko should compile and run without any GTKMM library
// this makro shall be defined through the compiler (in the makefile)
//#define GTKMM


// code that is outdated -- leave this macro undefined
#undef OUTDATED
// code, that is postponed -- leave this macro undefined
#undef POSTPONED

// code that is deprecated -- leave this macro defined
#define DEPRECATED

// code that is workaround -- leave this macro defined
#define WORKAROUND

// code, that is an alternative -- leave this macro undefined
#undef ALTERNATIVE


extern char const* const GPL_string;
extern Version const* version;
extern std::vector<Version const*> all_versions;
#define CONTACT "freedoko@users.sourceforge.net"
#define PROGRAM_name "FreeDoko"


// In the release-version there are some elements hidden
//#define RELEASE
//#define VERSION_DESCRIPTION "preversion"

#ifdef RELEASE
#define NO_CRITICAL_CHECKS
#endif


// whether to check the runtime
#if !defined(RELEASE) && !defined(USE_THREADS)
//#define CHECK_RUNTIME
#endif
#ifdef CHECK_RUNTIME
#include "runtime.h"
#endif


// some utils (RAND(int), MARK(), expand_filename(string), Null_Str
#include "utils.h"
#include "misc/translation.h"


#ifndef WINDOWS
// PlaySound exists only under MS Windows
#undef USE_SOUND_PLAYSOUND
#endif


// whether an assertion shall create a segmentation fault (for a core dump)
// (define it in 'Makefile.local')
//#define ASSERTION_GENERATES_SEGFAULT

// debugging utils
#include "debug.h"

// creates a segmentation fault
#define SEGFAULT if (true) { cerr << "Created segmentation fault:\n" \
  << __FILE__ << " # " << __LINE__ << endl; \
  abort(); } else (void)0
//(*reinterpret_cast<volatile int*>(NULL) = 0); } else (void)0

#endif // #ifndef CONSTANTS_HEADER
