/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../basetypes.h"
#include "specialpoint.h"
#include "exception.h"
#include "game.h"
class Rule;
class Player;

/**
 ** The summary of a game:
 ** - seed
 ** - startplayer
 ** - gametype
 ** - gamepoints
 **/
class GameSummary {
  public:
    GameSummary() = delete;
    explicit GameSummary(Game const& game);
    GameSummary(Rule const& rule, istream& istr);
    GameSummary(GameSummary const& game_summary) = default;
    GameSummary& operator=(GameSummary const&) = default;
    ~GameSummary() = default;

    void write(ostream& ostr) const;
    void read(istream& istr);

    Rule const& rule() const;
    Seed seed() const;
    unsigned players_number() const;
    unsigned party_player(unsigned p) const;
    unsigned game_player(Player const& player) const;
    bool has_played(Player const& player) const;

    unsigned startplayer() const;
    unsigned soloplayer() const;
    GameType game_type() const;
    unsigned swines_player() const;
    unsigned hyperswines_player() const;
    Announcement highest_announcement_re() const;
    Announcement highest_announcement_contra() const;
    Team winnerteam() const;

    bool is_solo() const;
    bool is_duty_solo() const;
    bool is_lust_solo() const;
    bool startplayer_stays() const;
    unsigned next_startplayer() const;

    unsigned trick_points(Player const& player) const;
    unsigned trick_points(Team const team) const;

    Team team_of_points() const;
    bool get_points(Team team) const;
    int points() const;
    int points(Team team) const;

    bool get_points(Player const& player) const;
    int points(Player const& player) const;

    Team team(Player const& player) const;

    Specialpoints const& specialpoints() const;
    Specialpoint const& specialpoint(unsigned i) const;

    int bock_multiplier() const;
    vector<BockTrigger> const& bock_triggers() const;

  private:
    void evaluate(Game const& game);

  private:
    Rule const* rule_;
    unsigned seed_ = UINT_MAX;
    unsigned players_number_ = 0;
    vector<unsigned> party_players_;
    unsigned startplayer_ = UINT_MAX;
    unsigned soloplayer_ = UINT_MAX;

    GameType game_type_ = GameType::normal;
    bool duty_solo_ = false;
    unsigned swines_player_ = UINT_MAX;
    unsigned hyperswines_player_ = UINT_MAX;
    vector<Team> teams_;

    Announcement highest_announcement_re_ = Announcement::noannouncement;
    Announcement highest_announcement_contra_ = Announcement::noannouncement;

    vector<unsigned> trick_points_;
    Team winnerteam_ = Team::noteam;
    int points_ = 0;

    Specialpoints specialpoints_;
    vector<BockTrigger> bock_triggers_;

  private: // unused
}; // class GameSummary

// write the game summary into the output stream
ostream& operator<<(ostream& ostr, GameSummary const& game_summary);
