/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once


namespace GameplayActions {

enum class Type {
  // cards_distributed,
  reservation,
  poverty_shift,
  poverty_accepted,
  poverty_returned,
  poverty_denied,
  poverty_denied_by_all,
  card_played,
  announcement,
  swines,
  hyperswines,
  marriage,
  genscher,
  trick_open,
  trick_full,
  trick_taken,
  check,
  print
}; // enum class Type
enum class Discrepancy {
  future = -1,
  none = 0,
  skipped = 1,
  player,
  card,
  other
}; // enum class Discrepancy


/** base class for gameplay actions
 **/
class GameplayAction {
  public:
    static unique_ptr<GameplayAction> create(string const& line);
    static unique_ptr<GameplayAction> create(string const& line,
                                             istream& istr);
  protected:
    GameplayAction() = default;

  public:
    virtual ~GameplayAction() = default;

    unique_ptr<GameplayAction> clone() const;

    virtual bool equal(GameplayAction const& action) const;

    virtual void write(ostream& ostr) const = 0;

    virtual Type type() const = 0;

    virtual string data_translation() const = 0;
}; // class GameplayAction
unsigned player(GameplayAction const& action);

}; // namespace GameplayActions

using GameplayAction = GameplayActions::GameplayAction;

string to_string(GameplayActions::Type type);
string gettext(GameplayActions::Type type);
ostream& operator<<(ostream& ostr, GameplayActions::Type type);
string to_string(GameplayActions::Discrepancy discrepancy);
ostream& operator<<(ostream& ostr, GameplayActions::Discrepancy discrepancy);

bool operator!(GameplayActions::Discrepancy discrepancy);
bool operator||(GameplayActions::Discrepancy lhs, bool rhs);

string to_string(GameplayAction const& action);
ostream& operator<<(ostream& ostr, GameplayAction const& action);

bool operator==(GameplayAction const& lhs, GameplayAction const& rhs);
bool operator!=(GameplayAction const& lhs, GameplayAction const& rhs);
