/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#include "announcement.h"

#include "../../utils/string.h"

namespace GameplayActions {

/** constructor
 **
 ** @param    playerno   player who has made the announcement
 ** @param    announcement   announcement
 **/
Announcement::Announcement(unsigned const playerno,
                           ::Announcement const& announcement) :
  BasePlayer(playerno),
  announcement(announcement)
{ }

/** constructor
 **
 ** @param    line   line with the data
 **/
Announcement::Announcement(string line) :
  BasePlayer()
{
#ifndef OUTDATED
  // old format in the bug report:
  //   Announcement: player 1: no 120
  if (string(line, 0, strlen("Announcement: ")) == "Announcement: ") {
    // there is an announcement, look, which player has made it
    String::word_first_remove_with_blanks(line);
    String::word_first_remove_with_blanks(line);
    this->player = static_cast<unsigned>(atoi(line.c_str()));

    // read the announcement
    String::word_first_remove_with_blanks(line);
    this->announcement = announcement_from_string(line);

    return ;
  }
#endif

  // ex: announcement = 0, no 90
  auto const pre = to_string(this->type()) + " =";
  if (string(line, 0, pre.length()) != pre)
    return ;

  istringstream istr(string(line, pre.length()));
  istr >> this->player;
  if (istr.get() != ',')
    return ;
  if (istr.get() != ' ')
    return ;
  string name;
  getline(istr, name);
  this->announcement = announcement_from_string(name);
} // Announcement(string line)

/** -> result
 **
 ** @param    action   action to compare with
 **
 ** @return   whether the action is equal to this
 **/
bool
Announcement::equal(GameplayAction const& action) const
{
  return (   (this->BasePlayer::equal(action))
          && (this->announcement == dynamic_cast<Announcement const&>(action).announcement));
} // bool Announcement::equal(GameplayAction action) const

/** writes the action in the output stream
 **
 ** @param    ostr   output stream
 **/
void
Announcement::write(ostream& ostr) const
{
  ostr << this->type();
  ostr << " = " << this->player;
  ostr << ", " << this->announcement;
} // void write(ostream& ostr) const

/** @return   a translation representing the data
 **/
string
Announcement::data_translation() const
{
  return (std::to_string(this->player)
          + ": "
          + _(this->announcement));
} // virtual string Announcement::data_translation() const

} // namespace GameplayActions
