/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001-2016 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#pragma once

#include "constants.h"

#include "base_cards.h"

#include "../../utils/string.h"

namespace GameplayActions {

/**
 ** constructor
 **
 ** @param    player   player who has initiated the action
 ** @param    cards    cards
 **
 ** @return   -
 **/
template<Type type>
  BaseCards<type>::BaseCards(unsigned const player,
			     vector<Card> cards) :
    BasePlayer<type>(player),
    cards(std::move(cards))
{ }

/**
 ** constructor
 **
 ** @param    player   player who has initiated the action
 ** @param    cards    cards
 **
 ** @return   -
 **/
template<Type type>
BaseCards<type>::BaseCards(Player const& player,
			   vector<Card> cards) :
  BaseCards(player, std::move(cards))
{ }

/**
 ** constructor
 **
 ** @param    line   line with the data
 **
 ** @return   -
 **/
template<Type type>
BaseCards<type>::BaseCards(string const& line) :
  BaseCards<type>()
{
  auto const pre = to_string(this->type()) + " =";
  if (string(line, 0, pre.length()) != pre)
    return ;

  // first split the line in the single parts
  auto const parts(String::split(string(line, pre.length()),
					    ','));
  auto p = parts.begin();

  if (p == parts.end()) {
    this->player = UINT_MAX;
    return ;
  }
  { // read the player
    if (*p == "-") {
      this->player = UINT_MAX;
      return ;
    }

    istringstream istr(*p);
    istr >> this->player;
    ++p;
  } // read the player

  { // read the cards
    for (; p != parts.end(); ++p)
      this->cards.push_back(Card(*p));
  } // read the cards
} // BaseCards::BaseCards(string line)

/**
 ** -> result
 **
 ** @param    action   action to compare with
 **
 ** @return   -
 **/
template<Type type>
bool
BaseCards<type>::equal(GameplayAction const& action) const
{
  if (!this->BasePlayer<type>::equal(action))
    return false;

  auto const& rhs = static_cast<BaseCards<type> const&>(action);
  if (this->cards.size() != rhs.cards.size())
    return false;

  for (size_t i = 0; i < this->cards.size(); ++i)
    if (this->cards[i] != rhs.cards[i])
      return false;

  return true;
} // bool BaseCards<type>::equal(GameplayAction action) const

/**
 ** writes the action in the output stream
 **
 ** @param    ostr   output stream
 **
 ** @return   -
 **/
template<Type type>
void
BaseCards<type>::write(ostream& ostr) const
{
  ostr << this->type();
  ostr << " = " << this->player;
  for (auto const& c : this->cards)
    ostr << ", " << c;
} // void write(ostream& ostr) const

/**
 ** -> result
 **
 ** @param    -
 **
 ** @return   a translation representing the data (without the player number)
 **/
template<Type type>
string
BaseCards<type>::data_translation() const
{
  auto c = this->cards.begin();
  if (c == this->cards.end())
    return {};

  auto translation = gettext(*c);
  for (++c; c != this->cards.end(); ++c)
    translation += ", " + gettext(*c);

  return translation;
} // virtual string BaseCards::data_translation() const

} // namespace GameplayActions
