/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001-2016 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#include "base_player.h"

#include "../../player/player.h"
#include "../../utils/string.h"

namespace GameplayActions {

/**
 ** constructor
 **
 ** @param    player   player who has initiated the action
 **
 ** @return   -
 **/
template<Type type>
BasePlayer<type>::BasePlayer(unsigned const player) :
  Base<type>(),
  player(player)
{ }

/**
 ** constructor
 **
 ** @param    player   player who has initiated the action
 **
 ** @return   -
 **/
template<Type type>
BasePlayer<type>::BasePlayer(Player const& player) :
  BasePlayer(player.no())
{ }

/**
 ** constructor
 **
 ** @param    line   line with the data
 **
 ** @return   -
 **/
template<Type type>
BasePlayer<type>::BasePlayer(string const& line) :
  Base<type>()
{
  // ex: swines = 0
  string const pre = to_string(this->type()) + " =";
  if (string(line, 0, pre.length()) != pre)
    return ;

  istringstream istr(string(line, pre.length()));
  istr >> this->player;
} // BasePlayer::BasePlayer(string line)

/**
 ** -> result
 **
 ** @param    action   action to compare with
 **
 ** @return   -
 **/
template<Type type>
bool
BasePlayer<type>::equal(GameplayAction const& action) const
{
  return (   this->Base<type>::equal(action)
	  && (this->player == static_cast<BasePlayer<type> const&>(action).player));
} // bool BasePlayer<type>::equal(GameplayAction action) const

/**
 ** writes the action in the output stream
 **
 ** @param    ostr   output stream
 **
 ** @return   -
 **/
template<Type type>
void
BasePlayer<type>::write(ostream& ostr) const
{
  ostr << this->type();
  ostr << " = " << this->player;
} // void write(ostream& ostr) const

/**
 ** -> result
 **
 ** @param    -
 **
 ** @return   a translation representing the data (without the player number)
 **/
template<Type type>
string
BasePlayer<type>::data_translation() const
{
  return {};
} // virtual string BasePlayer::data_translation() const

} // namespace GameplayActions
