/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../gameplay_action.h"

/**
 ** base class for gameplay action of one player
 ** t.i. there is one player who has initiated the action
 ** example: Swines
 **/
struct GameplayAction::Player1 : public GameplayAction {
  Player1() = delete;
  Player1(Type type, unsigned playerno);
  Player1(Type type, string const& line);

  Player1(Player1 const& action) = default;
  Player1& operator=(Player1 const& action) = default;

  virtual ~Player1() = default override;

  virtual void write(ostream& ostr) const override;

  virtual string data_translation() const override;
}; // class GameplayAction::Player1 : public GameplayAction

// macro for simple child classes construction
//
// @param   ActionClassName   name for the class as subclass of 'GameplayAction'
// @param   ClassName         name for the class
// @param   type              type of the action
//
// example: gamePLAY_ACTION_player1(GameplayAction::PovertyDeniedByAll, PovertyDeniedByAll, poverty_denied_BY_ALL)
//
// @bug     'GameplayAction:: ## ClassName' does not work
#define gamePLAY_ACTION_player1(ActionClassName, ClassName, type) \
  struct ActionClassName : public GameplayAction::Player1 { \
    explicit ClassName(unsigned const playerno) : Player1(type, playerno) { } \
    explicit ClassName(string const& line)      : Player1(type, line)     { } \
    ClassName(ClassName const& action) = default; \
    ClassName& operator=(ClassName const& action) = default; \
    ~ClassName() = default; \
  };
