/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#include "reservation.h"

#include "../../utils/string.h"

namespace GameplayActions {

/** constructor
 **
 ** @param    player         player
 ** @param    reservation    reservation of the player
 **/
Reservation::Reservation(unsigned const player,
                         ::Reservation const& reservation) :
  BasePlayer(player),
  reservation(reservation)
{ }

/** constructor
 **
 ** @param    line   line with the data
 **/
Reservation::Reservation(string line) :
  BasePlayer()
{
  // ex: reservation = 0, marriage, first color, hyperswines
  string const pre = to_string(this->type()) + " =";
  if (string(line, 0, pre.length()) != pre)
    return ;

  istringstream istr(string(line, pre.length()));
  istr >> this->player;
  if (istr.get() != ',')
    return ;
  string reservation_text;
  getline(istr, reservation_text);
  this->reservation = ::Reservation(reservation_text);
} // Reservation(string line)

/** -> result
 **
 ** @param    action   action to compare with
 **
 ** @return   whether the action is equal to this
 **/
bool
Reservation::equal(GameplayAction const& action) const
{
  return (   this->BasePlayer::equal(action)
	  && (this->reservation == dynamic_cast<Reservation const&>(action).reservation));
} // bool Reservation::equal(GameplayAction action) const

/** writes the action in the output stream
 **
 ** @param    ostr   output stream
 **/
void
Reservation::write(ostream& ostr) const
{
  ostr << this->type();
  ostr << " = " << this->player;
  ostr << ", " << this->reservation.game_type;
  if (this->reservation.game_type == GameType::marriage)
    ostr << ", " << this->reservation.marriage_selector;
  if (this->reservation.swines)
    ostr << ", swines";
  if (this->reservation.hyperswines)
    ostr << ", hyperswines";
} // void write(ostream& ostr) const

/** @return   a translation representing the data
 **/
string
Reservation::data_translation() const
{
  string translation;

  translation += std::to_string(this->player);

  translation += ", " + _(this->reservation.game_type);

  if (this->reservation.game_type == GameType::marriage)
    translation += ", " + _(this->reservation.marriage_selector);

  if (this->reservation.swines)
    translation += ", " + _("swines");
  if (this->reservation.hyperswines)
    translation += ", " + _("hyperswines");

  return translation;
} // string Reservation::data_translation() const

} // namespace GameplayActions
