/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

class Game;
class Player;

/**
 ** The marriage part of a game
 **/
class GameMarriage {
  friend class Game;
  public:
  explicit GameMarriage(Game& game) noexcept;
  GameMarriage(GameMarriage const& marriage, Game& game) noexcept;

  GameMarriage(GameMarriage const&) = delete;
  GameMarriage& operator=(GameMarriage const&) = delete;

  void reset() noexcept;

  bool is_undetermined() const noexcept;
  MarriageSelector selector() const noexcept;
  unsigned determination_trickno() const noexcept;

  void check_for_genscher(HandCard const& played_card);
  void check_for_silent_marriage();
  void determine_silent_marriage();
  void determine_marriage();

  Signal<void(Player const&, Player const&)>& signal_marriage() noexcept;
  Signal<void()>& signal_get_genscher_partner() noexcept;
  Signal<void(Player const&, Player const&)>& signal_genscher() noexcept;

  private:
  Game* const game_;
  MarriageSelector selector_ = MarriageSelector::team_set;
  unsigned determination_trickno_ = UINT_MAX;

  Signal<void(Player const&, Player const&)> signal_marriage_;
  Signal<void()> signal_get_genscher_partner_;
  Signal<void(Player const&, Player const&)> signal_genscher_;

}; // class GameMarriage
