/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#include "constants.h"
#include "players.h"

/** constructor
 **
 ** @param    players   corresponding container
 ** @param    p         position in the container
 **/
GamePlayersIterator::GamePlayersIterator(GamePlayers& players,
                                         unsigned const p) noexcept :
  players_(&players),
  p_(p)
{ }

/** @return   player, the iterator is pointing at
 **/
GamePlayer&
GamePlayersIterator::operator*() noexcept
{
  return this->players_->player(this->p_);
}

GamePlayer const&
GamePlayersIterator::operator*() const noexcept
{
  return this->players_->player(this->p_);
}

/** @return   pointer to the player, the iterator is pointing at
 **/
GamePlayer*
GamePlayersIterator::operator->() noexcept
{
  return &(**this);
}

/** @return   pointer to the player, the iterator is pointing at
 **/
GamePlayer const*
GamePlayersIterator::operator->() const noexcept
{
  return &(**this);
}

/** increase the iterator
 **
 ** @return   increased iterator
 **/
GamePlayersIterator&
GamePlayersIterator::operator++() noexcept
{
  this->p_ += 1;
  return *this;
}

/** compare the iterators
 **
 ** @param    i   iterator to compare with
 **
 ** @return   whethe the iterators differ
 **/
bool
GamePlayersIterator::operator!=(GamePlayersIterator const& i) const noexcept
{
  return (   (this->p_ != i.p_)
          || (this->players_ != i.players_));
}

/** constructor
 **
 ** @param    players   corresponding container
 ** @param    p         position in the container
 **/
GamePlayersConstIterator::GamePlayersConstIterator(GamePlayers const& players,
                                                   unsigned const p) noexcept :
  players_(&players),
  p_(p)
{ }

/** @return   player, the iterator is pointing at
 **/
GamePlayer const&
GamePlayersConstIterator::operator*() const noexcept
{
  return this->players_->player(this->p_);
}

/** @return   pointer to the player, the iterator is pointing at
 **/
GamePlayer const*
GamePlayersConstIterator::operator->() const noexcept
{
  return &(**this);
}

/** increase the iterator
 **
 ** @return   increased iterator
 **/
GamePlayersConstIterator&
GamePlayersConstIterator::operator++() noexcept
{
  this->p_ += 1;
  return *this;
}

/** compare the iterators
 **
 ** @param    i   iterator to compare with
 **
 ** @return   whethe the iterators differ
 **/
bool
GamePlayersConstIterator::operator!=(GamePlayersConstIterator const& i) const noexcept
{
  return (   (this->p_ != i.p_)
          || (this->players_ != i.players_));
}
