/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../card/hand_cards.h"
class Game;
class Player;

/**
 ** The swines part of a game
 **/
class GameSwines {
  friend class Game;
  friend class GamePoverty;
  public:
  explicit GameSwines(Game& game) noexcept;
  GameSwines(GameSwines const& poverty, Game& game);

  GameSwines() = delete;
  GameSwines(GameSwines const&) = delete;
  GameSwines& operator=(GameSwines const&) = delete;

  void self_check() const;

  void reset() noexcept;

  bool first_fox_caught() const noexcept;
  Player const& first_fox_catcher() const noexcept;
  bool swines_announced() const noexcept;
  Player const& swines_owner() const noexcept;
  bool swines_announcement_valid(Player const& player) const;
  bool swines_announce(Player& player);
  bool hyperswines_announced() const noexcept;
  Player const& hyperswines_owner() const noexcept;
  bool hyperswines_announcement_valid(Player const& player) const;
  bool hyperswines_announce(Player& player);

  void test_swines_from_reservations();
  void evaluate_trick(Trick const& trick);

  Signal<void(Player const&)>& signal_swines_announced() noexcept;
  Signal<void(Player const&)>& signal_hyperswines_announced() noexcept;

  private:
  Game* const game_;

  Player* swines_owner_ = {};
  Player const* first_fox_catcher_ = {};
  bool swines_announced_ = false;

  Player* hyperswines_owner_ = {};
  bool hyperswines_announced_ = false;

  Signal<void(Player const&)> signal_swines_announced_;
  Signal<void(Player const&)> signal_hyperswines_announced_;
}; // class GameSwines
