/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../basetypes/team.h"

class Game;
class Player;

/**
 ** The teaminfo of a game
 **/
class GameTeamInfo {
  friend class Game;
  public:
  explicit GameTeamInfo(Game& game) noexcept;
  GameTeamInfo(GameTeamInfo const& teaminfo, Game& game) noexcept;

  GameTeamInfo(GameTeamInfo const&) = delete;
  GameTeamInfo& operator=(GameTeamInfo const&) = delete;

  void reset();
  void set_at_gamestart();
  void update();

  Team get(Player const& player) const;
  vector<Team> const& get_all() const noexcept;
  Team get_human_teaminfo(Player const& player) const;
  void set(Player const& player, Team team);
  void set_unknown(Team team) noexcept;

  Signal<void(Player const&)>& signal_changed() noexcept;

  void write(ostream& ostr) const;

  private:
  Game* const game_;
  vector<Team> teaminfo_;
  bool all_known_ = false;

  Signal<void(Player const&)> signal_changed_;
}; // class GameTeamInfo
