/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#include "constants.h"
#include "tricks.h"

/** constructor
 **
 ** @param    tricks   corresponding container
 ** @param    p         position in the container
 **/
GameTricksIterator::GameTricksIterator(GameTricks& tricks,
                                       size_t const p) noexcept :
  tricks_(&tricks),
  p_(p)
{ }

/** @return   trick, the iterator is pointing at
 **/
Trick&
GameTricksIterator::operator*()
{
  return this->tricks_->trick(this->p_);
}

Trick const&
GameTricksIterator::operator*() const
{
  return this->tricks_->trick(this->p_);
}

/** @return   pointer to the trick, the iterator is pointing at
 **/
Trick*
GameTricksIterator::operator->()
{
  return &(**this);
}

/** @return   pointer to the trick, the iterator is pointing at
 **/
Trick const*
GameTricksIterator::operator->() const
{
  return &(**this);
}

/** increase the iterator
 **
 ** @return   increased iterator
 **/
GameTricksIterator&
GameTricksIterator::operator++() noexcept
{
  this->p_ += 1;
  return *this;
}

/** compare the iterators
 **
 ** @param    i   iterator to compare with
 **
 ** @return   whethe the iterators differ
 **/
bool
GameTricksIterator::operator!=(GameTricksIterator const& i) const noexcept
{
  return (   (this->p_ != i.p_)
          || (this->tricks_ != i.tricks_));
}

/** constructor
 **
 ** @param    tricks   corresponding container
 ** @param    p         position in the container
 **/
GameTricksConstIterator::GameTricksConstIterator(GameTricks const& tricks,
                                                 size_t const p) noexcept :
  tricks_(&tricks),
  p_(p)
{ }

/** @return   trick, the iterator is pointing at
 **/
Trick const&
GameTricksConstIterator::operator*() const
{
  return this->tricks_->trick(this->p_);
}

/** @return   pointer to the trick, the iterator is pointing at
 **/
Trick const*
GameTricksConstIterator::operator->() const
{
  return &(**this);
}

/** increase the iterator
 **
 ** @return   increased iterator
 **/
GameTricksConstIterator&
GameTricksConstIterator::operator++() noexcept
{
  this->p_ += 1;
  return *this;
}

/** compare the iterators
 **
 ** @param    i   iterator to compare with
 **
 ** @return   whethe the iterators differ
 **/
bool
GameTricksConstIterator::operator!=(GameTricksConstIterator const& i) const noexcept
{
  return (   (this->p_ != i.p_)
          || (this->tricks_ != i.tricks_));
}
