/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#pragma once

#include "../basetypes.h"

#include "../card/card.h"

#include "setting.h"

class Setting::CardsOrder {
  public:
    enum class Direction {
      none, up, down
    };

  public:
    explicit CardsOrder(Setting& setting);
    CardsOrder(Setting& setting,
               Card::TColor c1, Direction d1,
               Card::TColor c2, Direction d2,
               Card::TColor c3, Direction d3,
               Card::TColor c4, Direction d4,
               Card::TColor c5, Direction d5);
    CardsOrder(Setting& setting, string const& s);
    CardsOrder(CardsOrder const& cards_order);
    CardsOrder& operator=(CardsOrder const& cards_order);

    bool ensure_validity();

    operator bool() const;
    operator string() const;

    bool operator==(CardsOrder const& rhs) const;

    bool sorted() const;
    bool mixed() const;

    int relative_position(HandCard const& a, HandCard const& b) const;

    Direction direction(Card::TColor tcolor) const;
    Direction direction(unsigned pos) const;
    unsigned pos(Card::TColor tcolor) const;
    Card::TColor tcolor(unsigned pos) const;

    // toggle / set the direction
    void direction_toggle(Card::TColor tcolor);
    void direction_toggle(unsigned pos);
    void direction_set(Card::TColor tcolor, Direction direction);
    void direction_set(unsigned pos, Direction direction);

    void pos_to_left(unsigned pos);
    void pos_to_right(unsigned pos);

    void sorted_set(bool sorted);
    void mix();

    // write of the cards order
    void write(ostream& ostr) const;

  private:
    Setting* const setting_ = nullptr;

    bool sorted_ = false;

    vector<unsigned> pos_;
    vector<Direction> direction_;
}; // class Setting::CardsOrder
string to_string(Setting::CardsOrder::Direction direction);
string gettext(Setting::CardsOrder::Direction direction);
ostream& operator<<(ostream& ostr, Setting::CardsOrder const& cards_order);
