/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 * 
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "setting.theme.h"

/** constructor
 **
 ** @param    name   name of the theme
 **/
Setting::Theme::Theme(string const name) :
  name_(name)
{ }

/** @return   the name of the theme
 **/
string const&
Setting::Theme::name() const
{
  return this->name_;
} // string const& Setting::Theme::name() const

/** set the bool setting
 **
 ** @param    type    type to set
 ** @param    value   value of 'type'
 **/
void
Setting::Theme::set(Type::Bool const type, bool const value)
{
  this->bool_[type] = value;
} // void Setting::Theme::set(Type::Bool type, bool value)

/** set the unsigned setting
 **
 ** @param    type    type to set
 ** @param    value   value of 'type'
 **/
void
Setting::Theme::set(Type::Unsigned const type, unsigned const value)
{
  this->unsigned_[type] = value;
} // void Setting::Theme::set(Type::Unsigned type, unsigned value)

/** set the string setting
 **
 ** @param    type    type to set
 ** @param    value   value of 'type'
 **/
void
Setting::Theme::set(Type::String type, string const& value)
{
  this->string_[type] = value;
} // void Setting::Theme::set(Type::String type, string value)

/** @return   whether the theme can be applied
 **
 ** @todo      search only the selected cardset
 **/
bool
Setting::Theme::can_be_applied() const
{
  Setting setting_tmp(setting);
  setting_tmp.set(*this);
  for (auto t : Type::string_list) {
    if (this->string_.find(t) == this->string_.end())
      continue;

    switch (t) {
    case Type::cardset:
    case Type::background:
    case Type::language:
    case Type::cards_back:
      if (   setting_tmp.path(static_cast<Type::String>(t)).empty()
          || (setting_tmp.value(static_cast<Type::String>(t))
              != (*this->string_.find(static_cast<Type::String>(t))).second)
         ) {
        return false;
      }
    default:
      break;
    } // switch(t)
  } // for (t \in Setting::TypeString)

  return true;
} // bool Setting::Theme::can_be_applied() const
