/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

#include "../game/gameplay_action.h"

namespace OS_NS {
class BugReport;
} // namespace OS_NS 
class Game;
class GameSummary;

extern unique_ptr<OS_NS::BugReport> bug_report;

namespace OS_NS {

extern string const BUG_REPORT_FILE;

// The default BugReport
class BugReport {
  public:
    // create a bug report
    string report(string const& message = "");
    string report(string const& message, string const& subdir);

  public:
    BugReport();
    BugReport(BugReport const&) = delete;
    BugReport& operator=(BugReport const&) = delete;
    ~BugReport();

    // a gameplay action has happened
    void gameplay_action(GameplayAction const& action);

    // the parts of a game
    void game_open(Game const& game);
    void game_redistribute();
    void new_game();
    void game_start();
    void game_finish();

    string directory() const;
    string filename() const;
    string filename(string directory) const;

  private:
    // clear the bug report data
    void clear();

  private:
    AutoDisconnector disconnector_;

    // the seed of the game
    unsigned seed_ = UINT_MAX;
    // the output stream (with the bug report)
    unique_ptr<ostringstream> ostr_;

    // the gameplay actions
    vector<unique_ptr<GameplayAction>> actions_;
    // the human gameplay actions
    vector<unique_ptr<GameplayAction>> human_actions_;
    // the game summary
    unique_ptr<GameSummary> game_summary_ = nullptr;

    // number of written bug reports
    unsigned written_ = 0;
}; // class BugReport

} // namespace OS_NS
