/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#include "constants.h"
#include "players.h"

/** constructor
 **
 ** @param    players   corresponding container
 ** @param    p         position in the container
 **/
PartyPlayersIterator::PartyPlayersIterator(PartyPlayers& players,
                                           unsigned p) :
  players_(&players),
  p_(p)
{ }

/** @return   player, the iterator is pointing at
 **/
PartyPlayer&
PartyPlayersIterator::operator*()
{
  return this->players_->player(this->p_);
}

PartyPlayer const&
PartyPlayersIterator::operator*() const
{
  return this->players_->player(this->p_);
}

/** @return   pointer to the player, the iterator is pointing at
 **/
PartyPlayer*
PartyPlayersIterator::operator->()
{
  return &(**this);
}

/** @return   pointer to the player, the iterator is pointing at
 **/
PartyPlayer const*
PartyPlayersIterator::operator->() const
{
  return &(**this);
}

/** increase the iterator
 **
 ** @return   increased iterator
 **/
PartyPlayersIterator&
PartyPlayersIterator::operator++()
{
  this->p_ += 1;
  return *this;
}

/** compare the iterators
 **
 ** @param    i   iterator to compare with
 **
 ** @return   whethe the iterators differ
 **/
bool
PartyPlayersIterator::operator!=(PartyPlayersIterator const& i) const
{
  return (   (this->p_ != i.p_)
          || (this->players_ != i.players_));
}

/** constructor
 **
 ** @param    players   corresponding container
 ** @param    p         position in the container
 **/
PartyPlayersConstIterator::PartyPlayersConstIterator(PartyPlayers const& players,
                                                     unsigned p) :
  players_(&players),
  p_(p)
{ }

PartyPlayer const&
PartyPlayersConstIterator::operator*() const
{
  return this->players_->player(this->p_);
}

/** @return   pointer to the player, the iterator is pointing at
 **/
PartyPlayer const*
PartyPlayersConstIterator::operator->() const
{
  return &(**this);
}

/** increase the iterator
 **
 ** @return   increased iterator
 **/
PartyPlayersConstIterator&
PartyPlayersConstIterator::operator++()
{
  this->p_ += 1;
  return *this;
}

/** compare the iterators
 **
 ** @param    i   iterator to compare with
 **
 ** @return   whethe the iterators differ
 **/
bool
PartyPlayersConstIterator::operator!=(PartyPlayersConstIterator const& i) const
{
  return (   (this->p_ != i.p_)
          || (this->players_ != i.players_));
}
