/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#pragma once

#include "../playersDb.h"
#include "../aiconfig.h"

class AiDb : public PlayersDb {
  public:
    AiDb();
    explicit AiDb(istream& istr);
    explicit AiDb(PlayersDb const& players_db);
    AiDb(AiDb const& aidb);
    ~AiDb() override;

    ostream& write(ostream& ostr) const override;
  private:
    bool read_group(string const& name, istream& istr) override;
    void read_group_heuristics(istream& istr);

  public:
    unsigned heuristics_group_general() const;
    unsigned heuristics_group_poverty() const;
    unsigned heuristics_group_solo() const;
    unsigned heuristics_group_solo_color() const;
    unsigned heuristics_group_solo_picture() const;
    unsigned heuristics_group_solo_meatless() const;

    /// how often each statistic was used
    unsigned heuristic(Aiconfig::Heuristic heuristic) const;

    void increase(Aiconfig::Heuristic heuristic);

    void clear() override;

  private:
    mutable std::map<Aiconfig::Heuristic, unsigned> heuristics_;
}; // class AiDb : public PlayersDb
