/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#pragma once

#include "ai.h"
#include "heuristic_interface.h"

class HeuristicAi : public HeuristicInterface {
  public:	
    HeuristicAi() = delete;
    HeuristicAi( Ai const& a ): HeuristicInterface(), ai_(&a ) { };
    HeuristicAi(HeuristicAi const&) = delete;
    HeuristicAi& operator=(HeuristicAi const&) = delete;
    ~HeuristicAi() override = default;

    CardsInformation const& cards_information() const override
    { return this->ai_->cards_information(); }
    TeamInformation const& team_information() const override
    { return this->ai_->team_information(); }

 
    Player const& player() const override
    { return *this->ai_; }
    Hand const& handofplayer( Player const& player ) const override
    { return this->ai_->handofplayer( player ); }
    Team teamofplayer( Player const& player ) const override
    { return this->ai_->teamofplayer( player ); }

    Hand const& hand() const override
    { return this->ai_->hand(); }
    Team team() const override
    { return this->ai_->team(); }
    unsigned no() const
    { return this->ai_->no(); }
    Game const& game() const override
    { return this->ai_->game(); }
    unsigned last_trick_to_calculate() const
    { return this->ai_->last_trick_to_calculate(); }

    bool value(Aiconfig::Type::Bool type) const override
    { return this->ai_->value( type ); }
    int value(Aiconfig::Type::Int type) const override
    { return this->ai_->value( type ); }
    Card value(Aiconfig::Type::Card type) const override
    { return this->ai_->value( type ); }

    bool colorjabbed(Card::TColor c) const override
    { return this->ai_->colorjabbed( c ); }

    unsigned color_runs(Card::TColor c) const override
    { return this->ai_->color_runs( c ); }

    bool jabbedbyownteam(Card::TColor c) const override
    { return this->ai_->jabbedbyownteam( c ); }

    bool jabbedbyotherteam(Card::TColor c) const override
    { return this->ai_->jabbedbyotherteam( c ); }


    bool next_low() const override
    { return this->ai_->next_low() ; }

    Card trump_card_limit() const override
    { return this->ai_->trump_card_limit(); }

    Card lowest_trump_card_limit() const override
    { return this->ai_->lowest_trump_card_limit(); }

  private:
    Ai const* ai_;	
}; // class HeuristicAi
