/**********************************************************************
 *
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "jab_color_over_fox.h"

#include "../../../card/trick.h"
#include "../../../game/game.h"

#include "../ai.h"
#include "../heuristics.h"

/** constructor
 **/
Heuristic::JabColorOverFox::JabColorOverFox(Ai const& ai) :
  Heuristic(ai, Aiconfig::Heuristic::jab_color_over_fox)
{ }

/** destructor
 **/
Heuristic::JabColorOverFox::~JabColorOverFox() = default;

/** @return  card to play or empty card
 **/
Card
Heuristic::JabColorOverFox::get_card(Trick const& trick)
{
  this->rationale_.clear();
  if (!(   !this->condition_solo_meatless(trick)
        && !this->condition_solo_picture(trick)
        && !this->condition_startcard(trick)
        && !this->condition_last_card(trick)
        && this->condition_color_trick(trick)
        && this->condition_can_jab_with_trump(trick)
        && !this->condition_jabbed_with_trump(trick)))
    return {};

  auto const& ai = *this->ai_;
  auto const& hand = ai.hand();
  auto const& game = ai.game();

  auto const trump_ace = Card(game.cards().trumpcolor(), Card::ace);
  auto const card = hand.next_higher_card(trump_ace);
  // check to play a club queen
  if (   game.type() == GameType::normal
      // ToDo: check the own gameplay for team informations
      && game.teaminfo().get(this->player()) == Team::unknown
      && ai.team() == Team::re
      && hand.contains(Card::club_queen)
      && card.value() == Card::queen) {
    if (card.color() == Card::spade) {
      return Card::club_queen;
    }
    if (!Heuristics::checkAllMyTeam(trick, ai)) {
      return Card::club_queen;
    }
  }
  return card;
}

