/**********************************************************************
 *
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "play_bad_color.h"

#include "../team_information.h"
#include "../../../card/trick.h"
#include "../../../game/game.h"
#include "../../../party/rule.h"

#include "../ai.h"
#include "../heuristics.h"

/** constructor
 **/
Heuristic::PlayBadColor::PlayBadColor(Ai const& ai) :
  Heuristic(ai, Aiconfig::Heuristic::play_bad_color)
{ }

/** destructor
 **/
Heuristic::PlayBadColor::~PlayBadColor() = default;

/** @return  card to play or empty card
 **/
Card
Heuristic::PlayBadColor::get_card(Trick const& trick)
{
  this->rationale_.clear();
  if (!(   this->condition_startcard(trick)
        && this->condition_last_player_guessed_same_team(trick)))
    return {};

  auto const& game = trick.game();
  auto const trumpcolor = game.cards().trumpcolor();
  auto const& hand = this->player().hand();

  Player const& partner = trick.lastplayer();
  auto const& hand_of_partner = this->ai_->handofplayer(partner);

  Card::Color selected_color = Card::nocardcolor;
  for (auto color : game.rule().card_colors()) {
    if (color == trumpcolor)
      continue;
    if (!hand.contains(color))
      continue;

    if (!hand_of_partner.contains(color))
      continue;

    if (selected_color == Card::nocardcolor) {
      this->rationale_.add(_("Heuristic::info::check color %s", _(selected_color)));
      selected_color = color;
    } else if (hand_of_partner.count(color)
               < hand_of_partner.count(selected_color)) {
      this->rationale_.add(_("Heuristic::info::prefer color %s, the partner has less cards", _(selected_color)));
      selected_color = color;
    }
  } // for (color : colors)

  if (selected_color == Card::nocardcolor) {
    this->rationale_.add(_("Heuristic::reject::no color found, the partner has, too"));
    return Card::empty;
  }

  auto const card = hand.lowest_card(selected_color);
  this->rationale_.add(_("Heuristic::result::take lowest card of %s: %s", _(selected_color), _(card)));
  return card;
}

