/**********************************************************************
 *
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "play_highest_color.h"

#include "../ai.h"
#include "../heuristics.h"

/** constructor
 **/
Heuristic::PlayHighestColor::PlayHighestColor(Ai const& ai) :
  Heuristic(ai, Aiconfig::Heuristic::play_highest_color_card_in_game)
{ }

/** destructor
 **/
Heuristic::PlayHighestColor::~PlayHighestColor() = default;

/** @return  card to play or empty card
 **/
Card
Heuristic::PlayHighestColor::get_card(Trick const& trick)
{
  this->rationale_.clear();
  if (!(   !this->condition_startcard(trick)
        && this->condition_pure_color_trick(trick)))
    return {};

  auto const& ai = *this->ai_;
  auto const& hand = ai.hand();
  auto const color = trick.startcard().tcolor();
  if (!(   this->condition_has(color)))
    return {};
  auto const card = hand.highest_card(color);
  if (!(   this->condition_can_jab_with(trick, card)))
    return {};

  auto trick2 = trick;
  trick2.add(card);

  if (Heuristics::oppositeTeamCanWinTrick(trick2, ai)) {
    this->rationale_.add(_("Heuristic::condition::the opposite team can jab %s", _(card)));
    return {};
  }
  this->rationale_.add(_("Heuristic::condition::the opposite team cannot jab %s, take it", _(card)));

  return card;
}

