/**********************************************************************
 *
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "save_dolle.h"

#include "../ai.h"
#include "../team_information.h"
#include "../heuristics.h"
#include "../../../game/game.h"
#include "../../../party/rule.h"


/** constructor
 **/
Heuristic::SaveDolle::SaveDolle(Ai const& ai) :
  Heuristic(ai, Aiconfig::Heuristic::save_dolle)
{ }

/** destructor
 **/
Heuristic::SaveDolle::~SaveDolle() = default;

/** @return  card to play or empty card
 **/
Card
Heuristic::SaveDolle::get_card(Trick const& trick)
{
  this->rationale_.clear();
  if (!(   !this->condition_startcard(trick)
        && this->condition_has_dolle()
        && this->condition_opposite_swine_or_hyperswine(trick)
        && this->condition_can_jab_with(trick, Card::dolle)
        && !this->condition_can_be_jabbed_by_opposite_team(trick, Card::dolle)
       ))
    return {};

  auto const& game = trick.game();
  auto const& swines = game.swines();
  auto const& cards_information = this->ai_->cards_information();
  auto const& player_following = game.players().following(this->player());
  unsigned n = 0;
  if (swines.swines_announced()) {
    n += cards_information.remaining_others(game.cards().swine());
    if (player_following == swines.swines_owner())
      n += 1;
  }
  if (swines.hyperswines_announced()) {
    n += cards_information.remaining_others(game.cards().hyperswine());
    if (player_following == swines.hyperswines_owner())
      n += 1;
  }
  auto const& hand = this->hand();
  auto const trumpno = hand.count_trumps();
  if (trumpno + hand.count_dolle() + hand.count_trump_aces() - hand.count_swines()
      <= n + 2) {
    this->rationale_.add(_("Heuristic::return::too few non-dolle trumps, jab the trick with a dolle"));
    return Card::dolle;
  }

  if (trumpno + hand.count_dolle() + hand.count_trump_aces() - hand.count_swines()
      <= n + 4) {
    if (trick.points() < 10) {
      this->rationale_.add(_("Heuristic::reject::not enough points in the trick"));
      return {};
    }
    this->rationale_.add(_("Heuristic::return::few non-dolle trumps, 10 points in the trick, jab the trick with a dolle"));
    return Card::dolle;
  }

  this->rationale_.add(_("Heuristic::reject::still enough trumps on the hand"));
  return {};
}
