/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 ********************************************************************/

#include "constants.h"

#include "virtual_games_ai.h"

/** create a virtual game with virtual players (ais!), uses card_informations and team information
 **
 ** @return   the virtual game
 **/
unique_ptr<Game>
VirtualGamesAi::create_virtual_game(vector<unique_ptr<Player>>& players) const
{
  Game const& game = this->game();
  Ai const& real_ai = dynamic_cast<Ai const&>(game.player(this->no()));

  // create the virtual game
  auto virt_game_p = make_unique<Game>(this->game(), players);
  Game& virt_game = *virt_game_p;
  for (unsigned p = 0; p < virt_game.playerno(); ++p) {
    Ai& ai_virtual = dynamic_cast<Ai&>(*players[p]);

    ai_virtual.set_name(game.player(p).name());

    if (   players.size() != this->no()
        || game.isvirtual())  {
      // set the cards information
      ai_virtual.set_cards_information(real_ai.cards_information());
      // set the hand
      ai_virtual.set_hand(this->handofplayer(game.player(p)));
      // set the teaminfo
      ai_virtual.set_teams(real_ai.team_information().guessed_teams());
      // change the future limit
    } // if (ai_virtual.no() != real_ai.no())
#ifndef POSTPONED
    // *** workaround
    ai_virtual.set_future_limit_for_all_tricks(real_ai.future_limit() / 10);
#endif // #ifndef POSTPONED

  } // for (player \in game.players())

  return virt_game_p;
}

/** create virtual ais, uses card_informations and team information
 **
 ** @return   the virtual ais
 **/
vector<unique_ptr<Player>>
VirtualGamesAi::create_virtual_ais() const
{
  Game const& game = this->game();
  Ai const& real_ai = dynamic_cast<Ai const&>(game.player(this->no()));

  vector<unique_ptr<Player>> ai_virt;
  for (unsigned n = 0; n < game.players().size(); ++n) {
    ai_virt.push_back(make_unique<Ai>(real_ai));
  }
  return ai_virt;
}

