/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#include "constants.h"

#include "aiconfig_type.h"

constexpr AiconfigType::Bool AiconfigType::bool_list[];
constexpr AiconfigType::Int AiconfigType::int_list[];
constexpr AiconfigType::Card AiconfigType::card_list[];

/** -> result
 **
 ** @param    difficulty    difficulty
 **
 ** @return   the name of 'difficulty'
 **/
string
to_string(AiconfigDifficulty const difficulty)
{
  switch(difficulty) {
  case AiconfigDifficulty::custom:
    return "custom";
    (void)_("AiConfig::Difficulty::custom");
  case AiconfigDifficulty::novice:
    return "novice";
    (void)_("AiConfig::Difficulty::novice");
  case AiconfigDifficulty::standard_defensive:
    return "standard defensive";
    (void)_("AiConfig::Difficulty::standard defensive");
  case AiconfigDifficulty::standard_offensive:
    return "standard offensive";
    (void)_("AiConfig::Difficulty::standard offensive");
  case AiconfigDifficulty::profi:
    return "profi";
    (void)_("AiConfig::Difficulty::profi");
  case AiconfigDifficulty::profi_aggressive:
    return "profi aggressive";
    (void)_("AiConfig::Difficulty::profi aggressive");
  case AiconfigDifficulty::profi_unfair:
    return "profi unfair";
    (void)_("AiConfig::Difficulty::profi unfair");
  } // switch(difficulty)

  return "";
}

/** @return   the translation of the difficulty
 **/
string
gettext(AiconfigDifficulty const difficulty)
{
  return gettext("AiConfig::Difficulty::" + to_string(difficulty));
}

/** write the difficulty into the output stream
 **/
ostream&
operator<<(ostream& ostr, AiconfigDifficulty const difficulty)
{
  return (ostr << to_string(difficulty));
}

/** -> result
 **
 ** @param    type   bool type
 **
 ** @return   the name of 'type'
 **/
string
to_string(AiconfigType::Bool const type)
{
  switch(type) {
  case AiconfigType::trusting:
    return "trusting";
    (void)_("AiConfig::trusting");
  case AiconfigType::aggressive:
    return "aggressive";
    (void)_("AiConfig::aggressive");
  case AiconfigType::teams_known:
    return "teams known";
    (void)_("AiConfig::teams known");
  case AiconfigType::hands_known:
    return "hands known";
    (void)_("AiConfig::hands known");
  case AiconfigType::estimate_hands:
    return "estimate hands";
    (void)_("AiConfig::estimate hands");
  case AiconfigType::fehlcreationonfirstcard:
    return "first card fehl creation";
    (void)_("AiConfig::first card fehl creation");
  } // switch(type)

  return "";
}

/** -> result
 **
 ** @param    type   unsigned type
 **
 ** @return   the name of 'type'
 **/
string
to_string(AiconfigType::Int const type)
{
  switch(type) {
  case AiconfigType::remembertricks:
    return "remember tricks";
    (void)_("AiConfig::remember tricks");
  case AiconfigType::limit_throw_fehl:
    return "limit throw fehl";
    (void)_("AiConfig::limit throw fehl");
  case AiconfigType::limitqueen:
    return "limit queen";
    (void)_("AiConfig::limit queen");
  case AiconfigType::limitdolle:
    return "limit dolle";
    (void)_("AiConfig::limit dolle");
  case AiconfigType::lastfehlcreation:
    return "last fehlcreation";
    (void)_("AiConfig::last fehlcreation");
  case AiconfigType::last_tricks_without_heuristics:
    return "last tricks without heuristics";
    (void)_("AiConfig::last tricks without heuristics");
  case AiconfigType::first_trick_for_trump_points_optimization:
    return "first trick for trump points optimization";
    (void)_("AiConfig::first trick for trump points optimization");
  case AiconfigType::announcelimit:
    return "announce limit";
    (void)_("AiConfig::announce limit");
  case AiconfigType::announcelimitdec:
    return "announce limit decrement";
    (void)_("AiConfig::announce limit decrement");
  case AiconfigType::announceconfig:
    return "announce config";
    (void)_("AiConfig::announce config");
  case AiconfigType::announcelimitreply:
    return "announce limit reply";
    (void)_("AiConfig::announce limit reply");
  case AiconfigType::announceconfigreply:
    return "announce config reply";
    (void)_("AiConfig::announce config reply");

  case AiconfigType::takepoverty:
    return "take poverty";
    (void)_("AiConfig::take poverty");
  case AiconfigType::singlesolo:
    return "handvalue single solo";
    (void)_("AiConfig::handvalue single solo");
  case AiconfigType::doublesolo:
    return "handvalue double solo";
    (void)_("AiConfig::handvalue double solo");
  case AiconfigType::triplesolo:
    return "handvalue trible solo";
    (void)_("AiConfig::handvalue trible solo");
  case AiconfigType::colorsolo:
    return "handvalue color solo";
    (void)_("AiConfig::handvalue color solo");
  case AiconfigType::meatless:
    return "handvalue meatless";
    (void)_("AiConfig::handvalue meatless");
  } // switch(type)

  return "";
}

/** -> result
 **
 ** @param    type   card type
 **
 ** @return   the name of 'type'
 **/
string
to_string(AiconfigType::Card const type)
{
  switch(type) {
  case AiconfigType::limitthrowing:
    return "limit throwing";
    (void)_("AiConfig::limit throwing");
  case AiconfigType::limithigh:
    return "limit high";
    (void)_("AiConfig::limit high");
  case AiconfigType::trumplimit_solocolor:
    return "trump limit color";
    (void)_("AiConfig::trump limit color");
  case AiconfigType::trumplimit_solojack:
    return "trump limit jack";
    (void)_("AiConfig::trump limit jack");
  case AiconfigType::trumplimit_soloqueen:
    return "trump limit queen";
    (void)_("AiConfig::trump limit queen");
  case AiconfigType::trumplimit_soloking:
    return "trump limit king";
    (void)_("AiConfig::trump limit king");
  case AiconfigType::trumplimit_solojackking:
    return "trump limit jack king";
    (void)_("AiConfig::trump limit jack king");
  case AiconfigType::trumplimit_solojackqueen:
    return "trump limit jack queen";
    (void)_("AiConfig::trump limit jack queen");
  case AiconfigType::trumplimit_soloqueenking:
    return "trump limit queen king";
    (void)_("AiConfig::trump limit queen king");
  case AiconfigType::trumplimit_solokoehler:
    return "trump limit koehler";
    (void)_("AiConfig::trump limit koehler");
  case AiconfigType::trumplimit_meatless:
    return "trump limit meatless";
    (void)_("AiConfig::trump limit meatless");
  case AiconfigType::trumplimit_normal:
    return "trump limit normal";
    (void)_("AiConfig::trump limit normal");
  case AiconfigType::lowest_trumplimit_solocolor:
    return "lowest trump limit color";
    (void)_("AiConfig::lowest trump limit color");
  case AiconfigType::lowest_trumplimit_solojack:
    return "lowest trump limit jack";
    (void)_("AiConfig::lowest trump limit jack");
  case AiconfigType::lowest_trumplimit_soloqueen:
    return "lowest trump limit queen";
    (void)_("AiConfig::lowest trump limit queen");
  case AiconfigType::lowest_trumplimit_soloking:
    return "lowest trump limit king";
    (void)_("AiConfig::lowest trump limit king");
  case AiconfigType::lowest_trumplimit_solojackking:
    return "lowest trump limit jack king";
    (void)_("AiConfig::lowest trump limit jack king");
  case AiconfigType::lowest_trumplimit_solojackqueen:
    return "lowest trump limit jack queen";
    (void)_("AiConfig::lowest trump limit jack queen");
  case AiconfigType::lowest_trumplimit_soloqueenking:
    return "lowest trump limit queen king";
    (void)_("AiConfig::lowest trump limit queen king");
  case AiconfigType::lowest_trumplimit_solokoehler:
    return "lowest trump limit koehler";
    (void)_("AiConfig::lowest trump limit koehler");
  case AiconfigType::lowest_trumplimit_meatless:
    return "lowest trump limit meatless";
    (void)_("AiConfig::lowest trump limit meatless");
  case AiconfigType::lowest_trumplimit_normal:
    return "lowest trump limit normal";
    (void)_("AiConfig::lowest trump limit normal");
  } // switch(type)

  return "";
}

/** @return   the translation of the type
 **/
string
gettext(AiconfigType::Bool const type)
{
  return gettext("AiConfig::" + to_string(type));
}

/** write the type into the output stream
 **/
ostream&
operator<<(ostream& ostr, AiconfigType::Bool const type)
{
  return (ostr << to_string(type));
}

/** @return   the translation of the type
 **/
string
gettext(AiconfigType::Int const type)
{
  return gettext("AiConfig::" + to_string(type));
}

/** write the type into the output stream
 **/
ostream&
operator<<(ostream& ostr, AiconfigType::Int const type)
{
  return (ostr << to_string(type));
}

/** @return   the translation of the type
 **/
string
gettext(AiconfigType::Card const type)
{
  return gettext("AiConfig::" + to_string(type));
}

/** write the type into the output stream
 **/
ostream&
operator<<(ostream& ostr, AiconfigType::Card const type)
{
  return (ostr << to_string(type));
}
