/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

enum class AiconfigDifficulty {
  custom,
  novice,
  standard_defensive,
  standard_offensive,
  profi,
  profi_aggressive,
  profi_unfair
};
static constexpr AiconfigDifficulty aiconfig_difficulty_list[] = {
  AiconfigDifficulty::custom,
  AiconfigDifficulty::novice,
  AiconfigDifficulty::standard_defensive,
  AiconfigDifficulty::standard_offensive,
  AiconfigDifficulty::profi,
  AiconfigDifficulty::profi_aggressive,
  AiconfigDifficulty::profi_unfair
};

struct AiconfigType {
  ENUM_WITH_STATIC_LIST(Bool, bool_list, 0,
                        trusting, // whether the ai is trusting

                        aggressive,

                        teams_known,   // whether all teams are known
                        hands_known,   // whether all hands are known
                        estimate_hands, // whether to estimate the hands of the other players

                        fehlcreationonfirstcard,
                       ); // enum Bool
  static auto constexpr bool_number = sizeof(bool_list) / sizeof(Bool);
  static auto constexpr bool_first = bool_list[0];
  static auto constexpr bool_last = bool_list[bool_number - 1];

  ENUM_WITH_STATIC_LIST(Int, int_list, fehlcreationonfirstcard + 1,
                        remembertricks, // how many tricks are remembered

                        limit_throw_fehl, // maximum points for playing a fehl
                        limitqueen, // mininum points per trick for jabbing with a queen
                        limitdolle, // minimum points per trick for jabbing with dollen or higher

                        lastfehlcreation, // last trick to create a fehl color
                        last_tricks_without_heuristics, // how many tricks at the end of the game no heuristics are used
                        first_trick_for_trump_points_optimization,

                        announcelimit, // points of hand for first announcement 
                        announcelimitdec, // points of hand you can lose between two announcements
                        announceconfig, // Parameter for Playerconfiguration of announcements
                        announcelimitreply, // points of hand for first Reply
                        announceconfigreply, // Decrement for each announcment made, when replying 

                        takepoverty,

                        singlesolo, // minimum cardvalue for a solo with only one picture trump
                        doublesolo, // minimum cardvalue for a solo with two pictures trump
                        triplesolo, // minimum cardvalue for a solo with three pictures trump
                        colorsolo, // minimun cardvalue for playing a colorsolo
                        meatless, // minimun cardvalue for playing a fleischlosen
                        ); // enum Int
  static auto constexpr int_number = sizeof(int_list) / sizeof(Int);
  static auto constexpr int_first = int_list[0];
  static auto constexpr int_last = int_list[int_number - 1];

  ENUM_WITH_STATIC_LIST(Card, card_list, meatless + 1,
                        limitthrowing, // minimum team mate card for not over jabbing

                        trumplimit_solocolor,
                        trumplimit_solojack,
                        trumplimit_soloqueen,
                        trumplimit_soloking,
                        trumplimit_solojackking,
                        trumplimit_solojackqueen,
                        trumplimit_soloqueenking,
                        trumplimit_solokoehler,
                        trumplimit_meatless,
                        trumplimit_normal,

                        lowest_trumplimit_solocolor,
                        lowest_trumplimit_solojack,
                        lowest_trumplimit_soloqueen,
                        lowest_trumplimit_soloking,
                        lowest_trumplimit_solojackking,
                        lowest_trumplimit_solojackqueen,
                        lowest_trumplimit_soloqueenking,
                        lowest_trumplimit_solokoehler,
                        lowest_trumplimit_meatless,
                        lowest_trumplimit_normal,

                        limithigh, // minimum team mate card for throwing a ten or ace into this trick
                        ); // enum Card
  static auto constexpr card_number = sizeof(card_list) / sizeof(Card);
  static auto constexpr card_first = card_list[0];
  static auto constexpr card_last = card_list[card_number - 1];
}; // struct Type


string to_string(AiconfigDifficulty difficulty);
string gettext(AiconfigDifficulty difficulty);
ostream& operator<<(ostream& ostr, AiconfigDifficulty difficulty);

string to_string(AiconfigType::Bool type);
string gettext(AiconfigType::Bool type);
ostream& operator<<(ostream& ostr, AiconfigType::Bool type);

string to_string(AiconfigType::Int type);
string gettext(AiconfigType::Int type);
ostream& operator<<(ostream& ostr, AiconfigType::Int type);

string to_string(AiconfigType::Card type);
string gettext(AiconfigType::Card type);
ostream& operator<<(ostream& ostr, AiconfigType::Card type);
