/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

class Game;
class Player;

namespace HeuristicsMap {
  enum class GameTypeGroup {
    normal,
    poverty,
    marriage_undetermined,
    marriage_silent,
    solo_meatless,
    soli_color,
    soli_single_picture,
    soli_double_picture,
    solo_koehler,
  }; // enum GameTypeGroup
  constexpr GameTypeGroup game_type_group_all[] = {
    GameTypeGroup::normal,
    GameTypeGroup::poverty,
    GameTypeGroup::marriage_undetermined,
    GameTypeGroup::marriage_silent,
    GameTypeGroup::solo_meatless,
    GameTypeGroup::soli_color,
    GameTypeGroup::soli_single_picture,
    GameTypeGroup::soli_double_picture,
    GameTypeGroup::solo_koehler
  };
  GameTypeGroup group(Game const& game);
  GameTypeGroup GameTypeGroup_from_name(string const& name);

  enum class PlayerTypeGroup {
    special,
    re,
    contra,
  }; // enum PlayerTypeGroup
  constexpr PlayerTypeGroup player_type_group_all[] = {
    PlayerTypeGroup::special,
    PlayerTypeGroup::re,
    PlayerTypeGroup::contra,
  };
  PlayerTypeGroup group(Player const& player);
  PlayerTypeGroup PlayerTypeGroup_from_name(string const& name);

  struct Key {
    constexpr Key() = default;
    explicit Key(Player const& player);
    constexpr Key(GameTypeGroup gametype_group,
                  PlayerTypeGroup playertype_group) :
      gametype_group(gametype_group), playertype_group(playertype_group)
    { }

    GameTypeGroup gametype_group = GameTypeGroup::normal;
    PlayerTypeGroup playertype_group = PlayerTypeGroup::re;
  }; // class Key

  bool operator==(Key lhs, Key rhs);
  bool operator!=(Key lhs, Key rhs);
  bool operator<(Key lhs, Key rhs);
} // namespace HeuristicsMap

string name(HeuristicsMap::GameTypeGroup group);
string name(HeuristicsMap::PlayerTypeGroup group);
inline ostream& operator<<(ostream& ostr, HeuristicsMap::GameTypeGroup const g)
{ return (ostr << ::name(g)); }
inline ostream& operator<<(ostream& ostr, HeuristicsMap::PlayerTypeGroup const g)
{ return (ostr << ::name(g)); }

string gettext(HeuristicsMap::GameTypeGroup gametype_group);
string gettext(HeuristicsMap::PlayerTypeGroup playertype_group);
