/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "aiconfig.h"
#include "ui.h"

#include "../../party/party.h"
#include "../../party/rule.h"
#include "../../player/aiconfig.h"
#include "../../misc/setting.h"
#include "../../utils/string.h"

#include "widgets/labelscalespinbutton.h"
#include "widgets/filemenu.h"
#include "widgets/label_card_selector.h"
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/label.h>
#include <gtkmm/box.h>
#include <gtkmm/grid.h>
#include <gtkmm/checkbutton.h>
#include <gtkmm/filechooserdialog.h>
#include <gtkmm/combobox.h>
#include <gtkmm/notebook.h>
#include <gtkmm/frame.h>
#include <gtkmm/textview.h>
#include <gtkmm/main.h>

namespace UI_GTKMM_NS {

/** constructor
 **
 ** @param    player   the corresponding (Gtk-)Player
 **/
Players::Player::AiConfig::AiConfig(Player* const player) :
  Base(player),
  sigc::trackable(),
  player(player)
{
  this->init();
} // Players::Player::AiConfig::AiConfig(Player* player)

/** destruktor
 **/
Players::Player::AiConfig::~AiConfig()
{
  for (auto& i : this->type_bool)
    delete static_cast<Aiconfig::Type::Bool*>(i->steal_data("type"));

  for (auto& i : this->type_int)
    delete static_cast<Aiconfig::Type::Int*>(i->steal_data("type"));

  for (auto& i : this->type_card)
    delete static_cast<Aiconfig::Type::Card*>(i->steal_data("type"));
} // Players::Player::AiConfig::~AiConfig()

/** creates all subelements
 **/
void
Players::Player::AiConfig::init()
{
  { // create the widgets
    { // buttons
      this->difficulty_label
        = Gtk::manage(new Gtk::Label(_("Label::difficulty")));
      this->difficulty_button
        = Gtk::manage(new Gtk::Button(_("Button::hardcoded")));
      this->difficulty_button->set_image_from_icon_name("edit-clear-all");
      this->difficulty_button->set_always_show_image();
      this->difficulty_selector
        = Gtk::manage(new Gtk::LabelTypeSelector< ::Aiconfig::Difficulty>());
      this->difficulty_selector->get_label()->set_label(_("AiConfig::difficulty") + ": ");
      for (auto const d : aiconfig_difficulty_list) {
        this->difficulty_selector->add(d);
      }

      this->adopt_button = Gtk::manage(new Gtk::Button(_("Button::adopt")));
      this->allot_button = Gtk::manage(new Gtk::Button(_("Button::allot")));
      this->load_button = Gtk::manage(new Gtk::Button(_("Button::load")));
      this->load_button->set_image_from_icon_name("document-open");
      this->load_button->set_always_show_image();
      this->save_button = Gtk::manage(new Gtk::Button(_("Button::save")));
      this->save_button->set_image_from_icon_name("document-save");
      this->save_button->set_always_show_image();

      { // file chooser
        // gettext: %s = player name
        this->save_file_chooser = make_unique<Gtk::FileChooserDialogWrapper>(_("save aiconfig of %s",
                                                                               this->player->player().name()),
                                                                             Gtk::FILE_CHOOSER_ACTION_SAVE,
                                                                             sigc::mem_fun(*this, &AiConfig::save));
        {
          string dir = (::setting(Setting::Type::private_data_directory)
                        + "/"
                        + ::setting(Setting::Type::ai_directory)
                        + "/"
                       );
#ifdef WINDOWS
          String::replace_all(dir, "/", "\\");
#endif
          this->save_file_chooser->set_current_folder(dir);
        }
      } // file chooser
    } // buttons


    for (int t = Aiconfig::Type::bool_first;
         t <= Aiconfig::Type::bool_last;
         t++) {
      this->type_bool.push_back(Gtk::manage(new Gtk::CheckButton(_(Aiconfig::Type::Bool(t)))));
      this->type_bool.back()->set_data("type", new Aiconfig::Type::Bool(Aiconfig::Type::Bool(t)));
      this->type_bool.back()->signal_toggled().connect(sigc::bind<int const>(sigc::mem_fun(*this, &AiConfig::change),
                                                                             t)
                                                      );
    } // for (t \in Aiconfig::Bool)
    for (int t = Aiconfig::Type::int_first;
         t <= Aiconfig::Type::int_last;
         t++) {
      switch (t) {
      case Aiconfig::Type::remembertricks:
      case Aiconfig::Type::lastfehlcreation:
      case Aiconfig::Type::last_tricks_without_heuristics:
      case Aiconfig::Type::first_trick_for_trump_points_optimization:
        this->type_int.push_back(Gtk::manage(new Gtk::LabelScaleSpinButton(_(Aiconfig::Type::Int(t)) + ":")));
        break;
      case Aiconfig::Type::limit_throw_fehl:
      case Aiconfig::Type::limitqueen:
      case Aiconfig::Type::limitdolle:
      case Aiconfig::Type::announcelimit:
      case Aiconfig::Type::announcelimitdec:
      case Aiconfig::Type::announceconfig:
      case Aiconfig::Type::announcelimitreply:
      case Aiconfig::Type::announceconfigreply:
      case Aiconfig::Type::takepoverty:
      case Aiconfig::Type::singlesolo:
      case Aiconfig::Type::doublesolo:
      case Aiconfig::Type::triplesolo:
      case Aiconfig::Type::colorsolo:
      case Aiconfig::Type::meatless:
        this->type_int.push_back(Gtk::manage(new Gtk::LabelSpinButton(_(Aiconfig::Type::Int(t)) + ":")));
        break;
      } // switch(t)
      this->type_int.back()->set_data("type", new Aiconfig::Type::Int(Aiconfig::Type::Int(t)));
      this->type_int.back()->signal_value_changed().connect(sigc::bind<int const>(sigc::mem_fun(*this, &AiConfig::change),
                                                                                  t)
                                                           );
    } // for (t \in Aiconfig::Type::Int)
    for (int t = Aiconfig::Type::card_first;
         t <= Aiconfig::Type::card_last;
         t++) {
      Aiconfig::Type::Card const type = static_cast<Aiconfig::Type::Card>(t);
      switch (type) {
      case Aiconfig::Type::limitthrowing:
      case Aiconfig::Type::limithigh:
      case Aiconfig::Type::trumplimit_normal:
      case Aiconfig::Type::lowest_trumplimit_normal:
        this->type_card.push_back(Gtk::manage(new Gtk::LabelCardSelector(_(type))));
        break;
      case Aiconfig::Type::trumplimit_solocolor:
      case Aiconfig::Type::lowest_trumplimit_solocolor:
      case Aiconfig::Type::trumplimit_solojack:
      case Aiconfig::Type::lowest_trumplimit_solojack:
      case Aiconfig::Type::trumplimit_soloqueen:
      case Aiconfig::Type::lowest_trumplimit_soloqueen:
      case Aiconfig::Type::trumplimit_soloking:
      case Aiconfig::Type::lowest_trumplimit_soloking:
      case Aiconfig::Type::trumplimit_solojackking:
      case Aiconfig::Type::lowest_trumplimit_solojackking:
      case Aiconfig::Type::trumplimit_solojackqueen:
      case Aiconfig::Type::lowest_trumplimit_solojackqueen:
      case Aiconfig::Type::trumplimit_soloqueenking:
      case Aiconfig::Type::lowest_trumplimit_soloqueenking:
      case Aiconfig::Type::trumplimit_solokoehler:
      case Aiconfig::Type::lowest_trumplimit_solokoehler:
      case Aiconfig::Type::trumplimit_meatless:
      case Aiconfig::Type::lowest_trumplimit_meatless:
        // the translation is added by a separate label
        this->type_card.push_back(Gtk::manage(new Gtk::LabelCardSelector()));
        break;
      default:
        DEBUG_ASSERTION(false,
                        "UI_GTKMM::AiConfig::init()\n"
                        "  Aiconfig::Type::Card '" << type << "' "
                        "not taken into account.");
        break;
      } // switch(type)
      this->type_card.back()->set_data("type",
                                       new Aiconfig::Type::Card(type));
      for (auto c : this->aiconfig().valid_cards(type))
        this->type_card.back()->add(c);
      this->type_card.back()->signal_value_changed().connect(sigc::bind<int>(sigc::mem_fun(*this, &AiConfig::change), t) );
    } // for (t \in Aiconfig::Type::Card)
  } // create the widgets

  this->create_container();

  { // the signals
    this->difficulty_button->signal_clicked().connect(sigc::mem_fun(*this, &AiConfig::hardcoded));
    this->difficulty_selector->signal_value_changed().connect(sigc::mem_fun(*this, &AiConfig::difficulty_changed));
    this->adopt_button->signal_clicked().connect(sigc::mem_fun(*this, &AiConfig::adopt));
    this->allot_button->signal_clicked().connect(sigc::mem_fun(*this, &AiConfig::allot));
    this->load_button->signal_clicked().connect(sigc::mem_fun(*this, &AiConfig::load));
    this->save_button->signal_clicked().connect(sigc::mem_fun(*this->save_file_chooser, &Gtk::FileChooserDialogWrapper::present));

    for (auto& m : this->heuristics_default_button)
      m.second->signal_toggled().connect(sigc::bind<HeuristicsMap::Key>(sigc::mem_fun(*this, &AiConfig::heuristics_default_button_toggled_event), m.first));
  } // the signals

  this->update();
  this->sensitivity_update();

  this->heuristics_signals_active = true;
} // void Players::Player::AiConfig::init()

/** @return   a container with the configuration (in notebooks)
 **/
void
Players::Player::AiConfig::create_container()
{
  this->container
    = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 1 EX));

  { // the difficulty
    auto box = Gtk::manage(new Gtk::Box());

    box->add(*this->difficulty_label);
    box->add(*this->difficulty_selector);

    this->difficulty_container = box;
  } // the difficulty

  { // the settings
    this->notebook = Gtk::manage(new Gtk::Notebook());
    this->notebook->set_scrollable(true);

    { // the notebook pages
      { // data buttons
        auto label = Gtk::manage(new Gtk::Label(_("AiConfig::Group::data")));

        auto grid = Gtk::manage(new Gtk::Grid());
        grid->set_row_homogeneous(true);
        grid->set_column_homogeneous(true);
        grid->set_row_spacing(1 EX);
        grid->set_column_spacing(2 EM);
        grid->set_border_width(1 EX);
        grid->set_halign(Gtk::ALIGN_CENTER);
        grid->set_valign(Gtk::ALIGN_CENTER);

        //grid->attach(*this->difficulty_button, 0, 1, 0, 1);
        // the difficulty selector is put in the 'player' page
        // (see players.cpp)

        grid->attach(*this->adopt_button,        0, 0, 1, 1);
        grid->attach(*this->allot_button,        1, 0, 1, 1);
        grid->attach(*this->load_button,         0, 1, 1, 1);
        grid->attach(*this->save_button,         1, 1, 1, 1);

        this->notebook->append_page(*grid, *label);
      } // data buttons
      { // simple settings
        auto label = Gtk::manage(new Gtk::Label(_("AiConfig::Group::simple")));

        auto vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 1 EM));
        vbox->set_halign(Gtk::ALIGN_CENTER);
        vbox->set_valign(Gtk::ALIGN_CENTER);

        vbox->add(*this->type_bool[Aiconfig::Type::teams_known - Aiconfig::Type::bool_first]);
        vbox->add(*this->type_bool[Aiconfig::Type::hands_known - Aiconfig::Type::bool_first]);
        vbox->add(*this->type_bool[Aiconfig::Type::estimate_hands - Aiconfig::Type::bool_first]);
        vbox->add(*this->type_bool[Aiconfig::Type::trusting - Aiconfig::Type::bool_first]);
        vbox->add(*this->type_bool[Aiconfig::Type::aggressive - Aiconfig::Type::bool_first]);
        vbox->add(*this->type_int[Aiconfig::Type::remembertricks - Aiconfig::Type::int_first]);

        this->notebook->append_page(*vbox, *label);
      } // simple settings
      { // aitype
        auto label = Gtk::manage(new Gtk::Label(_("AiConfig::Group::aitype")));

        // ToDo: table -> ListStore
        auto grid = Gtk::manage(new Gtk::Grid());
        grid->set_halign(Gtk::ALIGN_CENTER);
        grid->set_valign(Gtk::ALIGN_CENTER);
        { // titles
          Gtk::Label* label;
#ifdef POSTPONED
          label = Gtk::manage(new Gtk::Label(_("Label::as previous")));
          grid->attach(*label, 0, 0, 1, 1);
#endif

          label = Gtk::manage(new Gtk::Label(_("Label::trick")));
          grid->attach(*label, 1, 0, 1, 1);

          label = Gtk::manage(new Gtk::Label(_("Label::AiConfig::aitype")));
          grid->attach(*label, 2, 0, 1, 1);

          label = Gtk::manage(new Gtk::Label(_("Label::AiConfig::rating")));
          grid->attach(*label, 3, 0, 1, 1);

          label = Gtk::manage(new Gtk::Label(_("Label::AiConfig::future limit")));
          grid->attach(*label, 4, 0, 1, 1);

          label = Gtk::manage(new Gtk::Label(""));
          grid->attach(*label, 0, 1, 4, 1);
        } // titles
        for (unsigned t = 0;
             t < this->ui->party().rule()(Rule::Type::max_number_of_tricks_in_game); 
             ++t) {
          { // aitype as previous
            this->aitype_as_previous.push_back(Gtk::manage(new Gtk::CheckButton()));
            if (t > 0) {
              this->aitype_as_previous.back()->set_active(   (this->aiconfig().aitype(t) != this->aiconfig().aitype(t - 1))
                                                          || (this->aiconfig().rating(t) != this->aiconfig().rating(t - 1))
                                                          || (this->aiconfig().future_limit(t) != this->aiconfig().future_limit(t - 1))
                                                         );
              grid->attach(*this->aitype_as_previous.back(), 0, t + 2, 1, 1);
              this->aitype_as_previous.back()->signal_toggled().connect(sigc::bind<unsigned const>(sigc::mem_fun(*this, &AiConfig::aitype_as_previous_changed), t));
            } // if (t > 0)
          } // aitype as previous

          { // label
            auto label = Gtk::manage(new Gtk::Label( " " + String::to_string(t + 1) + ": "));
            grid->attach(*label, 1, t + 2, 1, 1);
          } // label

          { // aitype menu
            this->aitype.push_back(Gtk::manage(new Gtk::ComboBoxText()));
            auto aitype = this->aitype.back();
            for (auto const a : ai_type_list) {
              if (a == AiType::no_choosebestcard)
                continue;
              aitype->append(to_string(a), _(a));
            }
            aitype->set_active_text(_(this->aiconfig().aitype(t)));
            aitype->signal_changed().connect(sigc::bind<unsigned>(sigc::mem_fun(*this, &AiConfig::aitype_changed), t));
            grid->attach(*aitype, 2, t + 2, 1, 1);
          } // aitype menu
          { // rating menu
            this->rating.push_back(Gtk::manage(new Gtk::ComboBoxText()));
            auto rating = this->rating.back();
            for (auto const r : Rating::type_list) {
              rating->append(to_string(r), _(r));
            }
            rating->set_active_text(_(this->aiconfig().rating(t)));
            rating->signal_changed().connect(sigc::bind<unsigned>(sigc::mem_fun(*this, &AiConfig::rating_changed), t));
            grid->attach(*rating, 3, t + 2, 1, 1);
          } // aitype menu
          { // future limit
            this->future_limit.push_back(Gtk::manage(new Gtk::SpinButton()));
            this->future_limit.back()->set_range(1, 1e9);
            this->future_limit.back()->set_increments(1, 1000);
            this->future_limit.back()->set_numeric(true);
            grid->attach(*this->future_limit.back(), 4, t + 2, 1, 1);

            this->future_limit.back()->signal_value_changed().connect(sigc::bind<unsigned const>(sigc::mem_fun(*this, &AiConfig::future_limit_changed), t));
          } // future limit
        } // for (t < max_number_of_tricks_in_game)

        this->notebook->append_page(*grid, *label);
      } // aitype
      { // game decision
        auto label = Gtk::manage(new Gtk::Label(_("AiConfig::Group::game decision")));

        auto vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 1 EM));
        vbox->set_halign(Gtk::ALIGN_CENTER);
        vbox->set_valign(Gtk::ALIGN_CENTER);

        vbox->add(*this->type_int[Aiconfig::Type::takepoverty - Aiconfig::Type::int_first]);
        vbox->add(*this->type_int[Aiconfig::Type::singlesolo - Aiconfig::Type::int_first]);
        vbox->add(*this->type_int[Aiconfig::Type::doublesolo - Aiconfig::Type::int_first]);
        vbox->add(*this->type_int[Aiconfig::Type::triplesolo - Aiconfig::Type::int_first]);
        vbox->add(*this->type_int[Aiconfig::Type::colorsolo - Aiconfig::Type::int_first]);
        vbox->add(*this->type_int[Aiconfig::Type::meatless - Aiconfig::Type::int_first]);

        this->notebook->append_page(*vbox, *label);
      } // game decision
      { // announcement
        auto label = Gtk::manage(new Gtk::Label(_("AiConfig::Group::announcement")));

        auto vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 1 EM));
        vbox->set_halign(Gtk::ALIGN_CENTER);
        vbox->set_valign(Gtk::ALIGN_CENTER);

        vbox->add(*(this->type_int[Aiconfig::Type::announcelimit - Aiconfig::Type::int_first]));
        vbox->add(*(this->type_int[Aiconfig::Type::announcelimitdec - Aiconfig::Type::int_first]));
        vbox->add(*(this->type_int[Aiconfig::Type::announceconfig - Aiconfig::Type::int_first]));
        vbox->add(*(this->type_int[Aiconfig::Type::announcelimitreply - Aiconfig::Type::int_first]));
        vbox->add(*(this->type_int[Aiconfig::Type::announceconfigreply - Aiconfig::Type::int_first]));

        this->notebook->append_page(*vbox, *label);
      } // announcement
      { // trick decision
        auto label = Gtk::manage(new Gtk::Label(_("AiConfig::Group::trick decision")));

        auto vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 1 EM));
        vbox->set_halign(Gtk::ALIGN_CENTER);
        vbox->set_valign(Gtk::ALIGN_CENTER);

        vbox->add(*this->type_bool[Aiconfig::Type::fehlcreationonfirstcard - Aiconfig::Type::bool_first]);
        vbox->add(*this->type_int[Aiconfig::Type::last_tricks_without_heuristics - Aiconfig::Type::int_first]);
        vbox->add(*this->type_int[Aiconfig::Type::first_trick_for_trump_points_optimization - Aiconfig::Type::int_first]);
        vbox->add(*this->type_int[Aiconfig::Type::lastfehlcreation - Aiconfig::Type::int_first]);
        vbox->add(*this->type_int[Aiconfig::Type::limit_throw_fehl - Aiconfig::Type::int_first]);
        vbox->add(*this->type_int[Aiconfig::Type::limitqueen - Aiconfig::Type::int_first]);
        vbox->add(*this->type_int[Aiconfig::Type::limitdolle - Aiconfig::Type::int_first]);

        this->notebook->append_page(*vbox, *label);
      } // trick decision
      { // card limits
        auto label = Gtk::manage(new Gtk::Label(_("AiConfig::Group::card limits")));

        auto card_limits_notebook = Gtk::manage(new Gtk::Notebook());
        this->notebooks.push_back(card_limits_notebook);

        { // normal game
          auto label = Gtk::manage(new Gtk::Label(_("AiConfig::CardLimit::Group::normal game")));

          auto vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 1 EX / 2));
          vbox->set_halign(Gtk::ALIGN_CENTER);
          vbox->set_valign(Gtk::ALIGN_CENTER);

          vbox->add(*this->type_card[Aiconfig::Type::limitthrowing - Aiconfig::Type::card_first]);
          vbox->add(*this->type_card[Aiconfig::Type::trumplimit_normal - Aiconfig::Type::card_first]);
          vbox->add(*this->type_card[Aiconfig::Type::lowest_trumplimit_normal - Aiconfig::Type::card_first]);
          vbox->add(*this->type_card[Aiconfig::Type::limithigh - Aiconfig::Type::card_first]);

          card_limits_notebook->append_page(*vbox, *label);
        } // normal game
        { // other solo
          auto label = Gtk::manage(new Gtk::Label(_("AiConfig::CardLimit::Group::solo")));

          auto grid = Gtk::manage(new Gtk::Grid());
          grid->set_row_spacing(1 EX / 2);
          grid->set_column_spacing(1 EM / 2);
          grid->set_border_width(1 EX);
          grid->set_halign(Gtk::ALIGN_CENTER);
          grid->set_valign(Gtk::ALIGN_CENTER);
          { // the grid
            Gtk::Label* label;
            int row = 0;

            // title
            label = Gtk::manage(new Gtk::Label(_("AiConfig::CardLimit::Group::trump limit")));
            grid->attach(*label, 1, row, 1, 1);
            label = Gtk::manage(new Gtk::Label(_("AiConfig::CardLimit::Group::lowest trump limit")));
            grid->attach(*label, 2, row, 1, 1);
            row += 1;
            grid->set_row_spacing(3 EX / 2);
            // color solo
            label = Gtk::manage(new Gtk::Label(_("GameType::Group::Solo::color") + ": "));
            grid->attach(*label, 0, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::trumplimit_solocolor - ::Aiconfig::Type::card_first],
                         1, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::lowest_trumplimit_solocolor - ::Aiconfig::Type::card_first],
                         2, row, 1, 1);
            row += 1;
            // jack solo
            label = Gtk::manage(new Gtk::Label(_(GameType::solo_jack) + ": "));
            grid->attach(*label, 0, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::trumplimit_solojack - ::Aiconfig::Type::card_first],
                         1, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::lowest_trumplimit_solojack - ::Aiconfig::Type::card_first],
                         2, row, 1, 1);
            row += 1;
            // queen solo
            label = Gtk::manage(new Gtk::Label(_(GameType::solo_queen) + ": "));
            grid->attach(*label, 0, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::trumplimit_soloqueen - ::Aiconfig::Type::card_first],
                         1, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::lowest_trumplimit_soloqueen - ::Aiconfig::Type::card_first],
                         2, row, 1, 1);
            row += 1;
            // king solo
            label = Gtk::manage(new Gtk::Label(_(GameType::solo_king) + ": "));
            grid->attach(*label, 0, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::trumplimit_soloking - ::Aiconfig::Type::card_first],
                         1, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::lowest_trumplimit_soloking - ::Aiconfig::Type::card_first],
                         2, row, 1, 1);
            row += 1;
            // queen-jack solo
            label = Gtk::manage(new Gtk::Label(_(GameType::solo_queen_jack) + ": "));
            grid->attach(*label, 0, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::trumplimit_solojackqueen - ::Aiconfig::Type::card_first],
                         1, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::lowest_trumplimit_solojackqueen - ::Aiconfig::Type::card_first],
                         2, row, 1, 1);
            row += 1;
            // king-jack solo
            label = Gtk::manage(new Gtk::Label(_(GameType::solo_king_jack) + ": "));
            grid->attach(*label, 0, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::trumplimit_solojackking - ::Aiconfig::Type::card_first],
                         1, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::lowest_trumplimit_solojackking - ::Aiconfig::Type::card_first],
                         2, row, 1, 1);
            row += 1;
            // king-queen solo
            label = Gtk::manage(new Gtk::Label(_(GameType::solo_king_queen) + ": "));
            grid->attach(*label, 0, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::trumplimit_soloqueenking - ::Aiconfig::Type::card_first],
                         1, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::lowest_trumplimit_soloqueenking - ::Aiconfig::Type::card_first],
                         2, row, 1, 1);
            row += 1;
            // koehler
            label = Gtk::manage(new Gtk::Label(_(GameType::solo_koehler) + ": "));
            grid->attach(*label, 0, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::trumplimit_solokoehler - ::Aiconfig::Type::card_first],
                         1, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::lowest_trumplimit_solokoehler - ::Aiconfig::Type::card_first],
                         2, row, 1, 1);
            row += 1;
            // meatless
            label = Gtk::manage(new Gtk::Label(_(GameType::solo_meatless) + ": "));
            grid->attach(*label, 0, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::trumplimit_meatless - ::Aiconfig::Type::card_first],
                         1, row, 1, 1);
            grid->attach(*this->type_card[::Aiconfig::Type::lowest_trumplimit_meatless - ::Aiconfig::Type::card_first],
                         2, row, 1, 1);
            row += 1;

            (void) row;
          } // the table
          card_limits_notebook->append_page(*grid, *label);

        } // other solo

        this->notebook->append_page(*card_limits_notebook, *label);
      } // card limits
      { // heuristics
        auto label = Gtk::manage(new Gtk::Label(_("AiConfig::Group::heuristics")));

        auto heuristics_notebook = Gtk::manage(new Gtk::Notebook());
        this->notebooks.push_back(heuristics_notebook);
        this->heuristics_model = new HeuristicsModel();


        { // normal game
#if 0
          Gtk::Label* label
            = Gtk::manage(new Gtk::Label(_((HeuristicsMap::normal))));


          heuristics_notebook->append_page(*this->create_heuristic_treeview(HeuristicsMap::DEFAULT, HeuristicsMap::re),
                                           *label);
#else
          auto normal_notebook = Gtk::manage(new Gtk::Notebook());
          this->notebooks.push_back(normal_notebook);
          this->add_heuristic_treeviews_re_contra(heuristics_notebook,
                                                  HeuristicsMap::GameTypeGroup::normal);
#endif
        } // normal game
        { // marriage
          auto label = Gtk::manage(new Gtk::Label(_("HeuristicsMap::GameTypeGroup::marriage")));

          auto marriage_notebook = Gtk::manage(new Gtk::Notebook());
          this->notebooks.push_back(marriage_notebook);

          this->add_heuristic_treeviews_re_contra(marriage_notebook,
                                                  HeuristicsMap::GameTypeGroup::marriage_undetermined);
          this->add_heuristic_treeviews_re_contra(marriage_notebook,
                                                  HeuristicsMap::GameTypeGroup::marriage_silent);

          heuristics_notebook->append_page(*marriage_notebook,
                                           *label);
        } // marriage
        { // poverty
          this->add_heuristic_treeviews_special_re_contra(heuristics_notebook,
                                                          HeuristicsMap::GameTypeGroup::poverty);
        } // poverty
        { // soli
          auto label = Gtk::manage(new Gtk::Label(_("HeuristicsMap::GameTypeGroup::soli")));

          auto soli_notebook = Gtk::manage(new Gtk::Notebook());
          this->notebooks.push_back(soli_notebook);

          this->add_heuristic_treeviews_re_contra(soli_notebook,
                                                  HeuristicsMap::GameTypeGroup::soli_color);
          this->add_heuristic_treeviews_re_contra(soli_notebook,
                                                  HeuristicsMap::GameTypeGroup::soli_single_picture);
          this->add_heuristic_treeviews_re_contra(soli_notebook,
                                                  HeuristicsMap::GameTypeGroup::soli_double_picture);
          this->add_heuristic_treeviews_re_contra(soli_notebook,
                                                  HeuristicsMap::GameTypeGroup::solo_koehler);
          this->add_heuristic_treeviews_re_contra(soli_notebook,
                                                  HeuristicsMap::GameTypeGroup::solo_meatless);

          heuristics_notebook->append_page(*soli_notebook,
                                           *label);
        } // soli

        this->notebook->append_page(*heuristics_notebook, *label);
      } // heuristics
    } // the notebook pages

    this->container->pack_start(*this->notebook, true, true);
  } // the settings

#ifndef RELEASE
#ifdef DKNOF
  { // test, whether all configuration buttons are packed in a container
    for (auto const type : Aiconfig::Type::bool_list) {
      if (!this->type_bool[type - Aiconfig::Type::bool_first]->get_parent()) {
        cerr << "UI_GTKMM::Players::Player::AiConfig::create_container()\n"
          << "  config '" << type << "' not packed\n";
      }
    }
    for (auto const type : Aiconfig::Type::int_list) {
      if (!this->type_int[type - Aiconfig::Type::int_first]->get_parent()) {
        cerr << "UI_GTKMM::Players::Player::AiConfig::create_container()\n"
          << "  config '" << type << "' not packed\n";
      }
    }
    for (auto const type : Aiconfig::Type::card_list) {
      if (!this->type_card[type - Aiconfig::Type::card_first]->get_parent()) {
        cerr << "UI_GTKMM::Players::Player::AiConfig::create_container()\n"
          << "  config '" << type << "' not packed\n";
      }
    }
  } // test, whether all configuration buttons are packed in a container
#endif
#endif
} // void Players::Player::AiConfig::create_container()

/** adds a treeview for the heuristic group to the notbook
 **
 ** @param    notebook           notebook to add the treeview to
 ** @param    gametype_group     gametype group
 ** @param    playertype_group   playertype group
 ** @param    label_text         label translation
 **/
void
Players::Player::AiConfig::add_heuristic_treeview(Gtk::Notebook* const notebook,
                                                  HeuristicsMap::GameTypeGroup const gametype_group,
                                                  HeuristicsMap::PlayerTypeGroup const playertype_group,
                                                  string const& label_text)
{
  auto label = Gtk::manage(new Gtk::Label(label_text));

  notebook->append_page(*this->create_heuristic_treeview(gametype_group,
                                                         playertype_group),
                        *label);
} // void Players::Player::AiConfig::add_heuristic_treeview(Gtk::Notebook* notebook, HeuristicsMap::GameTypeGroup gametype_group, HeuristicsMap::PlayerTypeGroup playertype_group, string label)

/** adds a treeview for the heuristic group to the notbook
 ** a subpage for the re player and the contra player
 **
 ** @param    notebook           notebook to add the treeview to
 ** @param    gametype_group     gametype group
 **/
void
Players::Player::AiConfig::add_heuristic_treeviews_re_contra(Gtk::Notebook* const notebook,
                                                             HeuristicsMap::GameTypeGroup const gametype_group)
{
#ifdef WORKAROUND
  auto text = _("HeuristicsMap::GameTypeGroup::" + name(gametype_group));
  auto label = Gtk::manage(new Gtk::Label(text));
#else
  auto label = Gtk::manage(new Gtk::Label(_(gametype_group)));
#endif

  auto subnotebook = Gtk::manage(new Gtk::Notebook());
  this->notebooks.push_back(subnotebook);

  this->add_heuristic_treeview(subnotebook,
                               gametype_group,
                               HeuristicsMap::PlayerTypeGroup::re,
#ifdef WORKAROUND
                               _("HeuristicsMap::PlayerTypeGroup::" + name(HeuristicsMap::PlayerTypeGroup::re))
#else
                               _(HeuristicsMap::PlayerTypeGroup::re)
#endif
                              );
  this->add_heuristic_treeview(subnotebook,
                               gametype_group,
                               HeuristicsMap::PlayerTypeGroup::contra,
#ifdef WORKAROUND
                               _("HeuristicsMap::PlayerTypeGroup::" + name(HeuristicsMap::PlayerTypeGroup::contra))
#else
                               _(HeuristicsMap::PlayerTypeGroup::contra)
#endif
                              );

  notebook->append_page(*subnotebook, *label);
} // void Players::Player::AiConfig::add_heuristic_treeviews_re_contra(Gtk::Notebook* notebook, HeuristicsMap::GameTypeGroup gametype_group)

/** adds a treeview for the heuristic group to the notbook
 ** a subpage for the special player, the re player and the contra player
 **
 ** @param    notebook           notebook to add the treeview to
 ** @param    gametype_group     gametype group
 **/
void
Players::Player::AiConfig::add_heuristic_treeviews_special_re_contra(Gtk::Notebook* const notebook,
                                                                     HeuristicsMap::GameTypeGroup const gametype_group)
{
#ifdef WORKAROUND
  auto text = _("HeuristicsMap::GameTypeGroup::" + name(gametype_group));
  auto label = Gtk::manage(new Gtk::Label(text));
#else
  auto label = Gtk::manage(new Gtk::Label(_(gametype_group)));
#endif

  auto subnotebook = Gtk::manage(new Gtk::Notebook());
  this->notebooks.push_back(subnotebook);

  this->add_heuristic_treeview(subnotebook,
                               gametype_group,
                               HeuristicsMap::PlayerTypeGroup::special,
#ifdef WORKAROUND
                               _("HeuristicsMap::PlayerTypeGroup::" + name(HeuristicsMap::PlayerTypeGroup::special))
#else
                               _(HeuristicsMap::PlayerTypeGroup::special)
#endif
                              );
  this->add_heuristic_treeview(subnotebook,
                               gametype_group,
                               HeuristicsMap::PlayerTypeGroup::re,
#ifdef WORKAROUND
                               _("HeuristicsMap::PlayerTypeGroup::" + name(HeuristicsMap::PlayerTypeGroup::re))
#else
                               _(HeuristicsMap::PlayerTypeGroup::re)
#endif
                              );
  this->add_heuristic_treeview(subnotebook,
                               gametype_group,
                               HeuristicsMap::PlayerTypeGroup::contra,
#ifdef WORKAROUND
                               _("HeuristicsMap::PlayerTypeGroup::" + name(HeuristicsMap::PlayerTypeGroup::contra))
#else
                               _(HeuristicsMap::PlayerTypeGroup::contra)
#endif
                              );

  notebook->append_page(*subnotebook, *label);
} // void Players::Player::AiConfig::add_heuristic_treeviews_special_re_contra(Gtk::Notebook* notebook, HeuristicsMap::GameTypeGroup gametype_group)

/** creates a treeview for the given gametype/playertype combination
 **
 ** @param    gametype_group     gametype group
 ** @param    playertype_group   playertype group
 **
 ** @return   a scrolled window containing the treeview
 **/
Gtk::Widget*
Players::Player::AiConfig::create_heuristic_treeview(HeuristicsMap::GameTypeGroup const gametype_group,
                                                     HeuristicsMap::PlayerTypeGroup const playertype_group)
{
  auto box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL));
  box->signal_realize().connect(sigc::bind<Gtk::Box*, HeuristicsMap::GameTypeGroup, HeuristicsMap::PlayerTypeGroup>(sigc::mem_fun(*this, &Players::Player::AiConfig::init_heuristic_treeview), box, gametype_group, playertype_group));

  return box;
} // Gtk::Widget* Players::Player::AiConfig::create_heuristic_treeview(HeuristicsMap::GameTypeGroup gametype_group, HeuristicsMap::PlayerTypeGroup playertype_group)

/** creates a treeview for the given gametype/playertype combination
 **
 ** @param    gametype_group     gametype group
 ** @param    playertype_group   playertype group
 **
 ** @return   a scrolled window containing the treeview
 **/
void
Players::Player::AiConfig::init_heuristic_treeview(Gtk::Box* const box,
                                                   HeuristicsMap::GameTypeGroup const gametype_group,
                                                   HeuristicsMap::PlayerTypeGroup const playertype_group)
{
  HeuristicsMap::Key key(gametype_group, playertype_group);

  { // default button
    this->heuristics_default_button[key]
      = Gtk::manage(new Gtk::CheckButton(_("Button::heuristics as in normal game")));
    auto& default_button = *this->heuristics_default_button[key];

    if (   (gametype_group != HeuristicsMap::GameTypeGroup::normal)
        && (key != HeuristicsMap::Key()) ) {
      box->pack_start(default_button, Gtk::PACK_SHRINK);
    }
  } // default button

  { // table
    auto box_treeview = Gtk::manage(new Gtk::Box());
    { // treeview
      auto& model = this->heuristics_model;
      auto& store = this->heuristics_list[key];
      auto& treeview = heuristics_treeview[key];

      auto const& states = this->aiconfig_const().heuristic_states(key);

      store = Gtk::ListStore::create(*model);
      treeview = Gtk::manage(new Gtk::TreeView(store));
      treeview->set_property("reorderable", true);
      treeview->set_property("headers_visible", false);

      treeview->append_column_editable("active", model->active);
      treeview->get_column_cell_renderer(0)->set_property("xalign", 0.5);
      treeview->append_column("heuristic", model->heuristic_name);
      treeview->get_column_cell_renderer(1)->set_property("xalign", 0);

      { // create the rows
        for (auto s : states) {
          auto row = *store->append();

          auto const heuristic = s.heuristic;
          if (!is_real(heuristic))
            continue;

          row[model->active] = this->aiconfig().value(key, s.heuristic);
          row[model->heuristic] = heuristic;
          row[model->heuristic_name] = _(heuristic);
          row[model->gametype_group] = gametype_group;
          row[model->playertype_group] = playertype_group;
        } // for (auto s : states)
      } // create the rows

      { // events
        store->signal_row_changed().connect(sigc::mem_fun(*this, &AiConfig::heuristics_row_changed_event));
        store->signal_row_deleted().connect(sigc::bind<Glib::RefPtr<Gtk::ListStore> >(sigc::mem_fun(*this, &AiConfig::heuristics_row_deleted_event), store));
        treeview->get_selection()->signal_changed().connect(sigc::bind<HeuristicsMap::Key>(sigc::mem_fun(*this, &AiConfig::update_heuristic_up_down_buttons), key));
        treeview->get_selection()->signal_changed().connect(sigc::bind<HeuristicsMap::Key>(sigc::mem_fun(*this, &AiConfig::update_heuristic_description), key));
      } // events

      treeview->set_valign(Gtk::ALIGN_FILL);
      auto scrolled_window = Gtk::manage(new Gtk::ScrolledWindow);
      scrolled_window->set_policy(Gtk::POLICY_NEVER,
                                  Gtk::POLICY_AUTOMATIC);
      scrolled_window->add(*treeview);

      scrolled_window->set_valign(Gtk::ALIGN_FILL);
      box_treeview->pack_start(*scrolled_window, Gtk::PACK_SHRINK);
    } // treeview
    { // right column
      auto box3 = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 1 EX));
      box3->set_border_width(1 EX);
      { // move up/down buttons
        auto button_box = Gtk::manage(new Gtk::ButtonBox(Gtk::ORIENTATION_VERTICAL));
        button_box->set_layout(Gtk::BUTTONBOX_SPREAD);
        button_box->set_spacing(1 EX);

        this->heuristic_up_button[key]
          = Gtk::manage(new Gtk::Button(_("Button::move up")));
        this->heuristic_up_button[key]->set_image_from_icon_name("go-up");
        this->heuristic_up_button[key]->set_always_show_image();
        this->heuristic_down_button[key]
          = Gtk::manage(new Gtk::Button(_("Button::move down")));
        this->heuristic_down_button[key]->set_image_from_icon_name("go-down");
        this->heuristic_down_button[key]->set_always_show_image();

        button_box->add(*this->heuristic_up_button[key]);
        button_box->add(*this->heuristic_down_button[key]);

        box3->pack_start(*button_box, Gtk::PACK_SHRINK);

        this->update_heuristic_up_down_buttons(key);
        this->heuristic_up_button[key]->signal_clicked().connect(sigc::bind<HeuristicsMap::Key>(sigc::mem_fun(*this, &AiConfig::heuristic_move_up_event), key));
        this->heuristic_down_button[key]->signal_clicked().connect(sigc::bind<HeuristicsMap::Key>(sigc::mem_fun(*this, &AiConfig::heuristic_move_down_event), key));
      } // move up/down buttons
      { // description
        auto description_view = Gtk::manage(new Gtk::TextView());
        this->heuristic_description[key] = description_view;

        description_view->set_editable(false);
        description_view->set_wrap_mode(Gtk::WRAP_WORD);

        description_view->get_buffer()->set_text("");

        auto scrolled_window = Gtk::manage(new Gtk::ScrolledWindow);
        scrolled_window->set_policy(Gtk::POLICY_NEVER,
                                    Gtk::POLICY_AUTOMATIC);
        scrolled_window->add(*description_view);
        box3->pack_end(*scrolled_window, Gtk::PACK_EXPAND_WIDGET);
      } // description
      box_treeview->pack_end(*box3, Gtk::PACK_EXPAND_WIDGET);
    } // right column
    box->pack_end(*box_treeview, Gtk::PACK_EXPAND_WIDGET);
  } // table
  box->show_all();
} // void Players::Player::AiConfig::init_heuristic_treeview(Gtk::Box* box, HeuristicsMap::GameTypeGroup gametype_group, HeuristicsMap::PlayerTypeGroup playertype_group)

/** @return   the ai configuration
 **/
::Aiconfig&
Players::Player::AiConfig::aiconfig()
{
  return dynamic_cast<Aiconfig&>(this->player->player());
} // ::Aiconfig& Players::Player::AiConfig::aiconfig()

/** @return   the ai configuration (const version)
 **/
::Aiconfig const&
Players::Player::AiConfig::aiconfig_const() const
{
  return dynamic_cast<Aiconfig&>(this->player->player());
} // ::Aiconfig const& Players::Player::AiConfig::aiconfig_const() const

/** sets the aiconfig to the selected difficulty
 **/
void
Players::Player::AiConfig::difficulty_changed()
{
  auto const difficulty = this->difficulty_selector->type();
  if (difficulty == ::Aiconfig::Difficulty::custom)
    return ;

  if (difficulty == this->aiconfig().difficulty())
    return ;

  for (size_t t = 1; t < this->aitype.size(); ++t)
    this->aitype_as_previous[t]->set_active(true);

  this->aiconfig().set_to_difficulty(difficulty);
  this->update();

  for (size_t t = 1; t < this->aitype.size(); ++t)
    if (   (this->aiconfig().aitype(t)
            == this->aiconfig().aitype(t - 1))
        && (this->aiconfig().rating(t)
            == this->aiconfig().rating(t - 1))
        && (this->aiconfig().future_limit(t)
            == this->aiconfig().future_limit(t - 1)) )
      this->aitype_as_previous[t]->set_active(false);
} // void Players::Player::AiConfig::difficulty_changed()

/** the aitype has changed
 **
 ** @param    trickno      the trick number of the changed type
 **/
void
Players::Player::AiConfig::aitype_changed(unsigned const trickno)
{
  this->aiconfig().enter_update();

  this->aiconfig().set_aitype(trickno, aitype_from_string(this->aitype[trickno]->get_active_id()));

  if (   (trickno < this->aitype_as_previous.size() - 1)
      && !this->aitype_as_previous[trickno + 1]->get_active()) {
    this->aitype[trickno + 1]->set_active_text(this->aitype[trickno]->get_active_text());
  }

  this->aiconfig().leave_update();

  if (trickno == 0)
    this->sensitivity_update();
} // void Players::Player::AiConfig::aitype_changed(unsigned trickno)

/** the rating has changed
 **
 ** @param    trickno      the trick number of the changed type
 **/
void
Players::Player::AiConfig::rating_changed(unsigned const trickno)
{
  this->aiconfig().enter_update();

  this->aiconfig().set_rating(trickno, Rating::type_from_string(this->rating[trickno]->get_active_id()));

  if (   (trickno < this->aitype_as_previous.size() - 1)
      && !this->aitype_as_previous[trickno + 1]->get_active()) {
    this->rating[trickno + 1]->set_active_text(this->rating[trickno]->get_active_text());
  }

  this->aiconfig().leave_update();
} // void Players::Player::AiConfig::rating_changed(unsigned trickno)

/** the future limit has changed
 **
 ** @param    trickno      the trick number of the changed future limit
 **/
void
Players::Player::AiConfig::future_limit_changed(unsigned const trickno)
{
  this->aiconfig().enter_update();

  auto const future_limit
    = static_cast<unsigned>(this->future_limit[trickno]->get_value());
  this->aiconfig().set_future_limit(trickno, future_limit);

  if ( (trickno < this->aitype_as_previous.size() - 1)
      && !this->aitype_as_previous[trickno + 1]->get_active())
    this->future_limit[trickno + 1]->set_value(future_limit);

  this->aiconfig().leave_update();
} // void Players::Player::AiConfig::future_limit_changed(unsigned trickno)

/** the aitype has changed
 **
 ** @param    trickno      the trick number of the changed type
 **/
void
Players::Player::AiConfig::aitype_as_previous_changed(unsigned const trickno)
{
  DEBUG_ASSERTION(trickno > 0,
                  "AiConfig::aitype_as_previous_changed(trickno):\n"
                  "  'trickno' == 0");

  auto const active = this->aitype_as_previous[trickno]->get_active();
  this->aitype[trickno]->set_sensitive(active);
  this->rating[trickno]->set_sensitive(active);
  this->future_limit[trickno]->set_sensitive(active);

  if (!active) {
    this->aitype[trickno]->set_active_text(this->aitype[trickno - 1]->get_active_text());
    this->rating[trickno]->set_active_text(this->rating[trickno - 1]->get_active_text());
    this->future_limit[trickno]->set_value(this->future_limit[trickno - 1]->get_value());
  } // if (!active)
} // void Players::Player::AiConfig::aitype_as_previous_changed(unsigned trickno)

/** a aiconfig has been changed by the user
 **
 ** @param    type   the type of the aiconfig
 **/
void
Players::Player::AiConfig::change(int const type)
{
  if ((type >= Aiconfig::Type::bool_first)
      && (type <= Aiconfig::Type::bool_last))
    this->aiconfig().set(Aiconfig::Type::Bool(type),
                         this->type_bool[type - Aiconfig::Type::bool_first]->get_active());
  else if ((type >= Aiconfig::Type::int_first)
           && (type <= Aiconfig::Type::int_last)) {
    if (this->type_int[type - Aiconfig::Type::int_first]->get_realized())
      this->aiconfig().set(Aiconfig::Type::Int(type),
                           this->type_int[type - Aiconfig::Type::int_first]->get_value_as_int());
  }
  else if ((type >= Aiconfig::Type::card_first)
           && (type <= Aiconfig::Type::card_last)) {
    if (this->type_card[type - Aiconfig::Type::card_first]->get_realized())
      this->aiconfig().set(Aiconfig::Type::Card(type),
                           this->type_card[type - Aiconfig::Type::card_first]->card());
  }
  else 
    DEBUG_ASSERTION(false,
                    "AiConfig::change(type):\n"
                    "  type '" << type << "' unknown.");

  this->difficulty_selector->set_type(this->aiconfig().difficulty());
} // void Players::Player::AiConfig::change(int type)

/** a row has changed
 **
 ** @param    path    path to the changed row
 ** @param    row     changed row
 **/
void
Players::Player::AiConfig::heuristics_row_changed_event(Gtk::TreeModel::Path const& path,
                                                        Gtk::TreeModel::iterator const& row)

{
  if (!this->heuristics_signals_active)
    return ;

  auto& model = *this->heuristics_model;

  HeuristicsMap::Key const key((*row)[model.gametype_group],
                               (*row)[model.playertype_group]);

  this->aiconfig().set(key,
                       (*row)[model.heuristic],
                       (*row)[model.active]);

  this->update_heuristics_default_button_state(key);

  return ;
} // void Players::Player::AiConfig::heuristics_row_changed_event(Gtk::TreeModel::Path path, Gtk::TreeModel::iterator row, Glib::RefPtr<Gtk::ListStore> store)

/** a row has been deleted
 ** This is used for recalculating the order of the rows.
 ** Assumptions are:
 ** * this is function is called last in the reordering
 ** * the indices of 'TreeModel::Path::get_indices()' are so that the first
 **   one indicates the row number (counted from 0 on)
 **
 ** @param    path    path to the changed row
 ** @param    store   corresponding store
 **
 ** @bug      too many uncertain assumptions (see description)
 **/
void
Players::Player::AiConfig::heuristics_row_deleted_event(Gtk::TreeModel::Path const& path,
                                                        Glib::RefPtr<Gtk::ListStore> store)
{
  if (!this->heuristics_signals_active)
    return ;

  auto& model = *this->heuristics_model;

  auto& row = *store->children().begin();
  HeuristicsMap::Key const key((*row)[model.gametype_group],
                               (*row)[model.playertype_group]);
  for (auto& row : store->children()) {
    this->aiconfig().move(key,
                          (*row)[model.heuristic],
                          store->get_path(row).front()
                         );
  }
  this->update_heuristics_default_button_state(key);
} // void Players::Player::AiConfig::heuristics_row_deleted_event(Gtk::TreeModel::Path path, Glib::RefPtr<Gtk::ListStore> store)

/** update the toggled state of the 'default' button of 'key'
 **
 ** @param    key      key of the heuristic
 **/
void
Players::Player::AiConfig::update_heuristics_default_button_state(HeuristicsMap::Key const& key)
{
  bool const equal_to_default
    = (this->aiconfig_const().heuristic_states(key)
       == this->aiconfig_const().heuristic_states(HeuristicsMap::Key()));

  this->heuristics_default_button[key]->set_active(equal_to_default);
  this->heuristics_default_button[key]->set_sensitive(!equal_to_default);

  return ;
} // void Players::Player::AiConfig::update_heuristics_default_button_state(HeuristicsMap::Key const& key)

/** the default button for the heuristics of 'key' has been toggled
 **
 ** @param    key      key of the heuristic
 **/
void
Players::Player::AiConfig::heuristics_default_button_toggled_event(HeuristicsMap::Key const key)
{
  if (   this->heuristics_default_button[key]->get_active()
      && (this->aiconfig_const().heuristic_states(key)
          != this->aiconfig_const().heuristic_states(HeuristicsMap::Key()))
     ) {
    this->aiconfig().set_to_default(key);
    this->update();
  }
} // void Players::Player::AiConfig::heuristics_default_button_toggled_event(HeuristicsMap::Key key)

/** update the sensitivity of the up and down buttons
 **
 ** @param    key      key of the heuristic
 **/
void
Players::Player::AiConfig::update_heuristic_up_down_buttons(HeuristicsMap::Key const& key)
{
  auto selection = this->heuristics_treeview[key]->get_selection();
  auto store = this->heuristics_list[key];

  DEBUG_ASSERTION((selection->count_selected_rows() <= 1),
                  "UI_GTKMM::Players::Player::AiConfig::update_heuristic_up_down_buttons()\n"
                  "  more than one line selected: (" << selection->count_selected_rows() << ") for key " << key.gametype_group << "-" << key.playertype_group);

  if (selection->count_selected_rows() == 0) {
    this->heuristic_up_button[key]->set_sensitive(false);
    this->heuristic_down_button[key]->set_sensitive(false);
    return ;
  } // if (selection->count_selected_rows() == 0)

  //auto const& row = *selection->get_selected();
  auto const& path = store->get_path(selection->get_selected());
  size_t const selected_row_no = path.front();
  this->heuristic_up_button[key]->set_sensitive(selected_row_no > 0);
  this->heuristic_down_button[key]->set_sensitive(selected_row_no <
                                                  store->children().size()
                                                  - 1);
} // void Players::Player::AiConfig::update_heuristic_up_down_buttons(HeuristicsMap::Key key)

/** update the description of the active heuristic
 **
 ** @param    key      key of the heuristic
 **/
void
Players::Player::AiConfig::update_heuristic_description(HeuristicsMap::Key const& key)
{
  auto selection = this->heuristics_treeview[key]->get_selection();
  auto store = this->heuristics_list[key];

  DEBUG_ASSERTION((selection->count_selected_rows() <= 1),
                  "UI_GTKMM::Players::Player::AiConfig::update_heuristic_description()\n"
                  "  more than one line selected: (" << selection->count_selected_rows() << ") for key " << key.gametype_group << "-" << key.playertype_group);

  if (selection->count_selected_rows() == 0) {
    this->heuristic_description[key]->get_buffer()->set_text("");
    return ;
  } // if (selection->count_selected_rows() == 0)

  auto const& row = *selection->get_selected();
  this->heuristic_description[key]->get_buffer()->set_text(gettext_description(row[this->heuristics_model->heuristic]));
} // void Players::Player::AiConfig::update_heuristic_description(HeuristicsMap::Key key)

/** move the selected heuristic up
 **
 ** @param    key      key of the heuristic
 **/
void
Players::Player::AiConfig::heuristic_move_up_event(HeuristicsMap::Key const key)
{
  auto& treeview = *this->heuristics_treeview[key];
  auto selection = treeview.get_selection();

  DEBUG_ASSERTION((selection->count_selected_rows() == 1),
                  "UI_GTKMM::Players::Player::AiConfig::heuristic_move_up_event()\n"
                  "  not one line selected but '" << selection->count_selected_rows() << "' for key " << key.gametype_group << "-" << key.playertype_group);
  auto store = this->heuristics_list[key];
  auto const& path = store->get_path(selection->get_selected());
  auto const selected_row_no = path.front();
  DEBUG_ASSERTION((selected_row_no > 0),
                  "UI_GTKMM::Players::Player::AiConfig::heuristic_move_up_event()\n"
                  "  selected row is '0'");

  store->iter_swap(selection->get_selected(),
                   --selection->get_selected());
#ifdef WORKAROUND
  // call this so that the aiconfig gets noted about the change
  this->heuristics_row_deleted_event(path, store);
  // and update the sensitivity of the buttons
  this->update_heuristic_up_down_buttons(key);
#endif
  // make the selected row be visible
  treeview.scroll_to_row(store->get_path(treeview.get_selection()->get_selected()));
} // void Players::Player::AiConfig::heuristic_move_up_event(HeuristicsMap::Key key)

/** move the selected heuristic down
 **
 ** @param    key      key of the heuristic
 **/
void
Players::Player::AiConfig::heuristic_move_down_event(HeuristicsMap::Key const key)
{
  auto& treeview = *this->heuristics_treeview[key];
  auto selection = treeview.get_selection();

  DEBUG_ASSERTION((selection->count_selected_rows() == 1),
                  "UI_GTKMM::Players::Player::AiConfig::heuristic_move_up_event()\n"
                  "  not one line selected but '" << selection->count_selected_rows() << "' for key " << key.gametype_group << "-" << key.playertype_group);

  auto store = this->heuristics_list[key];
  auto const& path = store->get_path(selection->get_selected());
  size_t const selected_row_no = path.front();
  DEBUG_ASSERTION((selected_row_no < store->children().size() - 1),
                  "UI_GTKMM::Players::Player::AiConfig::heuristic_move_up_event()\n"
                  "  selected row is too great: " << selected_row_no
                  << " >= " << store->children().size() << " = num rows");

  store->iter_swap(selection->get_selected(),
                   ++selection->get_selected());
#ifdef WORKAROUND
  // call this so that the aiconfig gets noted about the change
  this->heuristics_row_deleted_event(path, store);
  // and update the sensitivity of the buttons
  this->update_heuristic_up_down_buttons(key);
#endif
  // make the selected row be visible
  treeview.scroll_to_row(store->get_path(treeview.get_selection()->get_selected()));
} // void Players::Player::AiConfig::heuristic_move_down_event(HeuristicsMap::Key key)

/** sets the ai to the hardcoded
 **/
void
Players::Player::AiConfig::hardcoded()
{
  for (size_t t = 1; t < this->aitype.size(); ++t)
    this->aitype_as_previous[t]->set_active(true);

  this->aiconfig().reset_to_hardcoded();
  this->update();

  for (size_t t = 1; t < this->aitype.size(); ++t)
    if (   (this->aiconfig().aitype(t)
            == this->aiconfig().aitype(t - 1))
        && (this->aiconfig().rating(t)
            == this->aiconfig().rating(t - 1))
        && (this->aiconfig().future_limit(t)
            == this->aiconfig().future_limit(t - 1)) )
      this->aitype_as_previous[t]->set_active(false);
} // void Players::Player::AiConfig::hardcoded()

/** creates a menu with the players to adopt from
 **/
void
Players::Player::AiConfig::adopt()
{

  auto adopt_menu = Gtk::manage(new Gtk::Menu());

  for (size_t p = 0; p < this->ui->party().players().size(); ++p)
    if (p != this->player->playerno()) {
      auto item = Gtk::manage(new Gtk::MenuItem(this->player->players->player(p).player().name()));
      item->signal_activate().connect(sigc::bind<unsigned>(sigc::mem_fun(*this, &AiConfig::adopt_aiconfig), p));
      adopt_menu->append(*item);
    } // if (p != this->player().playerno())

  adopt_menu->show_all();
  adopt_menu->popup(0, 0);
} // void Players::Player::AiConfig::adopt()

/** adopt the aiconfig from player 'playerno'
 **
 ** @param    playerno   player to adopt from
 **/
void
Players::Player::AiConfig::adopt_aiconfig(unsigned const playerno)
{
#ifdef RELEASE
  DEBUG_ASSERTION((::game_status == GameStatus::party_new),
                  "Players::Player::AiConfig::adopt_aiconfig():\n"
                  "  game status is not party_init: "
                  << ::game_status);
#endif
  for (size_t t = 1; t < this->aitype.size(); ++t)
    this->aitype_as_previous[t]->set_active(true);

  this->aiconfig()
    = this->player->players->player(playerno).aiconfig().aiconfig();
  this->update();

  for (size_t t = 1; t < this->aitype.size(); ++t)
    if (   (this->aiconfig().aitype(t)
            == this->aiconfig().aitype(t - 1))
        && (this->aiconfig().rating(t)
            == this->aiconfig().rating(t - 1))
        && (this->aiconfig().future_limit(t)
            == this->aiconfig().future_limit(t - 1)) )
      this->aitype_as_previous[t]->set_active(false);
} // void Players::Player::AiConfig::adopt_aiconfig(unsigned playerno)

/** make all other aiconfigs like this
 **/
void
Players::Player::AiConfig::allot()
{
  for (size_t p = 0; p < this->ui->party().players().size(); ++p)
    if (p != this->player->playerno())
      this->player->players->player(p).aiconfig().adopt_aiconfig(this->player->playerno());
} // void Players::Player::AiConfig::allot()

/** creates a menu with the files in the aiconfig directories
 **/
void
Players::Player::AiConfig::load()
{
  auto load_menu = Gtk::manage(new Gtk::FileMenu(sigc::mem_fun(*this, &AiConfig::load_file)));
  load_menu->add_directory(::setting(Setting::Type::private_data_directory)
                           + "/"
                           + ::setting(Setting::Type::ai_directory));
  load_menu->add_directory(::setting(Setting::Type::public_data_directory)
                           + "/"
                           + ::setting(Setting::Type::ai_directory));

  load_menu->show();
} // void Players::Player::AiConfig::load()

/** loads the aiconfig
 **
 ** @param    filename   the aiconfig file
 **/
void
Players::Player::AiConfig::load_file(string const filename)
{
  for (size_t t = 1; t < this->aitype.size(); ++t)
    this->aitype_as_previous[t]->set_active(true);

  this->aiconfig().load(filename);
  this->update();

  for (size_t t = 1; t < this->aitype.size(); ++t)
    if (   (this->aiconfig().aitype(t)
            == this->aiconfig().aitype(t - 1))
        && (this->aiconfig().rating(t)
            == this->aiconfig().rating(t - 1))
        && (this->aiconfig().future_limit(t)
            == this->aiconfig().future_limit(t - 1)) )
      this->aitype_as_previous[t]->set_active(false);
} // void Players::Player::AiConfig::load_file(string filename)

/** saves the aiconfig.
 **
 ** @param    filename   the filename
 **
 ** @return   whether the saving was successfull
 **/
bool
Players::Player::AiConfig::save(string const filename)
{
  return this->aiconfig().save(filename);
} // bool Players::Player::AiConfig::save(string filename)

/** update the sensitivity of all widgets
 **
 ** @todo     heuristics
 **/
void
Players::Player::AiConfig::sensitivity_update()
{
#ifdef RELEASE
  bool sensitive = (::game_status == GameStatus::party_new);
#else
  bool sensitive = true;
#endif

  this->difficulty_button->set_sensitive(sensitive);
  this->adopt_button->set_sensitive(sensitive);
  this->allot_button->set_sensitive(sensitive);
  this->load_button->set_sensitive(sensitive);
  this->save_button->set_sensitive(true);
  this->difficulty_selector->set_sensitive(sensitive);

  for (auto t : this->type_bool)
    t->set_sensitive(sensitive);
  for (auto t : this->type_int)
    t->set_sensitive(sensitive);
  for (auto t : this->type_card)
    t->set_sensitive(sensitive);

  for (size_t t = 0; t < this->aitype.size(); ++t) {
    if (   (t == 0)
        || this->aitype_as_previous[t]->get_active() ) {
      this->aitype[t]->set_sensitive(sensitive);
      this->rating[t]->set_sensitive(::supports_rating(this->aiconfig().aitype(t))
                                     ? sensitive
                                     : false);
      this->future_limit[t]->set_sensitive(sensitive);
      this->aitype_as_previous[t]->set_sensitive(sensitive);
    } else {
      this->aitype[t]->set_sensitive(false);
      this->rating[t]->set_sensitive(false);
      this->future_limit[t]->set_sensitive(false);
    }
  } // for (size_t t = 1; t < this->aitype.size(); ++t)

  for (auto m : this->heuristics_default_button)
    m.second->set_sensitive(sensitive);
  for (auto m : this->heuristics_treeview)
    m.second->set_sensitive(sensitive);
} // void Players::Player::AiConfig::sensitivity_update()

/** update all widgets
 **/
void
Players::Player::AiConfig::update()
{
  this->difficulty_selector->set_type(this->aiconfig().difficulty());
  this->difficulty_description_update();

  for (size_t t = 0; t < this->aitype.size(); ++t) {
    if ((t < this->aitype.size() - 1)
        && (   (this->aiconfig().aitype(t + 1)
                != this->aiconfig().aitype(t))
            || (this->aiconfig().future_limit(t + 1)
                != this->aiconfig().future_limit(t))))
      this->aitype_as_previous[t + 1]->set_active(true);
    this->aitype[t]->set_active_text(_(this->aiconfig().aitype(t)));
    this->rating[t]->set_active_text(_(this->aiconfig().rating(t)));
    this->future_limit[t]->set_value(this->aiconfig().future_limit(t));
  }

  for (auto widget : this->type_bool)
    widget->set_active(this->aiconfig().value(*static_cast<Aiconfig::Type::Bool*>(widget->get_data("type"))));
  for (auto widget : this->type_int) {
    auto const type
      = *static_cast<Aiconfig::Type::Int*>(widget->get_data("type"));
    widget->set_range(this->aiconfig().min(type),
                      this->aiconfig().max(type));
    widget->set_value(this->aiconfig().value(type));
  } // for (auto widget = this->type_int)
  for (auto widget : this->type_card) {
    auto const type
      = *static_cast<Aiconfig::Type::Card*>(widget->get_data("type"));
    widget->set_card(this->aiconfig().value(type));
  } // for (widget \in this->type_card)

  { // heuristics
    this->heuristics_signals_active = false;
    auto& model = *this->heuristics_model;
    for (auto m : this->heuristics_list) {
      auto const& key = m.first;
      auto const& states = this->aiconfig_const().heuristic_states(key);
      auto& store = m.second;

      { // create the rows
        store->clear();
        for (auto s : states) {
          auto row = *store->append();

          row[model.active] = this->aiconfig().value(key, s.heuristic);
          row[model.heuristic] = s.heuristic;
          row[model.heuristic_name] = _(s.heuristic);
          row[model.gametype_group] = key.gametype_group;
          row[model.playertype_group] = key.playertype_group;

        } // for (auto s : states)
      } // create the rows

      this->update_heuristics_default_button_state(key);
    } // for (m \in this->heuristics_list)
    this->heuristics_signals_active = true;
  } // heuristics
} // void Players::Player::AiConfig::update()

/** update the name
 **
 ** @todo   block 'Entry::change_signal()'
 **/
void
Players::Player::AiConfig::name_update()
{
  // gettext: %s = player name
  this->save_file_chooser->set_title(_("save aiconfig of %s",
                                       this->player->player().name()));
} // void Players::Player::AiConfig::name_update()

/** update the description of the difficulty according to the player type
 **
 ** @bug       the un-sensitivity is not accepted when called before the container has been drawn (gtkmm bug?)
 **/
void
Players::Player::AiConfig::difficulty_description_update()
{
  // change the name of the difficulty button
  switch (this->player->player().type()) {
  case ::Player::Type::unset:
    DEBUG_ASSERTION(false,
                    "UI_GTKMM::Players::Player::AiConfig::type_change(type)\n"
                    "  type is 'unset'");
    break;
  case ::Player::Type::human:
  case ::Player::Type::ai:
    this->difficulty_selector->set_sensitive(true);
    this->container->set_sensitive(true);
    switch (this->player->player().type()) {
    case ::Player::Type::human:
      this->difficulty_label->set_label(_("card suggestion") + ": ");
      break;
    case ::Player::Type::ai:
      this->difficulty_label->set_label(_("difficulty") + ": ");
      break;
    default:
      break;
    } // switch (this->player->player().type())
    break;
  case ::Player::Type::ai_dummy:
  case ::Player::Type::ai_random:
    this->difficulty_label->set_label(_("no ai configuration") + ": ");
    this->difficulty_selector->set_sensitive(false);
    this->container->set_sensitive(false);
    break;
  } // switch (this->player->player().type())
} // void Players::Player::AiConfig::difficutly_description_update()

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
