/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "game_summary.h"

#include "ui.h"
#include "bug_report.h"

#include "../../party/party.h"
#include "../../party/rule.h"
#include "../../game/game_summary.h"
#include "../../utils/string.h"

#include <gtkmm/button.h>
#include <gtkmm/separator.h>
#include <gtkmm/label.h>
#include <gtkmm/main.h>
#include <gtkmm/treeview.h>
#include <gtkmm/scrolledwindow.h>

namespace UI_GTKMM_NS {

/** Constructor
 **
 ** @param    parent   parent object
 **/
GameSummary::GameSummary(Base* const parent) :
  Base(parent),
  Box()
{
  this->signal_realize().connect(sigc::mem_fun(*this, &GameSummary::init));
} // GameSummary::GameSummary(Base* parent)

/** destructor
 **/
GameSummary::~GameSummary() = default;

/** create all subelements
 **/
void
GameSummary::init()
{
  this->set_border_width(1 EM);
  this->set_spacing(1 EM);

  { // left column
    auto vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL));

    vbox->set_border_width(1 EX);
    vbox->set_spacing(1 EX);

    this->gamenumber = Gtk::manage(new Gtk::Label(_("Game: %u", 0u)));
    vbox->pack_start(*this->gamenumber, false, true);

    this->startplayer = Gtk::manage(new Gtk::Label(_("Startplayer: %s")));
    vbox->pack_start(*this->startplayer, false, true);

    this->winner = Gtk::manage(new Gtk::Label(_("Winner: no team")));
    vbox->pack_start(*this->winner, false, true);


    { // trickpoints
      this->trickpoints_list
        = Gtk::ListStore::create(this->trickpoints_model);
      this->trickpoints_treeview
        = Gtk::manage(new Gtk::TreeView(this->trickpoints_list));
      this->trickpoints_treeview->get_selection()->set_mode(Gtk::SELECTION_NONE);

      this->trickpoints_treeview->append_column(_("name"),
                                                this->trickpoints_model.name);
      this->trickpoints_treeview->append_column(_("trick points"),
                                                this->trickpoints_model.points);
      this->trickpoints_treeview->get_column_cell_renderer(1)->set_property("xalign", 0.5);
      this->trickpoints_treeview->append_column(_("team"),
                                                this->trickpoints_model.team);
      this->trickpoints_treeview->append_column(_("game points"),
                                                this->trickpoints_model.game_points);
      this->trickpoints_treeview->get_column_cell_renderer(3)->set_property("xalign", 0.5);
      this->trickpoints_treeview->append_column(_("party points"),
                                                this->trickpoints_model.party_points);
      this->trickpoints_treeview->get_column_cell_renderer(4)->set_property("xalign", 0.5);

      vbox->pack_start(*trickpoints_treeview, false, true);

      for (unsigned c = 0;
           c < this->trickpoints_treeview->get_columns().size();
           ++c) {
        auto column = this->trickpoints_treeview->get_column(c);
        column->set_cell_data_func(*column->get_first_cell(), sigc::mem_fun(*this, &GameSummary::set_points_cell_color));
      }
    } // trickpoints
    { // bock triggers
      this->bock_triggers_list
        = Gtk::ListStore::create(this->bock_triggers_model);
      this->bock_triggers_treeview
        = Gtk::manage(new Gtk::TreeView(this->bock_triggers_list));
      this->bock_triggers_treeview->get_selection()->set_mode(Gtk::SELECTION_NONE);

      this->bock_triggers_treeview->append_column(_("bock trigger"),
                                                  this->bock_triggers_model.name);

      vbox->pack_start(*bock_triggers_treeview, false, true);
    } // bock triggers

    this->pack_start(*vbox, Gtk::PACK_SHRINK);
  } // left column

  { // right column
    auto vbox = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL));

    vbox->set_border_width(1 EX);
    vbox->set_spacing(1 EX);

    { // game type
      this->type = Gtk::manage(new Gtk::Label(_("gametype: %t", _(GameType::normal))));

      vbox->pack_start(*this->type, false, true);
    } // game type


    { // points description
      this->points_description_list
        = Gtk::ListStore::create(this->points_description_model);
      this->points_description_treeview
        = Gtk::manage(new Gtk::TreeView(this->points_description_list));

      // ToDo: selecting a special point shows the corresponding trick
      // in the game review
      this->points_description_treeview->get_selection()->set_mode(Gtk::SELECTION_NONE);
      this->points_description_treeview->append_column(_("team"),
                                                       this->points_description_model.team);
      // ToDo: right-align the sign
      this->points_description_treeview->append_column(_("player"),
                                                       this->points_description_model.player);
      this->points_description_treeview->append_column(_("points"),
                                                       this->points_description_model.points);
      this->points_description_treeview->get_column_cell_renderer(2)->set_property("xalign", 0.5);
      this->points_description_treeview->append_column(_("description"),
                                                       this->points_description_model.description);

#ifdef POSTPONED
      // ToDo: make the scrolled window show the first four lines of the table
      auto scrolled_window = Gtk::manage(new Gtk::ScrolledWindow);
      scrolled_window->set_policy(Gtk::POLICY_NEVER,
                                  Gtk::POLICY_AUTOMATIC);
      scrolled_window->add(*(this->points_description_treeview));
      vbox->pack_start(*scrolled_window, true, true);
#else
      vbox->pack_start(*points_description_treeview, true, true);
      //this->pack_start(*(this->points_description_treeview), true, true);
#endif
    } // point description
    //this->pack_start(*(this->points_description));

    this->pack_start(*vbox);
  } // right column

  this->show_all_children();
  if (this->gameno != UINT_MAX)
    this->show();

  this->update();
} // void GameSummary::init()

/** set the gameno
 **
 ** @param    gameno   the new gameno
 **/
void
GameSummary::set_gameno(unsigned const gameno)
{
  this->gameno = gameno;
  if (this->gameno == UINT_MAX)
    this->game_summary = nullptr;
  else
    this->game_summary = &this->ui->party().game_summary(this->gameno);
  this->update();
} // void GameSummary::set_gameno(unsigned gameno)

/** set the game summary
 **
 ** @param    game_summary   new game summary
 **/
void
GameSummary::set_game_summary(::GameSummary const& game_summary)
{
  this->gameno = UINT_MAX;
  this->game_summary = &game_summary;
  this->update();
} // void GameSummary::set_game_summary(::GameSummary game_summary)

/** removes the game summary
 **/
void
GameSummary::remove_game_summary()
{
  this->game_summary = nullptr;
} // void GameSummary::remove_game_summary()

/** update (rewrite) all information
 **
 ** @todo   mark the winner gray
 ** @todo   if there is no 'player_of' don't show the column
 **/
void
GameSummary::update()
{
  if (!this->get_realized())
    return ;

  if (::game_status == GameStatus::programstart)
    return ;
  if (!this->game_summary) {
    this->hide();
    return ;
  }

#ifdef WORKAROUND
  // why?
  this->show();
#endif

  Party const& party = this->ui->party();

  this->gamenumber->set_label(_("Game: %u", this->gameno + 1));
  if (this->gameno == UINT_MAX)
    this->gamenumber->hide();
  else
    this->gamenumber->show();
  this->startplayer->set_label(_("Startplayer: %s",
                                 this->ui->party().players()[this->game_summary->party_player(this->game_summary->startplayer())].name()));

  if (this->game_summary->is_duty_solo()) {
    this->type->set_label(_("gametype: %t (duty solo)"s,
                            _(this->game_summary->game_type())));
  } else {
    this->type->set_label(_("gametype: %t"s,
                            _(this->game_summary->game_type())));
  }

  if (this->game_summary->winnerteam() == Team::noteam) {
    this->winner->set_label(_("Winner: no team"));
  } else {
    // gettext: %t = winnerteam, %u = points of the winnerteam, %t = looserteam, %u = points of the looserteam
    this->winner->set_label(_("Winner: team %t with %u points (%t: %u points)",

                              _(this->game_summary->winnerteam()),
                              this->game_summary->trick_points(this->game_summary->winnerteam()),
                              _(::opposite(this->game_summary->winnerteam())),
                              this->game_summary->trick_points(::opposite(this->game_summary->winnerteam()))
                             )
                           );
  }

  { // trickpoints
#ifdef WORKAROUND
    //if (!this->trickpoints_list->children().empty())
    //this->trickpoints_list->erase(this->trickpoints_list->get_iter("0"));
#else
    // after clearing the list, no row is added anymore
    this->trickpoints_list->clear();
#endif

    for (unsigned p = 0;
         p < this->game_summary->rule()(Rule::Type::number_of_players_in_game);
         p++) {
      auto const& player = party.players()[this->game_summary->party_player(p)];
      Gtk::ListStore::iterator i;
      if (p < this->trickpoints_list->children().size())
        i = this->trickpoints_list->get_iter(String::to_string(p));
      else
        i = this->trickpoints_list->append();
      auto row = *i;
      row[this->trickpoints_model.name] = player.name();
      if (this->game_summary->winnerteam() == Team::noteam)
        row[this->trickpoints_model.winner] = 0; // no winner
      else if (this->game_summary->team(player) == this->game_summary->winnerteam())
        row[this->trickpoints_model.winner] = 1; // winner
      else
        row[this->trickpoints_model.winner] = -1; // looser

      row[this->trickpoints_model.points] = this->game_summary->trick_points(player);
      row[this->trickpoints_model.team]
        = _(this->game_summary->team(player));
      if (this->game_summary->get_points(player))
        if (this->game_summary->points(player) > 0)
          row[this->trickpoints_model.game_points]
            = "+" + String::to_string(this->game_summary->points(player));
        else
          row[this->trickpoints_model.game_points]
            = String::to_string(this->game_summary->points(player));
      else
        row[this->trickpoints_model.game_points]
          = "";

      row[this->trickpoints_model.party_points]
        = (this->gameno == UINT_MAX
           ? party.pointsum(this->game_summary->party_player(p))
           : party.pointsum_till_game(this->gameno, this->game_summary->party_player(p)));
    } // for (p < this->game_summary->rule(Rule::Type::number_of_players_in_game)
  } // trickpoints

  { // points description
    if (this->game_summary->specialpoints().empty()) {
      this->points_description_treeview->hide();
    } else { // if !(this->game_summary->specialpoints().empty())
      this->points_description_treeview->show();

      this->points_description_list->clear();

      for (const auto & i : this->game_summary->specialpoints()) {
        auto const& specialpoint
          = i;
        auto row = *this->points_description_list->append();
        row[this->points_description_model.team]
          = (  (specialpoint.team == Team::noteam)
             ? ""s
             : _(specialpoint.team));

        if (specialpoint.player_get_no == UINT_MAX) {
          if (specialpoint.player_of_no == UINT_MAX)
            row[this->points_description_model.player] = string();
          else
            row[this->points_description_model.player]
              = party.players()[this->game_summary->party_player(specialpoint.player_of_no)].name();
        } else {
          row[this->points_description_model.player]
            = party.players()[this->game_summary->party_player(specialpoint.player_get_no)].name();
        }

        if (   !party.rule()(Rule::Type::knocking)
            && (specialpoint.type == Specialpoint::Type::no120_said) ) {
          if (specialpoint.team == Team::re)
            row[this->points_description_model.description]
              = _("Specialpoint::re said");
          else if (specialpoint.team == Team::contra)
            row[this->points_description_model.description]
              = _("Specialpoint::contra said");
        } else {
          row[this->points_description_model.description]
            = _(specialpoint);
        }


        { // show whether the points are added or subtracted
          bool add = true;
          if ((this->game_summary->winnerteam() == Team::re)
              || (this->game_summary->winnerteam() == Team::contra)) {
            add = (i.counting_team
                   == this->game_summary->winnerteam());
          } else { // if !(this->game_summary->winnerteam() == re, contra)
            if (specialpoint.counting_team == Team::re)
              add = true;
            else if (specialpoint.counting_team == Team::contra)
              add = false;
          } // if !(this->game_summary->winnerteam() != re, contra)
          if (add)
            row[this->points_description_model.points] = "+";
          else
            row[this->points_description_model.points] = "-";
        } // show whether the points are added or subtracted

        if (    (this->game_summary->winnerteam()
                 == Team::noteam)
            && (specialpoint.counting_team
                == Team::noteam))
          row[this->points_description_model.points] = " ";
        if (   (specialpoint.type
                == Specialpoint::Type::no120_said)
            && (this->game_summary->rule()(Rule::Type::announcement_re_doubles))
           ) {
          row[this->points_description_model.points]
            = "* " + String::to_string(specialpoint.value);
        } else if (specialpoint.type
                   == Specialpoint::Type::solo) {
          // a solo
          row[this->points_description_model.points]
            = "* " + String::to_string(specialpoint.value);
        } else if (specialpoint.type
                   == Specialpoint::Type::bock) {
          // a bock multiplicator
          row[this->points_description_model.points]
            = "* " + String::to_string(specialpoint.value);
        } else {
          // all other special points
          row[this->points_description_model.points]
            = (row[this->points_description_model.points]
               + " " + String::to_string(value(specialpoint.type))
              );

        }
      } // for (i < this->game_summary->specialpoints().size())
    } // if !(this->game_summary->specialpoints().empty())

  } // points description

  { // bock triggers
    if (this->game_summary->bock_triggers().empty()) {
      this->bock_triggers_treeview->hide();
    } else { // if !(this->game_summary->bock_triggers().empty())
      this->bock_triggers_treeview->show();

      this->bock_triggers_list->clear();

      for (auto i : this->game_summary->bock_triggers()) {
        auto row = *(this->bock_triggers_list->append());
        row[this->bock_triggers_model.name]
          = _(i);
      } // for (i < this->game_summary->specialpoints().size())
    } // if !(this->game_summary->specialpoints().empty())

  } // bock triggers
} // void GameSummary::update()

/** changes the color of the cell at 'iterator'
 ** mark the winner green and the loosers red
 **
 ** @param    cell_renderer   cell renderer to change
 ** @param    iterator   row
 **/
void
GameSummary::set_points_cell_color(Gtk::CellRenderer* const cell_renderer,
                                   Gtk::TreeModel::iterator const& iterator)
{
  if (!this->get_realized())
    return ;

  auto const row = *iterator;

  int const winner = row[this->trickpoints_model.winner];
  if (winner > 0) // winner
    cell_renderer->property_cell_background() = "#AFA"; // helles grün
  else if (winner < 0) // looser
    cell_renderer->property_cell_background() = "#FBB"; // helles rot
  else // no winner
    cell_renderer->property_cell_background() = "white";
} // void GameSummary::set_points_cell_color(Gtk::CellRenderer* cell_renderer, Gtk::TreeModel::iterator iterator)


/** the name of 'player' has changed
 **
 ** @param    player   the player with the changed name
 **/
void
GameSummary::name_changed(Player const& player)
{
  if (!this->get_realized())
    return ;

  this->update();
} // void GameSummary::name_changed(Player const& player)

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
