/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "gameinfo.announcement.h"

#include "ui.h"
#include "icons.h"

#include "../../player/player.h"
#include "../../game/game.h"
#include "../../party/rule.h"

#include <gtkmm/image.h>
#include <gtkmm/label.h>

namespace UI_GTKMM_NS {

/** constructor
 **
 ** @param    parent         the parent object
 ** @param    player         the player
 ** @param    announcement   the announcement made
 **/
GameInfoDialog::Information::Announcement::Announcement(Base* const parent,
                                                        Player const& player,
                                                        ::Announcement const
                                                        announcement) :
  GameInfoDialog::Information(parent, _("Window::GameInfo::announcement")),
  player(&player),
  announcement(announcement)
{

  if (   !this->ui->game().rule()(Rule::Type::knocking)
      && (   (announcement == ::Announcement::no120)
          || (announcement == ::Announcement::reply) ) ) {
    this->ui->icons->change_managed(this->icon, player.team());
    this->title = _(player.team());
  } else {
    this->ui->icons->change_managed(this->icon, player.announcement());
    this->title = _(announcement);
  }

  this->init();
} // GameInfoDialog::Information::Announcement::Announcement(Base parent, Game game)

/** destructor
 **/
GameInfoDialog::Information::Announcement::~Announcement() = default;

/** update all texts
 **/
void
GameInfoDialog::Information::Announcement::update_texts()
{
  if (   !this->ui->game().rule()(Rule::Type::knocking)
      && (   (this->announcement == ::Announcement::no120)
          || (this->announcement == ::Announcement::reply) ) ) {
    this->label->set_label(_("%s announces '%t'.",
                             this->player->name(),
                             _(this->player->team())
                            ));

  } else {
    this->label->set_label(_("%s announces '%t'.",
                             this->player->name(),
                             _(this->announcement)
                            ));
  }
} // void GameInfoDialog::Information::Announcement::update_texts()

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
