/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "gameinfo.h"

#include "ui.h"
#include "icons.h"

#include <gtkmm/image.h>
#include <gtkmm/label.h>

namespace UI_GTKMM_NS {

/** constructor
 **
 ** @param    parent   the parent object
 ** @param    title    the title
 **/
GameInfoDialog::Information::Information(Base* const parent,
                                         string  title) :
  Base(parent),
  Gtk::Box(Gtk::ORIENTATION_VERTICAL, 1 EX),
  title(std::move(title)),
  icon(this->ui->icons->new_managed_image()),
  label(Gtk::manage(new Gtk::Label("")))
{
  this->set_border_width(1 EX);
} // GameInfoDialog::Information::Information(Base* parent, Translation title)

/** destructor
 **/
GameInfoDialog::Information::~Information() = default;

/** initializes the information
 **/
void
GameInfoDialog::Information::init()
{
  if (this->icon)
    this->add(*this->icon);
  if (this->label)
    this->add(*this->label);

  this->show_all();

  this->update_texts();
} // virtual void GameInfoDialog::Information::init()

/** the name of 'player' has changed
 **
 ** @param    player   player whose name has changed
 **/
void
GameInfoDialog::Information::name_changed(Player const& player)
{
  this->update_texts();
} // void GameInfoDialog::Information::name_changed(Player player)

/** update the texts
 **/
void
GameInfoDialog::Information::update_texts()
{ }

/** @return   whether the information is blocking the gameplay
 **/
bool
GameInfoDialog::Information::is_blocking() const
{
  return true;
} // virtual bool GameInfoDialog::Information::is_blocking() const

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
