/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#ifdef USE_UI_GTKMM

#pragma once

#include "htin.h"
#include "icons.h"
#include "../../card/card.h"

namespace UI_GTKMM_NS {

/**
 ** the graphic representation of a hand
 **
 ** @todo   test overlapping of icons
 ** @todo   use of 'Gdk::Point' and 'Gdk::Rectangle'
 **/
class Icongroup : public HTIN {
  public:
    /**
     ** The status for checking changes (for redrawing)
     **/
    struct Status {
      Icons::Type team_icon;
      bool team_guessed;
      Announcement announcement;
      bool next_announcement;
      Card::Color trumpcolor;
      bool swines;
      bool hyperswines;
    }; // struct Status
    /**
     ** The geometry of the icongroup
     **/
    struct Geometry : public Table::Part::Geometry {
      function<bool()> flip_x = [](){ return false; };
      Outline outline() const override;
    }; // struct Geometry : public Table::Part::Geometry
  public:
    Icongroup(Table* table, Position position);
    ~Icongroup() override;

    bool changed() const override;
    Geometry geometry() const;
    Outline outline() const override;

    void draw(Cairo::RefPtr<::Cairo::Context> cr) override;
    void draw_team(Cairo::RefPtr<::Cairo::Context> cr);
    void draw_announcement(Cairo::RefPtr<::Cairo::Context> cr);
    void draw_swines(Cairo::RefPtr<::Cairo::Context> cr);

    bool mouse_over_next_announcement() const;

    Status current_status() const;
  private:
    Status last_status_;

  private:
    Icongroup() = delete;
    Icongroup(Icongroup const&) = delete;
    Icongroup& operator=(Icongroup const&) = delete;
}; // class Icongroup : public HTIN

bool operator!=(Icongroup::Status const& lhs, Icongroup::Status const& rhs);

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
