/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "icons.h"

#include "ui.h"
#include "cards.h"
#include "splash_screen.h" // ToDo: remove it

#include "../status_message.h"

#include "../../party/party.h"
#include "../../party/rule.h"
#include "../../game/game.h"
#include "../../misc/setting.h"

#include <gtkmm/image.h>

namespace UI_GTKMM_NS {
size_t const Icons::number_of_types = static_cast<size_t>(Type::last) + 1;

/** -> result
 **
 ** @param    gametype   the gametype
 **
 ** @return   icon type for 'gametype'
 **/
Icons::Type
  Icons::type(GameType const gametype)
  {
    switch(gametype) {
    case GameType::normal:
      return Type::re;
    case GameType::thrown_nines:
      return Type::thrown_nines;
    case GameType::thrown_kings:
      return Type::thrown_kings;
    case GameType::thrown_nines_and_kings:
      return Type::thrown_nines_and_kings;
    case GameType::thrown_richness:
      return Type::thrown_richness;
    case GameType::redistribute:
      return Type::redistribute;
    case GameType::poverty:
      if (::game_status == GameStatus::game_reservation) {
        auto const& player = ::party->game().players().current_player();
        if (player.type() == Player::Type::human) {
          switch (player.hand().count_trumps()) {
          case 0:
            return Type::poverty_trumps_0;
          case 1:
            return Type::poverty_trumps_1;
          case 2:
            return Type::poverty_trumps_2;
          case 3:
          case 4:
          case 5:
            return Type::poverty_trumps_3;
          case UINT_MAX:
            return Type::poverty;
          default:
            DEBUG_ASSERTION(false,
                            "Icons::icon(gametype):\n"
                            "  gametype: poverty\n"
                            "  number of poverty cards for player " << player.no() << ' ' << player.name() << " invalid: "
                            << player.hand().count_trumps());
            return Type::poverty;
          } // switch (::party->game().poverty_trumpno_returned())
        } else {
          return Type::poverty;
        }
      }
      if (::game_status == GameStatus::game_poverty_shift) {
        return Type::poverty;
      }

      switch (::party->game().poverty().returned_trumpno()) {
      case 0:
        return Type::poverty_trumps_0;
      case 1:
        return Type::poverty_trumps_1;
      case 2:
        return Type::poverty_trumps_2;
      case 3:
      case 4:
      case 5:
        return Type::poverty_trumps_3;
      case UINT_MAX:
        return Type::poverty;
      default:
        DEBUG_ASSERTION(false,
                        "Icons::icon(gametype):\n"
                        "  gametype: poverty\n"
                        "  number of poverty cards invalid: "
                        << ::party->game().poverty().returned_trumpno());
        return Type::poverty;
      } // switch (::party->game().poverty().returned_trumpno())
    case GameType::fox_highest_trump:
      return Type::fox_highest_trump;
    case GameType::genscher:
      return Type::genscher;
    case GameType::marriage:
      return Type::marriage;
    case GameType::marriage_solo:
    case GameType::marriage_silent:
      return Type::marriage_solo;
    case GameType::solo_diamond:
      return Type::solo_diamond;
    case GameType::solo_jack:
      return Type::solo_jack;
    case GameType::solo_queen:
      return Type::solo_queen;
    case GameType::solo_king:
      return Type::solo_king;
    case GameType::solo_queen_jack:
      return Type::solo_queen_jack;
    case GameType::solo_king_jack:
      return Type::solo_king_jack;
    case GameType::solo_king_queen:
      return Type::solo_king_queen;
    case GameType::solo_koehler:
      return Type::solo_koehler;
    case GameType::solo_club:
      return Type::solo_club;
    case GameType::solo_heart:
      return Type::solo_heart;
    case GameType::solo_spade:
      return Type::solo_spade;
    case GameType::solo_meatless:
      return Type::solo_meatless;
    default:
      DEBUG_ASSERTION(false,
                      "Icons::type(gametype):\n"
                      "  unknown gametype '" << gametype << "'");
      return Type::none;
    } // switch(gametype)
  } // static Icons::Type Icons::type(GameType gametype)

/** -> result
 **
 ** @param    marriage_selector   the marriage selector
 **
 ** @return   icon type for 'marriage_selector'
 **/
Icons::Type
  Icons::type(MarriageSelector const marriage_selector)
  {
    switch(marriage_selector) {
    case MarriageSelector::silent:
      return Type::marriage_solo;
    case MarriageSelector::team_set:
      return Type::marriage;
    case MarriageSelector::first_foreign:
      return Type::marriage_foreign;
    case MarriageSelector::first_trump:
      return Type::marriage_trump;
    case MarriageSelector::first_color:
      return Type::marriage_color;
    case MarriageSelector::first_club:
      return Type::marriage_club;
    case MarriageSelector::first_spade:
      return Type::marriage_spade;
    case MarriageSelector::first_heart:
      return Type::marriage_heart;
    } // switch (marriage_selector)
    DEBUG_ASSERTION(false,
                    "Icons::type(marriage_selector):\n"
                    "  unknown marriage selecter '" << marriage_selector << "'");
    return Type::none;
  } // static Icons::Type Icons::type(MarriageSelector marriage_selector)

/** -> result
 **
 ** @param    team      the team
 **
 ** @return   icon type for 'team'
 **/
Icons::Type
  Icons::type(Team const team)
  {
    switch(team) {
    case Team::re:
      return Type::re;
    case Team::contra:
      return Type::contra;
    default:
      return Type::none;
    } // switch(team)
  } // static Icons::Type Icons::type(Team team)

/** -> result
 **
 ** @param    announcement   the announcement
 **
 ** @return   icon type for 'announcement'
 **/
Icons::Type
  Icons::type(Announcement const announcement)
  {
    switch(announcement) {
    case Announcement::noannouncement:
      DEBUG_ASSERTION(false,
                      "Icon::icon(announcement, rotation):\n"
                      "  invalid announcement 'noannouncement'");
      break;
    case Announcement::no120:
      return Type::no_120;
    case Announcement::no90:
      return Type::no_90;
    case Announcement::no60:
      return Type::no_60;
    case Announcement::no30:
      return Type::no_30;
    case Announcement::no0:
      return Type::no_0;
    case Announcement::reply:
#ifdef POSTPONED
      DEBUG_ASSERTION(false,
                      "Icon::icon(announcement, rotation):\n"
                      "  invalid announcement 'reply'");
      return Type::none;
#else
      return Type::no_120;
#endif
    case Announcement::no120_reply:
      return Type::no_120_reply;
    case Announcement::no90_reply:
      return Type::no_90_reply;
    case Announcement::no60_reply:
      return Type::no_60_reply;
    case Announcement::no30_reply:
      return Type::no_30_reply;
    case Announcement::no0_reply:
      return Type::no_0_reply;
    default:
      DEBUG_ASSERTION(false,
                      "Icons::type(announcement):\n"
                      "  unknown announcement '" << announcement << "'");
      return Type::none;
    } // switch(announcement)
    return Type::none;
  } // static Icons::Type Icons::type(Announcement announcement)

/** -> result
 **
 ** @param    color      the color
 **
 ** @return   swines icon type for 'color'
 **/
Icons::Type
  Icons::type_swines(Card::Color const color)
  {
    switch(color) {
    case Card::club:
      return Type::swines_club;
    case Card::spade:
      return Type::swines_spade;
    case Card::heart:
      return Type::swines_heart;
    case Card::diamond:
      return Type::swines_diamond;
    default:
      DEBUG_ASSERTION(false,
                      "Icon::swines(color)\n"
                      "  Color is not valid\n");
      return Type::none;
    } // switch(color)
  } // static Icons::Type Icons::type_swines(Card::Color color)

/** -> result
 **
 ** @param    color      the color
 **
 ** @return   hyperswines icon type for 'color'
 **/
Icons::Type
  Icons::type_hyperswines(Card::Color const color)
  {
    switch(color) {
    case Card::club:
      return (this->ui->party().rule()(Rule::Type::with_nines)
              ? Type::hyperswines_club
              : Type::hyperswines_king_club);
    case Card::spade:
      return (this->ui->party().rule()(Rule::Type::with_nines)
              ? Type::hyperswines_spade
              : Type::hyperswines_king_spade);
    case Card::heart:
      return (this->ui->party().rule()(Rule::Type::with_nines)
              ? Type::hyperswines_heart
              : Type::hyperswines_king_heart);
    case Card::diamond:
      return (this->ui->party().rule()(Rule::Type::with_nines)
              ? Type::hyperswines_diamond
              : Type::hyperswines_king_diamond);
    default:
      DEBUG_ASSERTION(false,
                      "Icon::hyperswines(color)\n"
                      "  Color is not valid\n");
      return Type::none;
    } // switch(color)
  } // static Icons::Type Icons::type_hyperswines(Card::Color color)

/** -> result
 **
 ** @param    color      the color
 **
 ** @return   swines-hyperswines icon type for 'color'
 **/
Icons::Type
  Icons::type_swines_hyperswines(Card::Color const color)
  {
    switch(color) {
    case Card::club:
      return (this->ui->party().rule()(Rule::Type::with_nines)
              ? Type::swines_hyperswines_club
              : Type::swines_hyperswines_king_club);
    case Card::spade:
      return (this->ui->party().rule()(Rule::Type::with_nines)
              ? Type::swines_hyperswines_spade
              : Type::swines_hyperswines_king_spade);
    case Card::heart:
      return (this->ui->party().rule()(Rule::Type::with_nines)
              ? Type::swines_hyperswines_heart
              : Type::swines_hyperswines_king_heart);
    case Card::diamond:
      return (this->ui->party().rule()(Rule::Type::with_nines)
              ? Type::swines_hyperswines_diamond
              : Type::swines_hyperswines_king_diamond);
    default:
      DEBUG_ASSERTION(false,
                      "Icon::swines_hyperswines(color)\n"
                      "  Color is not valid\n");
      return Type::none;
    } // switch(color)
  } // static Icons::Type Icons::type_swines_hyperswines(Card::Color const color)

/** constructor
 **
 ** @param    parent   the parent object
 **/
Icons::Icons(Base* const parent) :
  Base(parent)
{
  this->load();
} // Icons::Icons(Base* parent)

/** destructor
 **/
Icons::~Icons() = default;

/** @return   the scaling
 **/
double
Icons::scaling() const
{
  return this->ui->cards->scaling();
} // double Icons::scaling() const

/** update the scaling
 **/
void
Icons::update_scaling()
{
  // the scaling value is already updated in 'Cards'

  // set the scaling of the 'ScaledPixbufRotations'
  for (auto& i : this->icon_)
    i.set_scaling(this->scaling());

  this->update_max_values();
} // void Icons::update_scaling()

/** update the maximal width and height
 **/
void
Icons::update_max_values()
{
  this->max_width_ = 0;
  for (auto const& i : this->icon_)
    if (i.get_width() > this->max_width_)
      this->max_width_ = i.get_width();

  this->max_height_ = 0;
  for (auto const& i : this->icon_)
    if (i.get_height() > this->max_height_)
      this->max_height_ = i.get_height();

  return ;
} // void Icons::update_max_values()

/** -> result
 **
 ** @param    rotation   the rotation
 **
 ** @return   maximal icon width
 **/
int
Icons::max_width(Rotation const rotation) const
{
  if (   (rotation == Rotation::left)
      || (rotation == Rotation::right))
    return this->max_height(clockwise(rotation));

  return this->max_width_;
} // int Icons::max_width(Rotation rotation = up) const

/** -> result
 **
 ** @param    rotation   the rotation
 **
 ** @return   maximal icon height
 **/
int
Icons::max_height(Rotation const rotation) const
{
  if (   (rotation == Rotation::left)
      || (rotation == Rotation::right))
    return this->max_width(clockwise(rotation));

  return this->max_height_;
} // int Icons::max_height(Rotation rotation = up) const

/** -> result
 **
 ** @param    icons      the icons to look at
 ** @param    rotation   the rotation
 **
 ** @return   maximal icon width of the icons
 **/
int
Icons::max_width(vector<Glib::RefPtr<Gdk::Pixbuf>> const& icons,
                 Rotation const rotation) const
{
  if (   (rotation == Rotation::left)
      || (rotation == Rotation::right))
    return this->max_height(icons, clockwise(rotation));

  int width = 0;

  for (auto const& i : icons)
    if (i->get_width() > width)
      width = i->get_width();

  return width;
} // int Icons::max_width(vector<Glib::RefPtr<Gdk::Pixbuf>> icons, Rotation rotation = up) const

/** -> result
 **
 ** @param    icons      the icons to look at
 ** @param    rotation   the rotation
 **
 ** @return   maximal icon height of the icons
 **/
int
Icons::max_height(vector<Glib::RefPtr<Gdk::Pixbuf>> const& icons,
                  Rotation const rotation) const
{
  if (   (rotation == Rotation::left)
      || (rotation == Rotation::right))
    return this->max_width(icons, clockwise(rotation));

  int height = 0;

  for (auto const& i : icons)
    if (i->get_height() > height)
      height = i->get_height();

  return height;
} // int Icons::max_height(vector<Glib::RefPtr<Gdk::Pixbuf>> icons, Rotation rotation = up) const

/** -> result
 **
 ** @param    type      the type
 ** @param    rotation   the rotation
 **
 ** @return   the pixbuf of 'icon'
 **   (with rotation 'rotation')
 **/
Glib::RefPtr<Gdk::Pixbuf> const&
Icons::icon(Type const type, Rotation const rotation)
{
  return this->icon_[static_cast<size_t>(type)][rotation];
} // Glib::RefPtr<Gdk::Pixbuf> Icons::icon(Type type, Rotation rotation = up)

/** -> result
 **
 ** @param    type   icon type
 **   (default: none)
 ** @param    scaling   additional scaling
 **   (default: 1)
 **
 ** @return   new image with the icon 'type' that is managed
 **   (that is size and theme)
 **/
Gtk::Image*
Icons::new_managed_image(Type const type, double const scaling)
{
  auto icon = this->icon(type);
  auto image = Gtk::manage(new Gtk::Image(icon));
  image->set_data("type", new Type(type));
  image->set_data("scaling", new double(scaling));
  if (scaling != 1)
    image->set(icon->scale_simple(static_cast<int>(icon->get_width() * scaling),
                                  static_cast<int>(icon->get_height() * scaling),
                                  Gdk::INTERP_TILES));

  image->signal_unrealize().connect(sigc::bind<Gtk::Image const* const>(sigc::mem_fun(*this, &Icons::remove_managed), image));

  this->managed_image_.push_back(image);

  return image;
} // Gtk::Image* Icons::new_managed_image(Type type = none, double scaling = 1)

/** changes the type of 'image'
 **
 ** @param    image   image to change the type of
 ** @param    type   new type
 **/
void
Icons::change_managed(Gtk::Image* const image, Type const type)
{
  DEBUG_ASSERTION(image->get_data("type"),
                  "Icons::change_managed:\n"
                  "  image has no data 'type'");
  // ToDo: Test whether the image is in the vector

  delete static_cast<Type*>(image->steal_data("type"));
  image->set_data("type", new Type(type));

  if (type == Type::none)
    image->clear();
  else
    image->set(this->icon(type));

  return ;
} // void Icons::change_managed(Gtk::Image* image, Type type)

/** changes the scaling of 'image'
 **
 ** @param    image   image to change the type of
 ** @param    scaling   new scaling
 **/
void
Icons::change_scaling(Gtk::Image* const image, double const scaling)
{
  DEBUG_ASSERTION(image->get_data("scaling"),
                  "Icons::change_scaling:\n"
                  "  image has no data 'scaling'");
  // ToDo: Test whether the image is in the vector

  delete static_cast<double*>(image->steal_data("scaling"));
  image->set_data("scaling", new double(scaling));

  auto const type = *(static_cast<Type*>(image->get_data("type")));
  if (type == Type::none)
    return ;

  if (scaling == 1)
    image->set(this->icon(type));
  else {
    auto icon = this->icon(type);
    image->set(icon->scale_simple(static_cast<int>(icon->get_width() * scaling),
                                  static_cast<int>(icon->get_height() * scaling),
                                  Gdk::INTERP_TILES));
  }

  return ;
} // void Icons::change_scaling(Gtk::Image* image, double scaling)

/** remove 'image' from the managed image list
 **
 ** @param    image   image to remove
 **/
void
Icons::remove_managed(Gtk::Image const* const image)
{
  for (auto i = this->managed_image_.begin();
       i != this->managed_image_.end();
      )
    if (*i == image) {
      delete static_cast<Type*>((*i)->steal_data("type"));
      delete static_cast<double*>((*i)->steal_data("scaling"));
      i = this->managed_image_.erase(i);
      break;
    } else {
      ++i;
    }

  return ;
} // void Icons::remove_managed(Gtk::Image const* image)

/** update the managed images
 **/
void
Icons::update_managed()
{
  for (auto& i : this->managed_image_) {
    auto const type = *(static_cast<Type*>(i->get_data("type")));
    if (type == Type::none)
      continue;

    auto const scaling = *(static_cast<double*>(i->get_data("scaling")));
    if ( (scaling == 1)
        || (scaling == 0) )
      i->set(this->icon(type));
    else {
      auto icon = this->icon(type);
      i->set(icon->scale_simple(static_cast<int>(icon->get_width() * scaling),
                                static_cast<int>(icon->get_height() * scaling),
                                Gdk::INTERP_TILES));
    }
  }

  return ;
} // void Icons::update_managed()

/** load the icons
 **/
void
Icons::load()
{
  if (Glib::file_test(::setting.path(Setting::Type::cardset) + "/"
                      + ::setting(Setting::Type::icons_directory) + "/"
                      + "no_120"
                      + "." + ::setting(Setting::Type::graphic_extension),
                      Glib::FILE_TEST_IS_REGULAR)) {

    if (this->icon_.empty()) {
      // gettext: %s = cardset, %s = path to FreeDokorc
      this->ui->error(_("Error at loading the icons from the cardset %s.\n"
                        "FreeDokorc: (%s).",
                        ::setting(Setting::Type::cardset),
                        ::setting.path(Setting::Type::settings_file)
                       ));
    } else { // if !(this->icon_.empty())
      this->ui->information(_("Error at loading the icons from the cardset %s.\n"
                              "Keeping the old icons.",
                              ::setting(Setting::Type::cardset)),
                            InformationType::warning);
    } // if !(this->icon_.empty())

    return ;
  } // if (directory is valid)

  StatusMessages sm(*this->ui);
  if (   (::game_status != GameStatus::programstart)
      && (::game_status != GameStatus::party_initial_loaded) ) {
    sm.add(_("loading the icons"));
  } // if (::game_status != GameStatus::programstart)

  vector<Gdk::ScaledPixbufRotations> icon_new;
  for (unsigned type = 0; type < number_of_types; type++) {
    sm.add(_("loading the icons (%u/%u)", type+1, static_cast<unsigned>(number_of_types)));
    icon_new.push_back(this->load(static_cast<Type>(type)));
    if (   (::game_status == GameStatus::programstart)
        || (::game_status == GameStatus::party_initial_loaded) )
      this->ui->add_progress(1.0 / number_of_types);
  } // for (n \in names)

  this->icon_ = icon_new;

  this->update_scaling();
  this->update_managed();
  this->update_max_values();

  return ;
} // void Icons::load()

/** load the icon from the file
 **
 ** @param    name_a   name of the file
 ** @param    name_b   alternate name of the file
 **
 ** @return   loaded or empty icon
 **/
Gdk::ScaledPixbufRotations
Icons::load_from_file(string const& name_a, string const& name_b)
{
  auto icon = Gdk::ScaledPixbufRotations(::setting.path(Setting::Type::iconset) + "/"
                                         + name_a + "."
                                         + ::setting(Setting::Type::graphic_extension));
  if (icon)
    return icon;
  if (!name_b.empty())
    icon = Gdk::ScaledPixbufRotations(::setting.path(Setting::Type::iconset) + "/"
                                      + name_b + "."
                                      + ::setting(Setting::Type::graphic_extension));
  if (icon)
    return icon;

  // fallback in the cardset directory
  icon = Gdk::ScaledPixbufRotations(::setting.path(Setting::Type::cardset) + "/"
                                    + ::setting.value(Setting::Type::icons_from_cardset_directory) + "/"
                                    + name_a + "."
                                    + ::setting(Setting::Type::graphic_extension));
  if (icon)
    return icon;
  if (!name_b.empty())
    icon = Gdk::ScaledPixbufRotations(::setting.path(Setting::Type::cardset) + "/"
                                      + ::setting.value(Setting::Type::icons_from_cardset_directory) + "/"
                                      + name_b + "."
                                      + ::setting(Setting::Type::graphic_extension));
  return icon;
} // Gdk::ScaledPixbufRotations Icons::load_from_file(string name_a, string name_b = "")

/** load the icon from the file or construct it
 **
 ** @param    name     name of the file
 ** @param    cards    cards to construct from
 **
 ** @return   loaded or empty icon
 **/
Gdk::ScaledPixbufRotations
Icons::load_from_file_or_construct(string const& name,
                                   vector<Card> const& cards)
{
  auto icon = this->load_from_file(name);
  if (icon)
    return icon;
  return this->construct(cards);
} // Gdk::ScaledPixbufRotations Icons::load_from_file_or_construct(string name, vector<Card> cards)

/** load the icon from the file or construct it
 **
 ** @param    name_a   name of the file
 ** @param    name_b   alternative name
 ** @param    cards    cards to construct from
 **
 ** @return   loaded or empty icon
 **/
Gdk::ScaledPixbufRotations
Icons::load_from_file_or_construct(string const& name_a,
                                   string const& name_b,
                                   vector<Card> const& cards)
{
  auto icon = this->load_from_file(name_a, name_b);
  if (icon)
    return icon;
  return this->construct(cards);
} // Gdk::ScaledPixbufRotations Icons::load_from_file_or_construct(string name, vector<Card> cards)

/** load or construct the icons
 **
 ** @param    type      the icon type
 **
 ** @return   loaded/constructed icon
 **/
Gdk::ScaledPixbufRotations
Icons::load(Type const type)
{
  Gdk::ScaledPixbufRotations icon;

  switch(type) {
  case Type::none:
    return Gdk::ScaledPixbufRotations();
  case Type::re:
    return this->load_from_file_or_construct("re",
                                             {Card::club_queen});
  case Type::contra:
    // ToDo: strike out the icon
    return this->load_from_file_or_construct("contra",
                                             {Card::club_queen});
  case Type::thrown_nines:
    return this->load_from_file_or_construct("thrown_nines",
                                             {Card::club_nine,
                                             Card::spade_nine,
                                             Card::heart_nine,
                                             Card::diamond_nine});
  case Type::thrown_kings:
    return this->load_from_file_or_construct("thrown_kings",
                                             {Card::club_king,
                                             Card::spade_king,
                                             Card::heart_king,
                                             Card::diamond_king});
  case Type::thrown_nines_and_kings:
    return this->load_from_file_or_construct("thrown_nines_and_kings",
                                             {Card::club_king,
                                             Card::spade_king,
                                             Card::heart_nine,
                                             Card::diamond_nine });
  case Type::thrown_richness:
    return this->load_from_file_or_construct("thrown_richness",
                                             {Card::club_ten,
                                             Card::spade_ten,
                                             Card::heart_ten,
                                             Card::diamond_ten});
  case Type::redistribute:
    //return this->load_from_file("redistribute");
    return this->load_from_file_or_construct("redistribute",
                                             {Card(), Card(), Card()});

  case Type::fox_highest_trump:
    return this->load_from_file_or_construct("fox_highest_trump",
                                             {Card::diamond_ace,
                                             Card::diamond_ten,
                                             Card::diamond_king,
                                             Card::diamond_nine});
  case Type::marriage:
    return this->load_from_file_or_construct("marriage",
                                             {Card::club_queen,
                                             Card::club_queen});
  case Type::marriage_solo:
    return this->load_from_file_or_construct("marriage.solo", "marriage",
                                             {Card::club_queen,
                                             Card::club_queen});
  case Type::marriage_foreign:
    return this->load_from_file_or_construct("marriage.foreign",
                                             {Card::club_queen,
                                             Card::club_queen,
                                             Card::diamond_jack,
                                             Card::spade_ace});
  case Type::marriage_trump:
    return this->load_from_file_or_construct("marriage.trump",
                                             {Card::club_queen,
                                             Card::club_queen,
                                             Card::diamond_jack,
                                             Card::spade_ace});
  case Type::marriage_color:
    return this->load_from_file_or_construct("marriage.color",
                                             {Card::club_queen,
                                             Card::club_queen,
                                             Card::heart_ace,
                                             Card::spade_ace});
  case Type::marriage_club:
    return this->load_from_file_or_construct("marriage.club",
                                             {Card::club_queen,
                                             Card::club_queen,
                                             Card::club_ace,
                                             Card::club_ace});
  case Type::marriage_spade:
    return this->load_from_file_or_construct("marriage.spade",
                                             {Card::club_queen,
                                             Card::club_queen,
                                             Card::spade_ace,
                                             Card::spade_ace});
  case Type::marriage_heart:
    return this->load_from_file_or_construct("marriage.heart",
                                             {Card::club_queen,
                                             Card::club_queen,
                                             Card::heart_ace,
                                             Card::heart_ace});
  case Type::marriage_partner: {
    auto icon = this->load_from_file("marriage.partner");
    if (!icon)
      icon = this->load(Type::re);
    return icon;
  }
  case Type::genscher:
    return this->load_from_file_or_construct("genscher",
                                             {Card::diamond_king,
                                             Card::diamond_king});
  case Type::poverty:
    return this->load_from_file_or_construct("poverty",
                                             {Card::heart_queen,
                                             Card::spade_jack,
                                             Card::diamond_ten});
  case Type::poverty_trumps_0:
    return this->load_from_file_or_construct("poverty.0",
                                             {Card::club_ace,
                                             Card::heart_king,
                                             Card::spade_ten});
  case Type::poverty_trumps_1:
    return this->load_from_file_or_construct("poverty.1",
                                             {Card::spade_queen,
                                             Card::heart_nine,
                                             Card::club_king});
  case Type::poverty_trumps_2:
    return this->load_from_file_or_construct("poverty.2",
                                             {Card::diamond_queen,
                                             Card::spade_jack,
                                             Card::club_ace});
  case Type::poverty_trumps_3:
    return this->load_from_file_or_construct("poverty.3",
                                             {Card::heart_queen,
                                             Card::club_jack,
                                             Card::diamond_king});
  case Type::poverty_partner: {
    auto icon = this->load_from_file("poverty.partner");
    if (!icon)
      icon = this->load(Type::re);
    return icon;
  }
  case Type::swines_club:
    return this->load_from_file_or_construct("swines.club", "swines",
                                             {Card::club_ace,
                                             Card::club_ace});
  case Type::swines_spade:
    return this->load_from_file_or_construct("swines.spade", "swines",
                                             {Card::spade_ace,
                                             Card::spade_ace});
  case Type::swines_heart:
    return this->load_from_file_or_construct("swines.heart", "swines",
                                             {Card::heart_ace,
                                             Card::heart_ace});
  case Type::swines_diamond:
    return this->load_from_file_or_construct("swines.diamond", "swines",
                                             {Card::diamond_ace,
                                             Card::diamond_ace});
  case Type::hyperswines_club:
    return this->load_from_file_or_construct("hyperswines.club", "hyperswines",
                                             {Card::club_nine,
                                             Card::club_nine});
  case Type::hyperswines_spade:
    return this->load_from_file_or_construct("hyperswines.spade", "hyperswines",
                                             {Card::spade_nine,
                                             Card::spade_nine});
  case Type::hyperswines_heart:
    return this->load_from_file_or_construct("hyperswines.heart", "hyperswines",
                                             {Card::heart_nine,
                                             Card::heart_nine});
  case Type::hyperswines_diamond:
    return this->load_from_file_or_construct("hyperswines.diamond", "hyperswines",
                                             {Card::diamond_nine,
                                             Card::diamond_nine});
  case Type::hyperswines_king_club:
    return this->load_from_file_or_construct("hyperswines.king.club",
                                             "hyperswines",
                                             {Card::club_king,
                                             Card::club_king});
  case Type::hyperswines_king_spade:
    return this->load_from_file_or_construct("hyperswines.king.spade",
                                             "hyperswines",
                                             {Card::spade_king,
                                             Card::spade_king});
  case Type::hyperswines_king_heart:
    return this->load_from_file_or_construct("hyperswines.king.heart",
                                             "hyperswines",
                                             {Card::heart_king,
                                             Card::heart_king});
  case Type::hyperswines_king_diamond:
    return this->load_from_file_or_construct("hyperswines.king.diamond",
                                             "hyperswines",
                                             {Card::diamond_king,
                                             Card::diamond_king});
  case Type::swines_hyperswines_club:
    return this->load_from_file_or_construct("swines-hyperswines.club",
                                             "swines-hyperswines",
                                             {Card::club_ace,
                                             Card::club_nine});
  case Type::swines_hyperswines_spade:
    return this->load_from_file_or_construct("swines-hyperswines.spade",
                                             "swines-hyperswines",
                                             {Card::spade_ace,
                                             Card::spade_nine});
  case Type::swines_hyperswines_heart:
    return this->load_from_file_or_construct("swines-hyperswines.heart",
                                             "swines-hyperswines",
                                             {Card::heart_ace,
                                             Card::heart_nine});
  case Type::swines_hyperswines_diamond:
    return this->load_from_file_or_construct("swines-hyperswines.diamond",
                                             "swines-hyperswines",
                                             {Card::diamond_ace,
                                             Card::diamond_nine});
  case Type::swines_hyperswines_king_club:
    return this->load_from_file_or_construct("swines-hyperswines.king.club",
                                             "swines-hyperswines",
                                             {Card::club_ace,
                                             Card::club_king});
  case Type::swines_hyperswines_king_spade:
    return this->load_from_file_or_construct("swines-hyperswines.king.spade",
                                             "swines-hyperswines",
                                             {Card::spade_ace,
                                             Card::spade_king});
  case Type::swines_hyperswines_king_heart:
    return this->load_from_file_or_construct("swines-hyperswines.king.heart",
                                             "swines-hyperswines",
                                             {Card::heart_ace,
                                             Card::heart_king});
  case Type::swines_hyperswines_king_diamond:
    return this->load_from_file_or_construct("swines-hyperswines.king.diamond",
                                             "swines-hyperswines",
                                             {Card::diamond_ace,
                                             Card::diamond_king});
  case Type::dollen:
    return this->load_from_file_or_construct("dollen",
                                             {Card::heart_ten,
                                             Card::heart_ten});

  case Type::doppelkopf:
    return this->load_from_file_or_construct("doppelkopf",
                                             {Card::club_ace,
                                             Card::spade_ten,
                                             Card::diamond_ace,
                                             Card::heart_ten});
  case Type::solo_club:
    return this->load_from_file_or_construct("solo.club",
                                             {Card::spade_queen,
                                             Card::heart_jack,
                                             Card::club_ace,
                                             Card::club_king});
  case Type::solo_spade:
    return this->load_from_file_or_construct("solo.spade",
                                             {Card::club_queen,
                                             Card::diamond_jack,
                                             Card::spade_ace,
                                             Card::spade_king});
  case Type::solo_heart:
    return this->load_from_file_or_construct("solo.heart",
                                             {Card::diamond_queen,
                                             Card::club_jack,
                                             Card::heart_ace,
                                             Card::heart_king});
  case Type::solo_diamond:
    return this->load_from_file_or_construct("solo.diamond",
                                             {Card::heart_queen,
                                             Card::spade_jack,
                                             Card::diamond_ace,
                                             Card::diamond_king});
  case Type::solo_jack:
    return this->load_from_file_or_construct("solo.jack",
                                             {Card::club_jack,
                                             Card::spade_jack,
                                             Card::heart_jack,
                                             Card::diamond_jack});
  case Type::solo_queen:
    return this->load_from_file_or_construct("solo.queen",
                                             {Card::club_queen,
                                             Card::spade_queen,
                                             Card::heart_queen,
                                             Card::diamond_queen});
  case Type::solo_king:
    return this->load_from_file_or_construct("solo.king",
                                             {Card::club_king,
                                             Card::spade_king,
                                             Card::heart_king,
                                             Card::diamond_king});
  case Type::solo_queen_jack:
    return this->load_from_file_or_construct("solo.queen-jack",
                                             {Card::club_queen,
                                             Card::spade_queen,
                                             Card::heart_jack,
                                             Card::diamond_jack});
  case Type::solo_king_jack:
    return this->load_from_file_or_construct("solo.king-jack",
                                             {Card::club_king,
                                             Card::spade_king,
                                             Card::heart_jack,
                                             Card::diamond_jack});
  case Type::solo_king_queen:
    return this->load_from_file_or_construct("solo.king-queen",
                                             {Card::club_king,
                                             Card::spade_king,
                                             Card::heart_queen,
                                             Card::diamond_queen});
  case Type::solo_koehler:
    return this->load_from_file_or_construct("solo.koehler",
                                             {Card::club_king,
                                             Card::spade_king,
                                             Card::heart_queen,
                                             Card::diamond_jack});
  case Type::solo_meatless:
    return this->load_from_file_or_construct("solo.meatless",
                                             {Card::club_ace,
                                             Card::spade_ace,
                                             Card::heart_ace,
                                             Card::diamond_ace});
  case Type::no_120: {
    auto icon = this->load_from_file("no_120");
    if (!icon) {
      if (this->icon_.size() <= static_cast<unsigned>(type)) {
        // no icon loaded so far -- exiting
        this->ui->error("Cannot construct the icon for the announcement 'no 120'.\n"
                        "Perhaps you should validate your 'FreeDokorc'-File "
                        "(" + ::setting.path(Setting::Type::settings_file) + ").\n"
                        "Exiting.");
      } // if (this->icon_.size() <= type)
      this->ui->information(_("Cannot construct the icon for the announcement '%t'.\nKeeping the old one.",
                              _(Announcement::no120)),
                            InformationType::warning);
      icon = this->icon(type);
    } // if (!icon)
    return icon;
  }
  case Type::no_90: {
    auto icon = this->load_from_file("no_90");
    if (!icon) {
      if (this->icon_.size() <= static_cast<unsigned>(type)) {
        // no icon loaded so far -- exiting
        this->ui->error("Cannot construct the icon for the announcement 'no 90'.\n"
                        "Perhaps you should validate your 'FreeDokorc'-File "
                        "(" + ::setting.path(Setting::Type::settings_file) + ").\n"
                        "Exiting.");
      } // if (this->icon_.size() <= type)
      this->ui->information(_("Cannot construct the icon for the announcement '%t'.\nKeeping the old one.",
                              _(Announcement::no90)),
                            InformationType::warning);
      icon = this->icon(type);
    } // if (!icon)
    return icon;
  }
  case Type::no_60: {
    auto icon = this->load_from_file("no_60");
    if (!icon) {
      if (this->icon_.size() <= static_cast<unsigned>(type)) {
        // no icon loaded so far -- exiting
        this->ui->error("Cannot construct the icon for the announcement 'no 60'.\n"
                        "Perhaps you should validate your 'FreeDokorc'-File "
                        "(" + ::setting.path(Setting::Type::settings_file) + ").\n"
                        "Exiting.");
      } // if (this->icon_.size() <= type)
      this->ui->information(_("Cannot construct the icon for the announcement '%t'.\nKeeping the old one.",
                              _(Announcement::no60)),
                            InformationType::warning);
      icon = this->icon(type);
    } // if (!icon)
    return icon;
  }
  case Type::no_30: {
    auto icon = this->load_from_file("no_30");
    if (!icon) {
      if (this->icon_.size() <= static_cast<unsigned>(type)) {
        // no icon loaded so far -- exiting
        this->ui->error("Cannot construct the icon for the announcement 'no 30'.\n"
                        "Perhaps you should validate your 'FreeDokorc'-File "
                        "(" + ::setting.path(Setting::Type::settings_file) + ").\n"
                        "Exiting.");
      } // if (this->icon_.size() <= type)
      this->ui->information(_("Cannot construct the icon for the announcement '%t'.\nKeeping the old one.",
                              _(Announcement::no30)),
                            InformationType::warning);
      icon = this->icon(type);
    } // if (!icon)
    return icon;
  }
  case Type::no_0: {
    auto icon = this->load_from_file("no_0");
    if (!icon) {
      if (this->icon_.size() <= static_cast<unsigned>(type)) {
        // no icon loaded so far -- exiting
        this->ui->error("Cannot construct the icon for the announcement 'no 0'.\n"
                        "Perhaps you should validate your 'FreeDokorc'-File "
                        "(" + ::setting.path(Setting::Type::settings_file) + ").\n"
                        "Exiting.");
      } // if (this->icon_.size() <= type)
      this->ui->information(_("Cannot construct the icon for the announcement '%t'.\nKeeping the old one.",
                              _(Announcement::no0)),
                            InformationType::warning);
      icon = this->icon(type);
    } // if (!icon)
    return icon;
  }
  case Type::no_120_reply: {
    auto icon = this->load_from_file("no_120_reply");
    if (!icon) {
      if (this->icon_.size() <= static_cast<unsigned>(type)) {
        // no icon loaded so far -- exiting
        this->ui->error("Cannot construct the icon for the announcement 'no 120 reply'.\n"
                        "Perhaps you should validate your 'FreeDokorc'-File "
                        "(" + ::setting.path(Setting::Type::settings_file) + ").\n"
                        "Exiting.");
      } // if (this->icon_.size() <= type)
      this->ui->information(_("Cannot construct the icon for the announcement '%t'.\nKeeping the old one.",
                              _(Announcement::no120_reply)),
                            InformationType::warning);
      icon = this->icon(type);
    } // if (!icon)
    return icon;
  }
  case Type::no_90_reply: {
    auto icon = this->load_from_file("no_90_reply");
    if (!icon) {
      if (this->icon_.size() <= static_cast<unsigned>(type)) {
        // no icon loaded so far -- exiting
        this->ui->error("Cannot construct the icon for the announcement 'no 90 reply'.\n"
                        "Perhaps you should validate your 'FreeDokorc'-File "
                        "(" + ::setting.path(Setting::Type::settings_file) + ").\n"
                        "Exiting.");
      } // if (this->icon_.size() <= type)
      this->ui->information(_("Cannot construct the icon for the announcement '%t'.\nKeeping the old one.",
                              _(Announcement::no90_reply)),
                            InformationType::warning);
      icon = this->icon(type);
    } // if (!icon)
    return icon;
  }
  case Type::no_60_reply: {
    auto icon = this->load_from_file("no_60_reply");
    if (!icon) {
      if (this->icon_.size() <= static_cast<unsigned>(type)) {
        // no icon loaded so far -- exiting
        this->ui->error("Cannot construct the icon for the announcement 'no 60 reply'.\n"
                        "Perhaps you should validate your 'FreeDokorc'-File "
                        "(" + ::setting.path(Setting::Type::settings_file) + ").\n"
                        "Exiting.");
      } // if (this->icon_.size() <= type)
      this->ui->information(_("Cannot construct the icon for the announcement '%t'.\nKeeping the old one.",
                              _(Announcement::no60_reply)),
                            InformationType::warning);
      icon = this->icon(type);
    } // if (!icon)
    return icon;
  }
  case Type::no_30_reply: {
    auto icon = this->load_from_file("no_30_reply");
    if (!icon) {
      if (this->icon_.size() <= static_cast<unsigned>(type)) {
        // no icon loaded so far -- exiting
        this->ui->error("Cannot construct the icon for the announcement 'no 30 reply'.\n"
                        "Perhaps you should validate your 'FreeDokorc'-File "
                        "(" + ::setting.path(Setting::Type::settings_file) + ").\n"
                        "Exiting.");
      } // if (this->icon_.size() <= type)
      this->ui->information(_("Cannot construct the icon for the announcement '%t'.\nKeeping the old one.",
                              _(Announcement::no30_reply)),
                            InformationType::warning);
      icon = this->icon(type);
    } // if (!icon)
    return icon;
  }
  case Type::no_0_reply: {
    auto icon = this->load_from_file("no_0_reply");
    if (!icon) {
      if (this->icon_.size() <= static_cast<unsigned>(type)) {
        // no icon loaded so far -- exiting
        this->ui->error("Cannot construct the icon for the announcement 'no 0 reply'.\n"
                        "Perhaps you should validate your 'FreeDokorc'-File "
                        "(" + ::setting.path(Setting::Type::settings_file) + ").\n"
                        "Exiting.");
      } // if (this->icon_.size() <= type)
      this->ui->information(_("Cannot construct the icon for the announcement '%t'.\nKeeping the old one.",
                              _(Announcement::no0_reply)),
                            InformationType::warning);
      icon = this->icon(type);
    } // if (!icon)
    return icon;
  }
  } // switch(type)

  return icon;
} // Gdk::ScaledPixbufRotations Icons::load(Type type)

/** construct an icon from the cards
 **
 ** @param    cards   cards
 **
 ** @return   constructed icon
 **
 ** @bug   the transparent regions are copied, too
 **/
Gdk::ScaledPixbufRotations
Icons::construct(vector<Card> const& cards)
{
  // take the half height and overlap with a fourth width

  int const width = (this->ui->cards->width_original()
                     * (4 + (cards.size() - 1))
                     / 4);
  int const height = (this->ui->cards->height_original() / 2);

  auto pixbuf = Gdk::Pixbuf::create(Gdk::COLORSPACE_RGB,
                                    true, 8,
                                    width, height);

  for (unsigned i = 0; i < cards.size(); i++)
    this->ui->cards->card_original(cards[i])->copy_area(0, 0,
                                                        this->ui->cards->width_original(),
                                                        this->ui->cards->height_original() / 2,
                                                        pixbuf,
                                                        i * this->ui->cards->width_original() / 4, 0);


  return Gdk::ScaledPixbufRotations(pixbuf->scale_simple(width * 4 / 5,
                                                         height * 4 / 5,
                                                         Gdk::INTERP_BILINEAR));
} // Gdk::ScaledPixbufRotations Icons::construct(vector<Card> cards)

/** @return   true, if type is none
 **/
bool
operator!(Icons::Type type)
{
  return (type == Icons::Type::none);
} // bool operator!(Icons::Type type)

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
