/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#ifdef USE_UI_GTKMM

#pragma once

#include "base.h"

#include "widgets/scaled_pixbuf_rotations.h"

#include "../../basetypes.h"
#include "../../card/card.h"

class Player;

#include <glibmm/refptr.h>
namespace Gtk {
class Image;
}; // namespace Gtk

namespace UI_GTKMM_NS {

/**
 ** the icons
 **/
class Icons : public Base, public sigc::trackable {
  friend class Icongroup;
  public:
  enum class Type {
    none = 0,
    re,
    contra,
    thrown_nines,
    thrown_kings,
    thrown_nines_and_kings,
    thrown_richness,
    redistribute,
    fox_highest_trump,
    marriage,
    marriage_solo,
    marriage_foreign,
    marriage_trump,
    marriage_color,
    marriage_club,
    marriage_spade,
    marriage_heart,
    marriage_partner,
    genscher,
    poverty,
    poverty_trumps_0,
    poverty_trumps_1,
    poverty_trumps_2,
    poverty_trumps_3,
    poverty_partner,
    swines_club,
    swines_spade,
    swines_heart,
    swines_diamond,
    swines = swines_diamond,
    hyperswines_club,
    hyperswines_spade,
    hyperswines_heart,
    hyperswines_diamond,
    hyperswines = hyperswines_diamond,
    hyperswines_king_club,
    hyperswines_king_spade,
    hyperswines_king_heart,
    hyperswines_king_diamond,
    swines_hyperswines_club,
    swines_hyperswines_spade,
    swines_hyperswines_heart,
    swines_hyperswines_diamond,
    swines_hyperswines = swines_hyperswines_diamond,
    swines_hyperswines_king_club,
    swines_hyperswines_king_spade,
    swines_hyperswines_king_heart,
    swines_hyperswines_king_diamond,
    dollen,
    doppelkopf,
    solo_club,
    solo_spade,
    solo_heart,
    solo_diamond,
    solo_jack,
    solo_queen,
    solo_king,
    solo_queen_jack,
    solo_king_jack,
    solo_king_queen,
    solo_koehler,
    solo_meatless,
    no_120_reply,
    no_90_reply,
    no_60_reply,
    no_30_reply,
    no_0_reply,
    no_120,
    no_90,
    no_60,
    no_30,
    no_0,
    last = no_0
  }; // enum Type
  static size_t const number_of_types; // number of icons

  static Type type(GameType gametype);
  static Type type(MarriageSelector marriage_selector);
  static Type type(Team team);
  static Type type(Announcement announcement);
  static Type type_swines(Card::Color color);
  Type type_hyperswines(Card::Color color);
  Type type_swines_hyperswines(Card::Color color);

  public:
  explicit Icons(Base* base);
  ~Icons() override;

  double scaling() const;
  void update_scaling();

  int max_width(Rotation rotation = Rotation::up) const;
  int max_height(Rotation rotation = Rotation::up) const;
  int max_width(vector<Glib::RefPtr<Gdk::Pixbuf>> const& icons,
                Rotation rotation = Rotation::up) const;
  int max_height(vector<Glib::RefPtr<Gdk::Pixbuf>> const& icons,
                 Rotation rotation = Rotation::up) const;

  // the specific icon
  Glib::RefPtr<Gdk::Pixbuf> const&
    icon(Type type, Rotation rotation = Rotation::up) ;

  // the specific gametype icon
  Glib::RefPtr<Gdk::Pixbuf> const&
    icon(GameType gametype, Rotation rotation = Rotation::up)
    { return this->icon(this->type(gametype), rotation); }
  // the specific marriage selector icon
  Glib::RefPtr<Gdk::Pixbuf> const&
    icon(MarriageSelector marriage_selector, Rotation rotation = Rotation::up)
    { return this->icon(this->type(marriage_selector), rotation); }
  // the specific team icon
  Glib::RefPtr<Gdk::Pixbuf> const&
    icon(Team team, Rotation rotation = Rotation::up)
    { return this->icon(this->type(team), rotation); }
  // the specific announcement icon
  Glib::RefPtr<Gdk::Pixbuf> const&
    icon(Announcement announcement, Rotation rotation = Rotation::up)
    { return this->icon(this->type(announcement), rotation); }
  // the specific swines icon
  Glib::RefPtr<Gdk::Pixbuf> const&
    swines(Card::Color color, Rotation rotation = Rotation::up)
    { return this->icon(this->type_swines(color), rotation); }
  // the specific hyperswines icon
  Glib::RefPtr<Gdk::Pixbuf> const&
    hyperswines(Card::Color color, Rotation rotation = Rotation::up)
    { return this->icon(this->type_hyperswines(color), rotation); }
  // the specific swines-hyperswines icon
  Glib::RefPtr<Gdk::Pixbuf> const&
    swines_hyperswines(Card::Color color, Rotation rotation = Rotation::up)
    { return this->icon(this->type_swines_hyperswines(color), rotation); }


  // managed images
  Gtk::Image* new_managed_image(Type type = Type::none, double scaling = 1);

  void change_managed(Gtk::Image* image, Type type);
  void change_managed(Gtk::Image* image,
                      GameType gametype)
  { this->change_managed(image, this->type(gametype)); return ; }
  void change_managed(Gtk::Image* image,
                      MarriageSelector marriage_selector)
  { this->change_managed(image, this->type(marriage_selector)); return ; }
  void change_managed(Gtk::Image* image,
                      Team team)
  { this->change_managed(image, this->type(team)); return ; }
  void change_managed(Gtk::Image* image,
                      Announcement announcement)
  { this->change_managed(image, this->type(announcement)); return ; }
  void change_managed_swines(Gtk::Image* image,
                             Card::Color color)
  { this->change_managed(image, this->type_swines(color)); return ; }
  void change_managed_hyperswines(Gtk::Image* image,
                                  Card::Color color)
  { this->change_managed(image, this->type_hyperswines(color)); return ; }
  void change_scaling(Gtk::Image* image, double scaling);

  void remove_managed(Gtk::Image const* image);

  void update_managed();

  // update the maximal width and height
  void update_max_values();

  void load();

  Gdk::ScaledPixbufRotations load(Type type);
  private:
  Gdk::ScaledPixbufRotations load_from_file(string const& name_a,
                                            string const& name_b = "");
  Gdk::ScaledPixbufRotations load_from_file_or_construct(string const& file,
                                                         vector<Card> const& cards);
  Gdk::ScaledPixbufRotations load_from_file_or_construct(string const& name_a,
                                                         string const& name_b,
                                                         vector<Card> const& cards);
  Gdk::ScaledPixbufRotations construct(vector<Card> const& cards);

  private:
  // the icons
  vector<Gdk::ScaledPixbufRotations> icon_;
  // maximal width of the icons
  int max_width_;
  // maximal height of the icons
  int max_height_;

#ifdef OUTDATED
  // a reference to a drawable
  Glib::RefPtr<Gdk::Drawable> drawable_ref;
#endif

  // the managed images
  vector<Gtk::Image*> managed_image_;

  private: // unused
  Icons() = delete;
  Icons(Icons const&) = delete;
  Icons& operator=(Icons const&) = delete;
}; // class Icons : public Base

bool operator!(Icons::Type type);

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
