/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "menu.h"

#include "ui.h"
#include "main_window.h"
#include "party_settings.h"
#include "rules.h"
#include "players.h"
#include "players_db.h"
#include "game_debug.h"
#include "preferences.h"
#include "table.h"
#include "party_points.h"
#include "bug_report.h"
#include "bug_report_replay.h"
#include "help.h"
#include "license.h"
#include "changelog.h"
#include "support.h"
#include "about.h"

#include "../../basetypes.h"
#include "../../party/rule.h"
#include "../../game/game.h"
#include "../../misc/setting.h"
#include "../../misc/setting.theme.h"
#include "../../utils/string.h"

#include <gtkmm/menu.h>
#include <gtkmm/menuitem.h>
#include <gtkmm/checkmenuitem.h>
#include <gtkmm/imagemenuitem.h>
#include <gtkmm/separatormenuitem.h>
#include <gtkmm/notebook.h>
#include <gtkmm/filechooserdialog.h>
namespace UI_GTKMM_NS {

/** Constructor
 **
 ** @param    main_window   the corresponding main window
 **/
Menu::Menu(MainWindow* const main_window) :
  Base(main_window),
  MenuBar(),
  main_window(main_window)
{
  this->init();
} // Menu::Menu(MainWindow* main_window)

/** Destruktor
 **/
Menu::~Menu()
{
  for (auto& item : this->themes_menu->get_children())
    delete static_cast<Setting::Theme*>(item->get_data("theme"));
} // Menu::~Menu()

/** creates all subelements
 **/
void
Menu::init()
{
  { // party
    this->party_menu = Gtk::manage(new Gtk::Menu());

    this->party = Gtk::manage(new Gtk::MenuItem(_("Menu::party")));

    this->new_party = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::new party")));

    this->load_party = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::load party") + "…"));

    this->save_party = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::save party") + "…"));

    this->end_party = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::end party")));

    this->quit = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::quit")));
  } // party

  { // information

    this->information_menu = Gtk::manage(new Gtk::Menu());

    this->information = Gtk::manage(new Gtk::MenuItem(_("Menu::information")));

    this->card_suggestion = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::card suggestion") + "…"));

    this->last_trick = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::last trick") + "…"));

    this->game_debug = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::game debug") + "…"));

    this->bug_report_replay = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::bug report replay") + "…"));

    this->rules = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::rules") + "…"));

    this->party_points = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::party points") + "…"));

    this->players = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::players") + "…"));

    this->players_db = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::players database") + "…"));
  } // information

  { // announcements
    this->announcements = Gtk::manage(new Gtk::MenuItem(_("Menu::announcements")));
  } // announcements


  { // preferences
    this->preferences_menu = Gtk::manage(new Gtk::Menu());

    this->preferences = Gtk::manage(new Gtk::MenuItem(_("Menu::preferences")));

    this->emphasize_valid_cards
      = Gtk::manage(new Gtk::CheckMenuItem(_(Setting::Type::emphasize_valid_cards)));

    this->show_all_hands
      = Gtk::manage(new Gtk::CheckMenuItem(_(Setting::Type::show_all_hands)));

    this->cards_order = Gtk::manage(new Gtk::MenuItem(_(Setting::Type::cards_order)));
    { // themes
      this->themes_menu = Gtk::manage(new Gtk::Menu());
      this->themes_item = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::preferences::themes")));
      for (auto const& theme : ::setting.themes()) {
        auto item = Gtk::manage(new Gtk::MenuItem(_("Setting::Theme::" + theme->name())));
        item->set_data("theme", new Setting::Theme(*theme));
        this->themes_menu->append(*item);
        item->signal_activate().connect(sigc::bind<Setting::Theme const* const>(sigc::mem_fun1(*this, &Menu::apply_theme_event), static_cast<Setting::Theme const*>(item->get_data("theme"))));
      } // for (theme \in ::setting.themes())
    } // themes

    this->preferences_preferences = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::preferences") + "…"));
  } // preferences


  { // help
    this->help_menu = Gtk::manage(new Gtk::Menu());

    this->help = Gtk::manage(new Gtk::MenuItem(_("Menu::help")));

    this->help_index = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::help index") + "…"));

    this->license = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::license") + "…"));

    this->changelog = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::changelog") + "…"));

    this->support = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::support") + "…"));

    this->about = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::about") + "…"));
    this->debug = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::debug") + "…"));

    this->generate_error = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::generate error")));
  } // help

  { // bug report
    this->bug_report = Gtk::manage(new Gtk::MenuItem(_("Menu::Item::bug report")));
  } // bug report

  this->append(*this->party);
  this->append(*this->information);
  this->append(*this->announcements);
  this->append(*this->preferences);
  this->append(*this->help);
  this->append(*this->bug_report);

  // the party menu
  this->party->set_submenu(*this->party_menu);

  this->party_menu->append(*this->new_party);
  this->party_menu->append(*this->load_party);
  this->party_menu->append(*this->save_party);
  this->party_menu->append(*this->end_party);
  this->party_menu->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
  this->party_menu->append(*this->quit);
  // the information menu
  this->information->set_submenu(*this->information_menu);
  this->information_menu->append(*this->card_suggestion);
  this->information_menu->append(*this->last_trick);
  this->information_menu->append(*this->game_debug);
  this->information_menu->append(*this->bug_report_replay);
  this->information_menu->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
  this->information_menu->append(*this->rules);
  this->information_menu->append(*this->party_points);
  this->information_menu->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
  this->information_menu->append(*this->players);
  this->information_menu->append(*this->players_db);
  // the preferences menu
  this->preferences->set_submenu(*this->preferences_menu);
  this->preferences_menu->append(*this->emphasize_valid_cards);
  this->preferences_menu->append(*this->show_all_hands);
  this->preferences_menu->append(*this->cards_order);
  this->preferences_menu->append(*this->themes_item);
  this->themes_item->set_submenu(*this->themes_menu);
  this->preferences_menu->append(*this->preferences_preferences);
  // the help menu
  this->help->set_submenu(*this->help_menu);
  this->help_menu->append(*this->help_index);
  this->help_menu->append(*this->license);
  this->help_menu->append(*this->changelog);
  this->help_menu->append(*this->support);
  this->help_menu->append(*this->about);
#ifndef RELEASE
  this->help_menu->append(*Gtk::manage(new Gtk::SeparatorMenuItem()));
  this->help_menu->append(*this->debug);
  this->help_menu->append(*this->generate_error);
#endif

  this->card_suggestion->set_sensitive(false);
  this->last_trick->set_sensitive(false);
  this->game_debug->set_sensitive(false);
  this->information->set_sensitive(false);

  this->emphasize_valid_cards->set_active(::setting.value(Setting::Type::emphasize_valid_cards));
  this->show_all_hands->set_active(::setting.value(Setting::Type::show_all_hands));


  { // load party file chooser
    this->load_party_file_chooser = make_unique<Gtk::FileChooserDialogWrapper>(_("load party"),
                                                                               Gtk::FILE_CHOOSER_ACTION_OPEN,
                                                                               sigc::mem_fun(*this, &Menu::load_party_event));

    string dir = (::setting(Setting::Type::private_data_directory)
                  + "/"
                  + ::setting(Setting::Type::parties_directory)
                  + "/"
                 );
#ifdef WINDOWS
    String::replace_all(dir, "/", "\\");
#endif
    this->load_party_file_chooser->set_current_folder(dir);
  } // load party file chooser
  { // save party file chooser
    this->save_party_file_chooser = make_unique<Gtk::FileChooserDialogWrapper>(_("save party"),
                                                                               Gtk::FILE_CHOOSER_ACTION_SAVE,
                                                                               sigc::mem_fun(*this, &Menu::save_party_event));
    string dir = (::setting(Setting::Type::private_data_directory)
                  + "/"
                  + ::setting(Setting::Type::parties_directory)
                  + "/"
                 );
#ifdef WINDOWS
    String::replace_all(dir, "/", "\\");
#endif
    this->save_party_file_chooser->set_current_folder(dir);
  } // save party file chooser

  this->show_all();
  this->bug_report_replay->hide();

  Party::signal_open().connect_back(*this, &Menu::party_open, this->disconnector_);
} // void Menu::init()

/** set the signals
 **/
void
Menu::set_signals()
{
  { // the signals
    this->new_party->signal_activate().connect(sigc::mem_fun(*this->main_window,
                                                             &MainWindow::start_new_party_event)
                                              );
    this->load_party->signal_activate().connect(sigc::mem_fun0(*this->load_party_file_chooser,
                                                               &Gtk::FileChooserDialogWrapper::present)
                                               );
    this->save_party->signal_activate().connect(sigc::mem_fun0(*this->save_party_file_chooser,
                                                               &Gtk::FileChooserDialogWrapper::present)
                                               );
    this->end_party->signal_activate().connect(sigc::mem_fun(*this->main_window,
                                                             &MainWindow::end_party_event)
                                              );
    this->quit->signal_activate().connect(sigc::mem_fun(*this->main_window,
                                                        &MainWindow::quit_program_event)
                                         );


    this->card_suggestion->signal_activate().connect(sigc::bind<bool const>(sigc::mem_fun(*this->ui->table, &Table::show_card_suggestion), true));
    this->last_trick->signal_activate().connect(sigc::mem_fun(*this->ui->table,
                                                              &Table::show_last_trick));

    this->game_debug->signal_activate().connect(sigc::mem_fun0(*this->ui->game_debug,
                                                               &Gtk::Window::present));
    //this->bug_report_replay->signal_activate().connect(sigc::mem_fun(*this->ui->bug_report_replay, &Gtk::Window::present));

    this->rules->signal_activate().connect(sigc::mem_fun0(*this->ui->party_settings->rules, &Gtk::Window::present));
    this->players->signal_activate().connect(sigc::mem_fun0(*this->ui->party_settings->players, &Gtk::Window::present));
    this->players_db->signal_activate().connect(sigc::mem_fun0(*this->ui->players_db, &Gtk::Window::present));


    this->emphasize_valid_cards->signal_toggled().connect(sigc::mem_fun(*this, &Menu::emphasize_valid_cards_set)
                                                         );
    this->show_all_hands->signal_toggled().connect(sigc::mem_fun(*this, &Menu::show_all_hands_set)
                                                  );
    this->cards_order->signal_activate().connect(sigc::mem_fun(*this->ui->preferences, &Preferences::show_cards_order));
    this->themes_item->signal_activate().connect(sigc::mem_fun(*this, &Menu::update_themes_sensitivity));
    this->preferences_preferences->signal_activate().connect(sigc::mem_fun0(*this->ui->preferences, &Gtk::Window::present));

    this->bug_report->signal_activate().connect(sigc::mem_fun(*this->ui->bug_report, &BugReport::create_report));


    this->help_index->signal_activate().connect(sigc::mem_fun0(*this->ui->help,
                                                               &Help::show_manual));
    this->license->signal_activate().connect(sigc::mem_fun0(*this->ui->license,
                                                            &Gtk::Window::present));
    this->changelog->signal_activate().connect(sigc::mem_fun0(*this->ui->changelog,
                                                              &Gtk::Window::present));
    this->support->signal_activate().connect(sigc::mem_fun0(*this->ui->support,
                                                            &Gtk::Window::present));
    this->about->signal_activate().connect(sigc::mem_fun0(*this->ui->about,
                                                          &Gtk::Window::present));
    this->generate_error->signal_activate().connect(sigc::mem_fun(*this->main_window,
                                                                  &MainWindow::generate_error_event));
  } // the signals

  { // keys

    // I have here some notes, whenever I differ from the HIG ('GNOME Human Interface Guidelines (1.0)', see http://developer.gnome.org/projects/gup/hig/)

    this->new_party->add_accelerator("activate",
                                     this->main_window->get_accel_group(),
                                     GDK_KEY_n,
                                     Gdk::CONTROL_MASK,
                                     Gtk::ACCEL_VISIBLE);
    // Note:
    // The HIG says 'C-o' for 'open a new document', but 'open' says to open in a new window (see HIG, the file menu). Here I do not want to open another FreeDoko window but to replace the existing party.
    // So I use 'C-l' (load) in contrast to 'C-s' (save).
    this->load_party->add_accelerator("activate",
                                      this->main_window->get_accel_group(),
                                      GDK_KEY_l,
                                      Gdk::CONTROL_MASK,
                                      Gtk::ACCEL_VISIBLE);
    this->save_party->add_accelerator("activate",
                                      this->main_window->get_accel_group(),
                                      GDK_KEY_s,
                                      Gdk::CONTROL_MASK,
                                      Gtk::ACCEL_VISIBLE);
    this->quit->add_accelerator("activate",
                                this->main_window->get_accel_group(),
                                GDK_KEY_q,
                                Gdk::CONTROL_MASK,
                                Gtk::ACCEL_VISIBLE);
#if 0
    this->card_suggestion->add_accelerator("activate",
                                           this->main_window->get_accel_group(),
                                           GDK_KEY_h,
                                           Gdk::ModifierType(0),
                                           Gtk::ACCEL_VISIBLE);
    this->last_trick->add_accelerator("activate",
                                      this->main_window->get_accel_group(),
                                      GDK_KEY_l,
                                      Gdk::ModifierType(0),
                                      Gtk::ACCEL_VISIBLE);
    this->rules->add_accelerator("activate",
                                 this->main_window->get_accel_group(),
                                 GDK_KEY_r,
                                 Gdk::ModifierType(0),
                                 Gtk::ACCEL_VISIBLE);
    this->party_points->add_accelerator("activate",
                                        this->main_window->get_accel_group(),
                                        GDK_KEY_p,
                                        Gdk::ModifierType(0),
                                        Gtk::ACCEL_VISIBLE);
#ifndef WORKAROUND
    // does not work
    this->players->add_accelerator("activate",
                                   this->main_window->get_accel_group(),
                                   GDK_KEY_P,
                                   Gdk::ModifierType(0),
                                   Gtk::ACCEL_VISIBLE);
#endif
#ifndef RELEASE
    this->game_debug->add_accelerator("activate",
                                      this->main_window->get_accel_group(),
                                      GDK_KEY_i,
                                      Gdk::ModifierType(0),
                                      Gtk::ACCEL_VISIBLE);
#endif
#endif
    // HIG: C-a should select all
    this->show_all_hands->add_accelerator("activate",
                                          this->main_window->get_accel_group(),
                                          GDK_KEY_a,
                                          Gdk::CONTROL_MASK,
                                          Gtk::ACCEL_VISIBLE);
    // HIG: C-o should open a new document / party
    this->cards_order->add_accelerator("activate",
                                       this->main_window->get_accel_group(),
                                       GDK_KEY_o,
                                       Gdk::CONTROL_MASK,
                                       Gtk::ACCEL_VISIBLE);

    // Note:
    // The HIG uses F2 for 'Rename' (p.e. a filename in the file manager).
    // Since I did not find some shortcut for preferences I use F2 because
    // it is next to F1 for help.
    // Later I will use S-F2 for context sensitive preferences
    // as S-F1 for context sensitive help.
    this->preferences_preferences->add_accelerator("activate",
                                                   this->main_window->get_accel_group(),
                                                   GDK_KEY_F2,
                                                   Gdk::ModifierType(0),
                                                   Gtk::ACCEL_VISIBLE);
    this->help_index->add_accelerator("activate",
                                      this->main_window->get_accel_group(),
                                      GDK_KEY_F1,
                                      Gdk::ModifierType(0),
                                      Gtk::ACCEL_VISIBLE);
#if 0
    // any suggestions?
    // Ctrl-l is used for loading a party.
    // Further I do not think that Ctrl-Shift-something is good usability (which program (but emacs) does use such shortcuts?).
    this->license->add_accelerator("activate",
                                   this->main_window->get_accel_group(),
                                   GDK_KEY_l,
                                   Gdk::CONTROL_MASK,
                                   Gtk::ACCEL_VISIBLE);
#endif
  } // keys
} // void Menu::set_signals()

/** sets the setting 'emphasize valid cards'
 **/
void
Menu::emphasize_valid_cards_set()
{
  ::setting.set(Setting::Type::emphasize_valid_cards,
                this->emphasize_valid_cards->get_active());
} // void Menu::emphasize_valid_cards_set()

/** sets the setting 'show all hands'
 **/
void
Menu::show_all_hands_set()
{
  ::setting.set(Setting::Type::show_all_hands,
                this->show_all_hands->get_active());
} // void Menu::show_all_hands_set()

/** event: make the announcement 'announcement' for player 'playerno'
 **
 ** @param    playerno   the number of the player who announces
 ** @param    announcement   the announcement to make
 **/
void
Menu::announcement_event(unsigned const playerno,
                         Announcement const announcement)
{
  // note, that here it is not the playernumber in the game but in the party
  this->ui->game().announcements().make_announcement(announcement,
                                     this->ui->game().player(playerno));
} // void Menu::announcement_event(unsigned playerno, int announcement)

/** event: announce 'swines' for player 'playerno'
 **
 ** @param    playerno   the number of the player who announces swines
 **   if it is 'UINT_MAX' the player is determined
 **   automatically
 **/
void
Menu::swines_announcement_event(unsigned const playerno)
{
  auto& game = this->ui->game();
  auto& swines = game.swines();

  // note, that here it is not the playernumber in the game but in the party
  if (playerno == UINT_MAX) {
    for (auto& p : game.players()) {
      if (swines.swines_announcement_valid(p)) {
        swines.swines_announce(p);
        break;
      } 
    } // for (p : this->ui->game().players())
  } else { // if !(playerno == UINT_MAX)
    swines.swines_announce(game.player(playerno));
  } // if !(playerno == UINT_MAX)
} // void Menu::swines_announcement_event(unsigned playerno)

/** event: announce 'hyperswines' for player 'playerno'
 **
 ** @param    playerno   the number of the player with hyperswines
 **   if it is 'UINT_MAX' the player is determined
 **   automatically
 **/
void
Menu::hyperswines_announcement_event(unsigned const playerno)
{
  auto& game = this->ui->game();
  auto& swines = game.swines();

  // note, that here it is not the playernumber in the game but in the party
  if (playerno == UINT_MAX) {
    for (auto& p : game.players()) {
      if (swines.hyperswines_announcement_valid(p)) {
        swines.hyperswines_announce(p);
        break;
      } 
    } // for (p : this->ui->game().players())
  } else { // if !(playerno == UINT_MAX)
    swines.hyperswines_announce(this->ui->game().player(playerno));
  } // if !(playerno == UINT_MAX)
} // void Menu::hyperswines_announcement_event(unsigned playerno)

/** the party is opened
 ** some sensitivity is set
 **
 ** @param    party   party
 **/
void
Menu::party_open(Party& party)
{
  this->party_points_present_connection
    = this->party_points->signal_activate().connect(sigc::mem_fun0(*this->ui->table->party_points_,
                                                                   &Gtk::Window::present));

  switch (::game_status) {

  case GameStatus::party_new:
    this->party_points->set_sensitive(false);
    this->end_party->set_sensitive(false);
    break;

  case GameStatus::party_initial_loaded:
  case GameStatus::party_loaded:
    this->party_points->set_sensitive(true);
    this->end_party->set_sensitive(true);
    break;

  default:
    DEBUG_ASSERTION(false,
                    "Menu::party_open():\n"
                    "  game status = '" << ::game_status << "'");
  } // switch (::game_status)

  this->information->set_sensitive(true);
  this->bug_report_replay->set_sensitive(true);
  this->rules->set_sensitive(true);
  this->players->set_sensitive(true);
  this->players_db->set_sensitive(true);

  this->card_suggestion->set_sensitive(false);
  this->last_trick->set_sensitive(false);
  this->game_debug->set_sensitive(false);
  this->announcements->set_sensitive(false);
  this->announcements->unset_submenu();

  party.signal_start().connect_back(*this, &Menu::party_start, this->disconnector_);
  party.signal_finish().connect_back(*this, &Menu::party_finish, this->disconnector_);
  party.signal_close().connect_back(*this, &Menu::party_close, this->disconnector_);
} // void Menu::party_open(Party& party)

/** the party has started
 ** create the announcements menu
 ** make some information accessable
 **/
void
Menu::party_start()
{
  this->end_party->set_sensitive(true);

  this->party_points->set_sensitive(true);

  { // create the announcements menu
    this->announcements_player.clear();
    this->announcements_player_menu.clear();
    this->announcements_announcement.clear();
    this->swines_separator.clear();
    this->swines_announcement.clear();
    this->hyperswines_announcement.clear();

    Gtk::Menu* menu = nullptr;
    menu  = Gtk::manage(new Gtk::Menu());
    for (auto const& p : this->ui->party().players()) {
      this->announcements_player.push_back(Gtk::manage(new Gtk::MenuItem(p.name())));

      this->announcements_player_menu.push_back(Gtk::manage(new Gtk::Menu()));
      this->announcements_announcement.push_back(vector<Gtk::MenuItem*>()); 

      this->announcements_announcement.back().push_back(Gtk::manage(new Gtk::MenuItem(_(Announcement::reply))));
      if (   this->ui->party().rule()(Rule::Type::announcement_reply)
          && this->ui->party().rule()(Rule::Type::announcement_strict_limit)) {
        this->announcements_player_menu.back()->append(*this->announcements_announcement.back().back());
        this->announcements_announcement.back().back()->signal_activate().connect(sigc::bind<unsigned, Announcement>(sigc::mem_fun(*this, &Menu::announcement_event), this->ui->party().players().no(p), Announcement::reply));
      }
      for (auto a : real_announcements) {
        this->announcements_announcement.back().push_back(Gtk::manage(new Gtk::MenuItem(_(a))));
        this->announcements_player_menu.back()->append(*this->announcements_announcement.back().back());

        this->announcements_announcement.back().back()->signal_activate().connect(sigc::bind<unsigned, Announcement>(sigc::mem_fun(*this, &Menu::announcement_event), this->ui->party().players().no(p), a));

      } // for (a \in Announcement)

      { // swines separator
        this->swines_separator.push_back(Gtk::manage(new Gtk::SeparatorMenuItem()));
        this->announcements_player_menu.back()->append(*this->swines_separator.back());
      } // swines separator
      { // swines
        this->swines_announcement.push_back(Gtk::manage(new Gtk::MenuItem(_("Menu::Item::swines"))));
        this->announcements_player_menu.back()->append(*this->swines_announcement.back());

        this->swines_announcement.back()->signal_activate().connect(sigc::bind<unsigned const>(sigc::mem_fun(*this, &Menu::swines_announcement_event), this->ui->party().players().no(p)));
      } // swines
      { // hyperswines
        this->hyperswines_announcement.push_back(Gtk::manage(new Gtk::MenuItem(_("Menu::Item::hyperswines"))));
        this->announcements_player_menu.back()->append(*this->hyperswines_announcement.back());

        this->hyperswines_announcement.back()->signal_activate().connect(sigc::bind<unsigned const>(sigc::mem_fun(*this, &Menu::hyperswines_announcement_event), this->ui->party().players().no(p)));
      } // hyperswines

      this->announcements_player_menu.back()->show_all_children();
      if ((this->ui->party().players().count_humans() == 1)
          && (p.type() == Player::Type::human))
        menu = this->announcements_player_menu.back();
      else
        this->announcements_player.back()->set_submenu(*this->announcements_player_menu.back());

      if ((this->ui->party().players().count_humans() != 1)
          && (p.type() == Player::Type::human))
        menu->append(*this->announcements_player.back());
    } // for (p < this->ui->party().playerno())
    this->announcements->set_submenu(*menu);
    menu->show_all_children();
  } // create the announcements menu
} // void Menu::party_start()

/** the party is finished
 **/
void
Menu::party_finish()
{
  this->end_party->set_sensitive(false);
} // void Menu::party_finish()

/** the party has closed
 **
 ** @todo   better signal disconnection
 **/
void
Menu::party_close()
{
  // ToDo: disconnection without extra variables
  this->party_points_present_connection.disconnect();

  this->party_points->set_sensitive(false);
  this->information->set_sensitive(false);
  this->last_trick->set_sensitive(false);
  this->game_debug->set_sensitive(false);
} // void Menu::party_close()

/** the game has started
 ** make the announcements menu accessable
 **/
void
Menu::game_start()
{
  this->announcements->set_sensitive(this->ui->game().players().count_humans() > 0);
  this->game_debug->set_sensitive(true);

  // update the announcements
  // to have 're'/'contra' instead of 'no 120'
  if (::game_status >= GameStatus::game_init)
    for (unsigned p = 0; p < this->ui->game().playerno(); ++p) {
      dynamic_cast<Gtk::Label*>(this->announcements_announcement[p][0]->get_child()
                               )->set_text(_(Announcement::reply));
      for (auto a : real_announcements) {
        // if the team is 're' or 'contra'
        // take 're'/'contra' instead of 'no 120'
        if ( (a == Announcement::no120)
            && !this->ui->game().rule()(Rule::Type::knocking)
            && ((this->ui->game().player(p).team() == Team::re)
                || (this->ui->game().player(p).team() == Team::contra)) )
          dynamic_cast<Gtk::Label*>(this->announcements_announcement[p][static_cast<int>(a)]->get_child()
                                   )->set_text(_(this->ui->game().player(p).team()));
        else
          dynamic_cast<Gtk::Label*>(this->announcements_announcement[p][static_cast<int>(a)]->get_child()
                                   )->set_text(_(Announcement(a)));
      } // for (auto a : real_announcements)
    } // for (p < this->ui->game().number_of_players())
} // void Menu::game_start()

/** a new trick is opened
 **/
void
Menu::trick_open()
{
  this->last_trick->set_sensitive(this->ui->game().tricks().real_current_no() >= 1);
  this->announcements_update();
} // void Menu::trick_open()

/** the (human) player has to play a card
 **/
void
Menu::card_get()
{
  this->card_suggestion->set_sensitive(true);
} // void Menu::card_get()

/** the (human) player has played a card
 **/
void
Menu::card_got()
{
  this->card_suggestion->set_sensitive(false);
} // void Menu::card_got()

/** the game is finished
 **/
void
Menu::game_finish()
{
  this->last_trick->set_sensitive(false);
} // void Menu::game_finish()

/** the game is closed
 **/
void
Menu::game_close()
{
  this->game_debug->set_sensitive(false);
} // void Menu::game_close()

/** updates the announcements, that is the sensitivity of them
 **/
void
Menu::announcements_update()
{
  auto const& game = this->ui->game();
  bool one_valid = false;
  for (unsigned p = 0; p < game.playerno(); p++) {
    auto const& player = game.player(p);
    bool valid = false;

    if (game.announcements().is_valid(Announcement::reply, player)) {
      auto const reply
        = to_reply(player.game().announcements().last(opposite(player.team())
                                                     ).announcement);
      dynamic_cast<Gtk::Label*>(this->announcements_announcement[p][0]->get_child())->set_text(_(reply));
      this->announcements_announcement[p][0]->set_sensitive(true);
    } else { // if !(reply valid)
      dynamic_cast<Gtk::Label*>(this->announcements_announcement[p][0]->get_child())->set_text(_(Announcement::reply));
      this->announcements_announcement[p][0]->set_sensitive(false);
    } // if !(reply valid)

    if (   !game.rule()(Rule::Type::knocking)
        && is_real(player.team())) {
      dynamic_cast<Gtk::Label*>(this->announcements_announcement[p][static_cast<int>(Announcement::no120)]->get_child())->set_text(_(player.team()));
    }
    for (auto a : real_announcements)  {
      auto const v = game.announcements().is_valid(a, player);
      this->announcements_announcement[p][static_cast<int>(a)]->set_sensitive(v);
      valid |= v;
    } // for (a \in Announcement)
    { // swines
      valid |= (   game.swines().swines_announcement_valid(player)
                || game.swines().hyperswines_announcement_valid(player));
      if (game.rule()(Rule::Type::swines)) {
        this->swines_separator[p]->show();
        this->swines_announcement[p]->show();
      } else {
        this->swines_separator[p]->hide();
        this->swines_announcement[p]->hide();
      }
      if (game.rule()(Rule::Type::hyperswines))
        this->hyperswines_announcement[p]->show();
      else
        this->hyperswines_announcement[p]->hide();

      this->swines_announcement[p]->set_sensitive(game.swines().swines_announcement_valid(player));
      this->hyperswines_announcement[p]->set_sensitive(game.swines().hyperswines_announcement_valid(player));
    } // swines

    this->announcements_player[p]->set_sensitive(valid);
    valid &= ((player.type() == Player::Type::human));
    one_valid |= valid;
  } // for (p < game.playerno())
  this->announcements->set_sensitive(one_valid);
} // void Menu::announcements_update()

/** loads the party
 ** The filename is taken from 'load_party_file_chooser'.
 **
 ** @param    filename   file to load
 **
 ** @return   whether the loading was successfull
 **/
bool
Menu::load_party_event(string const filename)
{
  if (this->ui->party().load(filename))
    this->ui->thrower(GameStatus::party_loaded,
                      __FILE__, __LINE__);
  return true;
} // bool Menu::load_party_event(string filename)

/** saves the party
 ** The filename is taken from 'save_party_file_chooser'.
 **
 ** @param    filename   file to save
 **
 ** @return   whether the saving was successfull
 **/
bool
Menu::save_party_event(string const filename)
{
  if (!this->ui->party().save(filename)) {
    this->ui->information(_("save party failed: %s",
                            filename),
                          InformationType::problem);
    return false;
  }
  return true;
} // bool Menu::save_party_event(string filename)

/** update the sensitivity of the themes
 **/
void
Menu::update_themes_sensitivity()
{
  for (auto& item : this->themes_menu->get_children())
    if (item->get_data("theme"))
      item->set_sensitive(static_cast<Setting::Theme*>(item->get_data("theme"))->can_be_applied());
} // void Menu::update_theme_sensitivity()

/** apply the theme
 **
 ** @param    theme   theme to apply
 **/
void
Menu::apply_theme_event(Setting::Theme const* const theme)
{
  if (theme)
    ::setting.set(*theme);
} // void Menu::save_party_event(Setting::Theme* theme)

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
