/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Party
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "party_summary.h"

#include "ui.h"
#include "main_window.h"
#include "program_updated.h"
#include "bug_report_replay.h"

#include "../../party/party.h"
#include "../../party/rule.h"
#include "../../os/bug_report_replay.h"

#include <gtkmm/button.h>
#include <gtkmm/label.h>
#include <gtkmm/main.h>
#include <gtkmm/treeview.h>

namespace UI_GTKMM_NS {

/** constructor
 **
 ** @param    playerno   number of players
 **/
PartySummary::PlayersSummaryModel::PlayersSummaryModel(unsigned const playerno) :
  name(), player(playerno), visible()
{
  this->add(this->name);
  for (unsigned p = 0; p < playerno; ++p)
    this->add(this->player[p]);
  this->add(this->visible);
}

/** constructor
 **
 ** @param    parent   the parent object
 **/
PartySummary::PartySummary(Base* const parent) :
  Base(parent),
  StickyDialog("FreeDoko – " + _("Window::party summary"), *parent->ui->main_window, false)
{
  this->ui->add_window(*this);

  this->signal_realize().connect(sigc::mem_fun(*this, &PartySummary::init));
#ifndef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
  this->signal_show().connect(sigc::mem_fun(*this, &PartySummary::on_show));
#endif
} // PartySummary::PartySummary(Base* parent)

/** destructor
 **/
PartySummary::~PartySummary() = default;

/** create all subelements
 **/
void
PartySummary::init()
{
  auto& party = this->ui->party();

  this->set_icon(this->ui->icon);

  this->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);

  this->get_content_area()->set_spacing(1 EX);

  { // action area
    this->new_party_button = Gtk::manage(new Gtk::Button(_("Button::new party")));
    this->add_action_widget(*this->new_party_button, Gtk::RESPONSE_CLOSE);
    this->new_party_button->signal_clicked().connect(sigc::mem_fun(*this,
                                                                   &PartySummary::new_party_event)
                                                    );

    this->continue_party_button = Gtk::manage(new Gtk::Button(_("Button::continue party")));
    this->add_action_widget(*this->continue_party_button, Gtk::RESPONSE_CLOSE);
    this->continue_party_button->signal_clicked().connect(sigc::mem_fun(*this,
                                                                        &PartySummary::continue_party_event)
                                                         );
    this->continue_party_button->set_can_default();
    this->continue_party_button->grab_default();
    this->continue_party_button->grab_focus();
  } // action area

  {
    this->roundnumber = Gtk::manage(new Gtk::Label(_("Roundnumber: %u", 0u)));
    this->get_content_area()->pack_start(*this->roundnumber, false, true);
  }
  {
    this->gamenumber = Gtk::manage(new Gtk::Label(_("Gamenumber: %u", 0u)));
    this->get_content_area()->pack_start(*this->gamenumber, false, true);
  }
  {
    this->remaining_rounds = Gtk::manage(new Gtk::Label(_("Remaining rounds: %u", 0u)));
    this->get_content_area()->pack_start(*this->remaining_rounds, false, true);
  }
  {
    this->remaining_points = Gtk::manage(new Gtk::Label(_("Remaining points: %i", 0)));
    this->get_content_area()->pack_start(*this->remaining_points, false, true);
  }
  {
    this->next_startplayer = Gtk::manage(new Gtk::Label(_("Next startplayer: %s", "Name")));
    this->get_content_area()->pack_end(*this->next_startplayer, false, true);
  }

  { // player
    this->player_model
      = std::make_unique<PlayersSummaryModel>(party.players().size());
    this->player_list
      = Gtk::ListStore::create(*this->player_model);
    this->points_row = *this->player_list->append();
    this->points_row[this->player_model->name] = _("points");
    this->duty_soli_row = *this->player_list->append();
    this->duty_soli_row[this->player_model->name] = _("duty soli");
    this->player_treeview
      = Gtk::manage(new Gtk::TreeView(this->player_list));
    this->player_treeview->get_selection()->set_mode(Gtk::SELECTION_NONE);

    this->player_treeview->append_column("", this->player_model->name);
    for (unsigned p = 0; p < party.players().size(); ++p) {
      this->player_treeview->append_column("Name", this->player_model->player[p]);
      this->player_treeview->get_column_cell_renderer(1 + p)->set_property("xalign", 0.5);
    }
  {
    int height, b;
    this->player_treeview->property_height_request() = -1;
    this->player_treeview->get_preferred_height(height, b);
    this->player_treeview->property_height_request() = 3 * height;
  }

    this->get_content_area()->pack_start(*this->player_treeview, Gtk::PACK_EXPAND_WIDGET);
  } // player

  this->update();
  this->show_all_children();
} // void PartySummary::init()

/** A party is loaded.
 ** Show this window.
 **/
void
PartySummary::party_loaded()
{
  if (::fast_play & FastPlay::party_start)
    return ;

  if (!this->get_realized())
    this->realize();

  this->update();

  if (::game_status == GameStatus::party_initial_loaded)
    this->new_party_button->show();
  else
    this->new_party_button->hide();

  this->present();
  if (this->ui->bug_report_replay->is_visible())
    this->ui->bug_report_replay->raise();

  while (   (this->is_visible())
         && !this->ui->thrower) {
    ::ui->wait();
    if (::bug_report_replay
        && ::bug_report_replay->auto_start_party())
      break;
  }

  this->hide();
} // void PartySummary::party_loaded()

/** show the window.
 ** Shows/hides the action buttons
 **/
void
PartySummary::on_show()
{
#ifdef GLIBMM_DEFAULT_SIGNAL_HANDLERS_ENABLED
  this->Dialog::on_show();
#endif

  if (this->ui->program_updated_window)
    this->ui->program_updated_window->raise();

  this->update();
} // void PartySummary::show()

/** update (rewrite) all information
 **
 ** @todo   remaining points
 ** @todo   duty soli
 **/
void
PartySummary::update()
{
  if (!this->get_realized())
    return ;

  if (   (::game_status != GameStatus::party_initial_loaded)
      && (::game_status != GameStatus::party_loaded))
    return ;

  Party const& party = this->ui->party();

  this->roundnumber->set_label(_("Roundnumber: %u",
                                 party.roundno() + 1));
  this->gamenumber->set_label(_("Gamenumber: %u",
                                party.gameno() + 1));
  if (party.rule()(Rule::Type::number_of_rounds_limited))
    this->remaining_rounds->set_label(_("Remaining rounds: %u",
                                        party.remaining_rounds()));
  else
    this->remaining_rounds->hide();
  if (party.rule()(Rule::Type::points_limited))
    this->remaining_points->set_label(_("Remaining points: %i",
                                        party.remaining_points()));
  else
    this->remaining_points->hide();

  this->next_startplayer->set_label(_("Next startplayer: %s",
                                      party.players()[party.startplayer()].name()));

#ifdef WORKAROUND
  // Wenn der Text neu gesetzt wird, ändert sich die Größe vom Fenster mehrmals schrittweise.
  if (this->continue_party_button->get_label()
      != (party.is_finished()
          ? _("Button::finish party")
          : _("Button::continue party")))
#endif
  {
    this->continue_party_button->set_label(party.is_finished()
                                           ? _("Button::finish party")
                                           : _("Button::continue party"));
  }
  this->duty_soli_row[this->player_model->visible]
    = (party.rule()(Rule::Type::number_of_duty_soli) > 0);
  for (unsigned p = 0; p < party.players().size(); ++p) {
    this->player_treeview->get_column(1 + p)->set_title(party.players()[p].name());
    this->points_row[this->player_model->player[p]] = party.pointsum(p);
    this->duty_soli_row[this->player_model->player[p]]
      = party.remaining_duty_soli(p);
  }
} // void PartySummary::update()

/** the name of 'player' has changed
 **
 ** @param    player   the player with the changed name
 **/
void
PartySummary::name_update(Player const& player)
{
  if (!this->get_realized())
    return ;

  auto const p = this->ui->party().players().no(player);
  if (p + 1 >= this->player_treeview->get_columns().size())
    return ;

  this->player_treeview->get_column(p + 1)->set_title(player.name());
} // void PartySummary::name_update(Player const& player)

/** event, that the party shall be continued
 **/
void
PartySummary::continue_party_event()
{
  this->hide();
  this->ui->main_quit();
} // void Party

/** event, that a new party shall be started
 **/
void
PartySummary::new_party_event()
{
  this->ui->thrower(GameStatus::party_new, __FILE__, __LINE__);

  this->hide();
  this->ui->main_quit();
} // void PartySummary::new_party_event()

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
