/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#ifdef USE_UI_GTKMM

#pragma once

#include "base.h"
#include "widgets/sticky_dialog.h"
#include <gtkmm/liststore.h>

class Player;

namespace Gtk {
class TreeView;
class Button;
class Label;
} // namespace Gtk

namespace UI_GTKMM_NS {

/**
 ** the summary of a party
 ** The summary is shown, when a party has been loaded.
 ** The summary contains:
 **   players
 **     points
 **     duty soli
 **   remaining points
 **   remaining rounds
 **   gamenumber
 **   next startplayer
 **/
class PartySummary : public Base, public Gtk::StickyDialog {
  /**
   ** the columns of the players table
   **/
  struct PlayersSummaryModel : public Gtk::TreeModel::ColumnRecord {
    PlayersSummaryModel(unsigned playerno);

    Gtk::TreeModelColumn<Glib::ustring> name;
    vector<Gtk::TreeModelColumn<int> > player;
    Gtk::TreeModelColumn<bool> visible;
  }; // struct PlayersSummartyTable : public Gtk::TreeModel::ColumnRecord

  public:
  explicit PartySummary(Base* parent);
  ~PartySummary() override;

  PartySummary() = delete;
  PartySummary(PartySummary const&) = delete;
  PartySummary& operator=(PartySummary const&) = delete;

  // the party is loaded
  void party_loaded();
  // the window is shown
  void on_show() override;

  // update the name of the player
  void name_update(Player const& player);

  private:
  // initialize the window
  void init();
  // update the window
  void update();

  // the party shall be continued
  void continue_party_event();
  // a new party shall be started
  void new_party_event();

  private:
  AutoDisconnector disconnector_;

  std::unique_ptr<PlayersSummaryModel> player_model = nullptr;
  Glib::RefPtr<Gtk::ListStore> player_list;
  Gtk::TreeRow points_row;
  Gtk::TreeRow duty_soli_row;
  Gtk::TreeView* player_treeview = nullptr;

  Gtk::Label* roundnumber = nullptr;
  Gtk::Label* gamenumber = nullptr;
  Gtk::Label* remaining_rounds = nullptr;
  Gtk::Label* remaining_points = nullptr;
  Gtk::Label* next_startplayer = nullptr;

  Gtk::Button* continue_party_button = nullptr;
  Gtk::Button* new_party_button = nullptr;
}; // class PartySummary : public Base, public Gtk::StickyDialog

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
