/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "preferences.h"
#include "ui.h"
#include "cards.h"

#include "../../misc/setting.h"

#include <gtkmm/notebook.h>
#include <gtkmm/image.h>
#include <gtkmm/label.h>
#include <gtkmm/radiobutton.h>
#include <gtkmm/grid.h>
#include <gtkmm/arrow.h>
namespace UI_GTKMM_NS {

/** creates all subelements for the cardsorder
 **/
void
  Preferences::init_cardsorder()
  {
    auto vbox = this->add_group_box("cards order");
    (void)_("Setting::Group::cards order");
    vbox->set_border_width(2 EX);

    { // sorted/unsorted
      auto vbox2 = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL));
      vbox2->set_halign(Gtk::ALIGN_CENTER);

      Gtk::RadioButton::Group group;

      this->sorted_button
        = Gtk::manage(new Gtk::RadioButton(group, _("Button::sorted")));
      vbox2->add(*(this->sorted_button));

      this->unsorted_button
        = Gtk::manage(new Gtk::RadioButton(group, _("Button::unsorted")));
      vbox2->add(*(this->unsorted_button));

      if (::setting(Setting::Type::cards_order).mixed())
        this->unsorted_button->set_active();
      else
        this->sorted_button->set_active();

      this->sorted_button->signal_toggled().connect(sigc::mem_fun(*this, &Preferences::cards_order_set_sorted));
      this->unsorted_button->signal_toggled().connect(sigc::mem_fun(*this, &Preferences::cards_order_set_sorted));
      vbox->add(*vbox2);
    } // sorted/unsorted

    { // the cards
      this->cards_order_container = Gtk::manage(new Gtk::Grid());
      auto grid = this->cards_order_container;
      vbox->add(*grid);
      grid->set_row_spacing(1 EX);
      grid->set_column_spacing(1 EM);

      for (unsigned p = 0; p < Card::number_of_tcolors; ++p) {
        { // left/right button
          auto hbox = Gtk::manage(new Gtk::Box());
          hbox->set_homogeneous();
          grid->attach(*hbox, p, 0, 1, 1);

          auto left_button = Gtk::manage(new Gtk::Button());
          left_button->add(*Gtk::manage(new Gtk::Arrow(Gtk::ARROW_LEFT,
                                                       Gtk::SHADOW_OUT)));
          hbox->add(*left_button);
          left_button->signal_clicked().connect(sigc::bind<unsigned const>(sigc::mem_fun(*this, &Preferences::cards_order_pos_to_left), p));

          auto right_button = Gtk::manage(new Gtk::Button());
          right_button->add(*Gtk::manage(new Gtk::Arrow(Gtk::ARROW_RIGHT,
                                                        Gtk::SHADOW_OUT)));
          hbox->add(*right_button);
          right_button->signal_clicked().connect(sigc::bind<unsigned const>(sigc::mem_fun(*this, &Preferences::cards_order_pos_to_right), p));
        } // left/right button
        { // the image
          if (::setting(Setting::Type::cards_order).tcolor(p) == Card::trump)
            this->sorting_tcolor_image.push_back(this->ui->cards->new_managed_image(Card::club_queen));
          else
            this->sorting_tcolor_image.push_back(this->ui->cards->new_managed_image(Card(::setting(Setting::Type::cards_order).tcolor(p), Card::ace)));
          grid->attach(*(this->sorting_tcolor_image.back()), p, 1, 1, 1);
        } // the image
        { // up/down buttons
          auto vbox2 = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL));
          grid->attach(*vbox2, p, 2, 1, 1);

          Gtk::RadioButton::Group group;
          this->sorting_up_button.push_back(Gtk::manage(new Gtk::RadioButton(group, _(Setting::CardsOrder::Direction::up))));
          vbox2->add(*(this->sorting_up_button.back()));

          this->sorting_down_button.push_back(Gtk::manage(new Gtk::RadioButton(group, _(Setting::CardsOrder::Direction::down))));
          vbox2->add(*(this->sorting_down_button.back()));

          // events
          this->sorting_up_button.back()->set_active(::setting(Setting::Type::cards_order).direction(p) == Setting::CardsOrder::Direction::up);
          this->sorting_down_button.back()->set_active(::setting(Setting::Type::cards_order).direction(p) == Setting::CardsOrder::Direction::down);
          this->sorting_up_button.back()->signal_toggled().connect(sigc::bind<unsigned const>(sigc::mem_fun(*this, &Preferences::cards_order_direction_set), p));
          this->sorting_down_button.back()->signal_toggled().connect(sigc::bind<unsigned const>(sigc::mem_fun(*this, &Preferences::cards_order_direction_set), p));
        } // up/down buttons
      } // for (c < Card::number_of_tcolors)
    } // the cards

    { // mix the cards
      this->mix_cards_button = Gtk::manage(new Gtk::Button(_("Setting::CardsOrder::mix cards")));
      this->mix_cards_button->set_halign(Gtk::ALIGN_CENTER);
      vbox->add(*this->mix_cards_button);
      this->mix_cards_button->signal_clicked().connect(sigc::mem_fun(*this, &Preferences::cards_order_mix));
    } // mix the cards
  } // void Preferences::init_cardsorder()

/** update the cards order
 **/
void
  Preferences::update_cards_order()
  {
    if (::setting(Setting::Type::cards_order).mixed()) {
      this->unsorted_button->set_active();
      this->cards_order_container->set_sensitive(false);
    } else { // if !(::setting(Setting::Type::cards_order).mixed())
      this->sorted_button->set_active();
      this->cards_order_container->set_sensitive(true);

      for (unsigned p = 0; p < Card::number_of_tcolors; ++p) {
        if (::setting(Setting::Type::cards_order).tcolor(p) == Card::trump)
          this->ui->cards->change_managed(this->sorting_tcolor_image[p],
                                          Card::club_queen);
        else
          this->ui->cards->change_managed(this->sorting_tcolor_image[p],
                                          Card(::setting(Setting::Type::cards_order).tcolor(p), Card::ace));

        if (::setting(Setting::Type::cards_order).direction(p)
            == Setting::CardsOrder::Direction::up)
          this->sorting_up_button[p]->set_active();
        else
          this->sorting_down_button[p]->set_active();

      } // for (n < Card::number_of_tcolors)
    } // if !(::setting(Setting::Type::cards_order).mixed())
  } // void Preferences::update_cards_order()

/** show the cards order
 **/
void
  Preferences::show_cards_order()
  {
    this->realize();
    this->group_notebook->set_current_page(5);
    this->present();
  } // void Preferences::show_cards_order()

/** event: set the sorting direction
 **
 ** @param    pos   the position which is changed
 **/
void
  Preferences::cards_order_direction_set(unsigned const pos)
  {
    ::setting(Setting::Type::cards_order).sorted_set(true);
    if (this->sorting_up_button[pos]->get_active())
      ::setting(Setting::Type::cards_order).direction_set(pos,
                                                    Setting::CardsOrder::Direction::up);
    else
      ::setting(Setting::Type::cards_order).direction_set(pos,
                                                    Setting::CardsOrder::Direction::down);
  } // void Preferences::cards_order_direction_set(unsigned pos)

/** event: shift the type at 'pos' one position to the left
 **
 ** @param    pos   the position which is shifted
 **/
void
  Preferences::cards_order_pos_to_left(unsigned const pos)
  {
    ::setting(Setting::Type::cards_order).sorted_set(true);
    ::setting(Setting::Type::cards_order).pos_to_left(pos);
  } // void Preferences::cards_order_pos_to_left(unsigned pos)

/** event: shift the type at 'pos' one position to the right
 **
 ** @param    pos   the position which is shifted
 **/
void
  Preferences::cards_order_pos_to_right(unsigned const pos)
  {
    ::setting(Setting::Type::cards_order).sorted_set(true);
    ::setting(Setting::Type::cards_order).pos_to_right(pos);
  } // void Preferences::cards_order_pos_to_right(unsigned pos)

/** event: set the sorted state
 **/
void
  Preferences::cards_order_set_sorted()
  {
    ::setting(Setting::Type::cards_order).sorted_set(this->sorted_button->get_active());
  } // void Preferences::cards_order_set_sorted()

/** event: mix the cards
 **/
void
  Preferences::cards_order_mix()
  {
    if (::setting(Setting::Type::cards_order).sorted() == true)
      ::setting(Setting::Type::cards_order).sorted_set(false);
    else
      ::setting(Setting::Type::cards_order).mix();
  } // void Preferences::cards_order_mix()

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
