/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#ifdef USE_UI_GTKMM

#pragma once

#include "base.h"

#include "../../basetypes.h"
#include "../../game/reservation.h"
#include "../../game/gameplay_action.h"
#include "../../misc/setting.h"
class Player;
class Trick;

#include <map>

#include <gtkmm/drawingarea.h>
#include <pangomm/layout.h>
#include <gdkmm/pixbuf.h>
#include <gdkmm/cursor.h>

namespace UI_GTKMM_NS {
class Reservation;
class Poverty;
class Trick;
class Hand;
class TrickPile;
class Icongroup;
class Name;
class GameFinished;
class PartyPoints;
class PartyFinished;

class GameInfoDialog;
class FullTrick;
class LastTrick;
class CardSuggestion;
class GenscherSelection;

/**
 ** the table
 **
 ** @todo   keep 'game_finished' window and 'poverty' over the games
 ** @todo   close full trick with click
 ** @todo   use 'Gdk::Point' and 'Gdk::Rectangle'
 **/
class Table : public Base, public Gtk::DrawingArea {
  public:
    enum class CursorType {
      none,
      standard,
      busy,
      poverty_shift,
      poverty_accept,
      poverty_get_card,
      poverty_put_card,
      poverty_fill_up,
      poverty_shift_back,
      poverty_getting_back,
      next_announcement,
      play_card,
      card_valid,
      card_invalid,
      close_full_trick,
      show_last_trick,
      game_review
    }; // enum CursorType

    enum class Layout {
      standard,
      widescreen
    }; // enum class Layout

  public:
    class Part;
    class HTIN;

    friend class Part;
    friend class HTIN;
    friend class Hand;
    friend class UI_GTKMM;
    friend class Menu;
  public:
    explicit Table(Base* parent);
    ~Table() override;

    Table() = delete;
    Table(Table const&) = delete;
    Table& operator=(Table const&) = delete;

    PartyPoints& party_points();

    Poverty& poverty();
    Trick& trick();
    Hand& hand(Player const& player);
    Hand const& hand(Player const& player) const;
    Hand& hand(Position position);
    Hand const& hand(Position position) const;
    TrickPile& trickpile(Player const& player);
    TrickPile const& trickpile(Player const& player) const;
    TrickPile& trickpile(Position position);
    TrickPile const& trickpile(Position position) const;
    Icongroup& icongroup(Player const& player);
    Icongroup const& icongroup(Player const& player) const;
    Icongroup& icongroup(Position position);
    Icongroup const& icongroup(Position position) const;
    Name& name(Player const& player);
    Name const& name(Player const& player) const;
    Name& name(Position position);
    Name const& name(Position position) const;
    Reservation& reservation(Player const& player);
    Reservation const& reservation(Player const& player) const;
    Reservation& reservation(Position position);
    Reservation const& reservation(Position position) const;

    bool get_card() const;

    Player const& player(Position position) const;
    Position position(Player const& player) const;
    Rotation rotation(Player const& player) const;
    Rotation rotation(Position position) const;

    Cairo::RefPtr<Cairo::Context> create_cairo_context();

    int width() const;
    int height() const;
    Layout layout() const;

    int border_x() const;
    int border_y() const;

    void party_finish();

    // a gameplay action
    void gameplay_action(GameplayAction const& action);

    void game_open();
    void game_cards_distributed();
    void game_redistribute();
    void reservation_ask(Player const& player);
    ::Reservation reservation_get(Player const& player);
    void game_start();
    void card_get(Player const& player);
    void card_got();
    void card_played(Player const& player);
    void trick_open();
    void trick_full();
    void trick_move_in_pile();
    void show_last_trick();
    void show_trick(::Trick const& trick);
    void show_card_suggestion(bool show_window = true);
    void game_finished();
    void game_close();

    bool in_game_review() const;
    unsigned game_review_trickno() const;
    ::Trick const& game_review_trick() const;
    bool game_review_trick_visible() const;

    void announcement_made(Announcement announcement,
                           Player const& player);
    void swines_announced(Player const& player);
    void hyperswines_announced(Player const& player);

    void marriage(Player const& bridegroom, Player const& bride);
    Player const* select_genscher_partner(Player const& genscher);
    void genscher(Player const& genscher, Player const& partner);

    void update_human_teaminfo();

    void update_from_buffer();
    void draw_all();
    void force_redraw_all();

    void draw_logo(Cairo::RefPtr<::Cairo::Context> cr);
    void draw_party_summary();
    void update_hands();
    void update_cards();
    void update_cards_back();
    void update_icons();

    void load_background();

    void save_screenshot() const;

    void cards_update() override;
    void cards_back_update() override;
    void name_changed(Player const& player);

    void new_font(string const& fontname, Setting::Type::String type);
    void new_color(string const& colorname, Setting::Type::String type);

    void setting_update(int type) override;
    void mouse_cursor_update();

    void progress_changed(double progress);
    void progress_finished();

  private:
    bool on_draw(::Cairo::RefPtr<::Cairo::Context> const& cr) override;
    bool on_configure_event(GdkEventConfigure* event) override;
    bool on_button_press_event(GdkEventButton* event) override;
    bool on_scroll_event(GdkEventScroll* event) override;
    bool on_motion_notify_event(GdkEventMotion* event) override;

  private:
    AutoDisconnector disconnector_;

    Glib::RefPtr<Gdk::Cursor> cursor(CursorType type);

  public:
    CursorType cursor_type = CursorType::none;

    Cairo::RefPtr<Cairo::SurfacePattern> background;

  private:
    std::map<Position, unique_ptr<Reservation>> reservation_;
    unique_ptr<Poverty> poverty_;
    unique_ptr<GameFinished> game_finished_;
    unique_ptr<PartyPoints> party_points_;
    unique_ptr<PartyFinished> party_finished_;
    unique_ptr<FullTrick> full_trick_;
    unique_ptr<LastTrick> last_trick_;
    unique_ptr<CardSuggestion> card_suggestion_;
    unique_ptr<GenscherSelection> genscher_selection_;

    unique_ptr<GameInfoDialog> gameinfo_;

    unique_ptr<Trick> trick_;
    std::map<Position, unique_ptr<Hand>> hand_;
    std::map<Position, unique_ptr<TrickPile>> trickpile_;
    std::map<Position, unique_ptr<Icongroup>> icongroup_;
    std::map<Position, unique_ptr<Name>> name_;

    vector<Part*> part_;

    bool get_card_ = false;

    Cairo::RefPtr<Cairo::ImageSurface> surface_;
    Layout layout_ = Layout::standard;
}; // class Table : public Base, protected Gtk::DrawingArea

string to_string(Table::Layout layout);
ostream& operator<<(ostream& ostr, Table::Layout layout);

void draw_pixbuf(Cairo::RefPtr<::Cairo::Context> cr,
                 Glib::RefPtr<Gdk::Pixbuf> pixbuf, int x, int y);
void draw_pixbuf(Cairo::RefPtr<::Cairo::Context> cr,
                 Glib::RefPtr<Gdk::Pixbuf> pixbuf, Gdk::Point const& pos);
} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
