/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#ifdef USE_UI_GTKMM

#pragma once

#include "base.h"

#include "table.h"

namespace UI_GTKMM_NS {
/**
 ** a part of a table
 **/
class Table::Part : public Base {
public:
  /**
   ** The outline of the part
   **/
  struct Outline {
    Outline() = default;
    Outline(int x, int y, int width, int height);
    Outline(function<int()> x, function<int()> y, function<int()> width, function<int()> height);
    function<int()> x;
    function<int()> y;
    function<int()> width;
    function<int()> height;
    operator Gdk::Rectangle() const;
  }; // struct Outline
  /**
   ** The size for drawing the part
   **/
  struct Geometry {
    function<int()> pos_x;
    function<int()> pos_y;
    Rotation rotation = Rotation::up;
    // relative to the rotation
    function<int()> width;
    function<int()> height;

    virtual Outline outline() const = 0;
    void transform(Cairo::RefPtr<::Cairo::Context> cr) const;
    Outline transform(Outline outline) const;
    void retransform(int& x, int& y) const;
  }; // struct Geometry

public:
  explicit Part(Table* table);
  ~Part() override;

  Table& table();
  Table const& table() const;
  Game const& game() const;

  virtual Cairo::RefPtr<Cairo::ImageSurface> surface();

  // whether there was a change in the element
  virtual bool changed() const = 0;
  virtual Outline outline() const = 0;

  virtual void draw(Cairo::RefPtr<::Cairo::Context> cr) = 0;
  void force_redraw();

protected:
  Cairo::RefPtr<Cairo::ImageSurface> surface_;

private:
  Part() = delete;
  Part(Part const&) = delete;
  Part& operator=(Part const&) = delete;
}; // class Table::Part : public Base

ostream& operator<<(ostream& ostr, Table::Part::Outline const& outline);
ostream& operator<<(ostream& ostr, Table::Part::Geometry const& geometry);
Table::Part::Outline operator+(Table::Part::Outline lhs,
                               Table::Part::Outline rhs);

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
