/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public Rules as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the Rules, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public Rules for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public Rules
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "file_chooser_dialog_wrapper.h"

namespace Gtk {

/** constructor
 **
 ** @param    title    the title of the window
 ** @param    action   save or open
 ** @param    slot     slot for the save/load signal
 **/
FileChooserDialogWrapper::FileChooserDialogWrapper(Glib::ustring const& title,
                                                   Gtk::FileChooserAction action,
                                                   sigc::slot1<bool, string> slot) :
  title_(title),
  action_(action),
  slot_(slot)
{ }

/** destructor
 **/
FileChooserDialogWrapper::~FileChooserDialogWrapper() = default;

/** set the current directory
 **
 ** @param    dir   the current directory
 **/
void
FileChooserDialogWrapper::set_current_folder(string const dir)
{
  this->current_folder_ = dir;
  if (this->file_chooser_dialog_)
    this->file_chooser_dialog_->set_current_folder(dir);
} // set_current_folder(string dir)

/** sets the title
 **
 ** @param    title   new title of the dialog
 **/
void
FileChooserDialogWrapper::set_title(string const title)
{
  this->title_ = title;
  if (this->file_chooser_dialog_)
    this->file_chooser_dialog_->set_title(title);
} // set_title(string title)

/** show the window
 **/
void
FileChooserDialogWrapper::present()
{
  if (!this->file_chooser_dialog_)
    this->init();

  this->file_chooser_dialog_->present();
} // void present()

/** event: the window is shown
 ** move the window to the remembered position
 **/
void
FileChooserDialogWrapper::init()
{
  this->file_chooser_dialog_ = make_unique<FileChooserDialog>(this->title_, this->action_);
  auto cancel_button = this->file_chooser_dialog_->add_button(_("Button::cancel"), Gtk::RESPONSE_CANCEL);
  cancel_button->set_image_from_icon_name("window-close");
  cancel_button->set_always_show_image();
  auto ok_button = this->file_chooser_dialog_->add_button((this->action_ == Gtk::FILE_CHOOSER_ACTION_SAVE)
                                                          ? _("Button::save")
                                                          : _("Button::load"),
                                                          Gtk::RESPONSE_OK);
  ok_button->set_image_from_icon_name((this->action_ == Gtk::FILE_CHOOSER_ACTION_SAVE)
                                      ? "document-save"
                                      : "document-open");
  ok_button->set_always_show_image();

  { // signals
    this->file_chooser_dialog_->signal_file_activated().connect(sigc::mem_fun(*this, &FileChooserDialogWrapper::signal_ok));
    this->file_chooser_dialog_->signal_file_activated().connect(sigc::mem_fun(*this->file_chooser_dialog_, &Widget::hide));

    ok_button->signal_clicked().connect(sigc::mem_fun(*this, &FileChooserDialogWrapper::signal_ok));
    ok_button->signal_clicked().connect(sigc::mem_fun(*this->file_chooser_dialog_, &Widget::hide));
    cancel_button->signal_clicked().connect(sigc::mem_fun(*this->file_chooser_dialog_, &Widget::hide));
  } // signals

  this->file_chooser_dialog_->set_current_folder(this->current_folder_);
} // void init()

/** the OK button was clicked
 **/
void
FileChooserDialogWrapper::signal_ok()
{
  this->slot_(this->file_chooser_dialog_->get_filename());
} // void signal_ok()

} // namespace Gtk

#endif // #ifdef USE_UI_GTKMM
