/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public Rules as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the Rules, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public Rules for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public Rules
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "labelentry.h"

namespace Gtk {

/** constructor
 **/
LabelEntry::LabelEntry() :
  LabelEntry("")
{  }

/** constructor
 **
 ** @param    label   text for the label
 **/
LabelEntry::LabelEntry(Glib::ustring const& text) :
  Box(),
  label_(Gtk::manage(new Gtk::Label(text))),
  entry_(Gtk::manage(new Gtk::Entry()))
{ 
  this->set_spacing(1 EX);

  this->label_->set_halign(Gtk::ALIGN_CENTER);
  this->pack_start(*this->label_, true, true);
  this->pack_end(*this->entry_, false, true);
} // LabelEntry::LabelEntry(Glib::ustring text)

/** destructor
 **/
LabelEntry::~LabelEntry() = default;

/** @return   the label
 **/
Label*
LabelEntry::get_label()
{
  return this->label_;
}

/** set the text for the label
 **
 ** @param    text   text for the label
 **
 ** @return   the label
 **/
void
LabelEntry::set_label(Glib::ustring const& text)
{
  this->label_->set_text(text);
}

/** @return   the entry
 **/
Entry*
LabelEntry::get_entry()
{
  return this->entry_;
}

/** @return   the text of the entry
 **/
Glib::ustring
LabelEntry::get_text()
{
  return this->entry_->get_text();
}

/** set the text for the entry
 **
 ** @param    text   text for the entry
 **/
void
LabelEntry::set_text(Glib::ustring const& text)
{
  this->entry_->set_text(text);
}

} // namespace Gtk

#endif // #ifdef USE_UI_GTKMM
