/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public Rules as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the Rules, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public Rules for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public Rules
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "labelspinbutton.h"

namespace Gtk {

/** constructor
 **/
LabelSpinButton::LabelSpinButton() :
LabelSpinButton("")
{  }

/** constructor
 **
 ** @param    label    name for the label
 **/
LabelSpinButton::LabelSpinButton(Glib::ustring const& label) :
  Box(),
  label_(Gtk::manage(new Gtk::Label(label))),
  spin_button_(Gtk::manage(new Gtk::SpinButton()))
{ 
  this->spin_button_->set_numeric(true);
  this->set_spacing(10);
  this->spin_button_->set_increments(1, 10);

  this->label_->set_halign(Gtk::ALIGN_START);
  this->pack_start(*this->label_, true, true);
  this->pack_start(*this->spin_button_, false, true);
} // LabelSpinButton::LabelSpinButton(Glib::ustring label);

/** destructor
 **/
LabelSpinButton::~LabelSpinButton()
  = default;

/** @return   the label
 **/
Label*
LabelSpinButton::get_label()
{
  return this->label_;
}

/** set the label
 **
 ** @param    str   the string for the label
 **/
void
LabelSpinButton::set_label(Glib::ustring const& str)
{
  this->label_->set_text(str);
} // void LabelSpinButton::set_label(Glib::ustring str)

/** @return   the label
 **/
SpinButton*
LabelSpinButton::get_spin_button()
{
  return this->spin_button_;
}

/** set the range
 **
 ** @param    min    minimum value
 ** @param    max    maximum value
 **/
void
LabelSpinButton::set_range(double const min, double const max)
{ 
  this->spin_button_->set_range(min, max);
} // void LabelSpinButton::set_range(double min, double max)

/** @return   the value
 **/
double
LabelSpinButton::get_value()
{
  return this->spin_button_->get_value();
}

/** @return   the value
 **/
int
LabelSpinButton::get_value_as_int()
{
  return this->spin_button_->get_value_as_int();
}

/** set the value
 **
 ** @param    value    new value
 **/
void
LabelSpinButton::set_value(double const value)
{
  return this->spin_button_->set_value(value);
}

/** @return   the signal for value changes
 **/
Glib::SignalProxy0<void>
LabelSpinButton::signal_value_changed()
{
  return this->spin_button_->signal_value_changed();
}

} // namespace Gtk

#endif // #ifdef USE_UI_GTKMM
