/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001 – 2018 by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **********************************************************************/

#pragma once

// some utils
// D.K.

#include "class/readconfig/readconfig.h"

#define QUAD(x) ((x) * (x))

#undef VOID
#define VOID

#ifdef MS_VISUAL_C
template<typename T>
inline T const& max(T const& a, T const& b)
{
  return ((a > b) ? a : b);
}
template<typename T>
inline T const& min(T const& a, T const& b)
{
  return ((a < b) ? a : b);
}
#endif

// writes a mark with a counter
unsigned MARK(char const* const string = nullptr, bool increase = true);


// file-utils

// all things written into this stream are ignored
class NullOStr : public std::ostream {
  public:
    NullOStr() : std::ostream(nullptr) {}
    ~NullOStr() override = default;

    template <class T>
      NullOStr const& operator<<(T const& t) const
      { (void) t; return *this; }
};
extern NullOStr null_ostr;


// reads a line from the input stream
string read_line(std::istream& istr);

// generate an error
void generate_error(string const error_message = "generated error");



// this class is an unsigned
// the difference is, that 'UINT_MAX' written in /read from a stream is a '-'
class Unsigned {
  public:
    Unsigned() = default;
    Unsigned(unsigned u) : u(u)   { }
    explicit Unsigned(string const& s);
    operator unsigned&() { return u; }
    operator unsigned const&() const { return u; }
    ~Unsigned() = default;
  private:
    unsigned u = 0;
}; // class Unsigned
std::ostream& operator<<(std::ostream& ostr, Unsigned const& u);
std::istream& operator>>(std::istream& istr, Unsigned& u);

#include "utils/optional.h"
// write and read -- no value is a '-'
std::ostream& operator<<(std::ostream& ostr, optional<unsigned> const& u);
std::istream& operator>>(std::istream& istr, optional<unsigned>& u);

// output of the name
#define WRITE_NAME(Type) \
  inline ostream& operator<<(ostream& ostr, Type const& t) \
{ return (ostr << ::name(t)); }
