/**
 **   Copyright (C) 2001 – 2018 by Diether Knof
 **
 **   This program is free software; you can redistribute it and/or 
 **   modify it under the terms of the GNU General Public License as 
 **   published by the Free Software Foundation; either version 2 of 
 **   the License, or (at your option) any later version.
 **
 **   This program is distributed in the hope that it will be useful,
 **   but WITHOUT ANY WARRANTY; without even the implied warranty of
 **   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **   GNU General Public License for more details. 
 **   You can find this license in the file 'gpl.txt'.
 **
 **   You should have received a copy of the GNU General Public License
 **   along with this program; if not, write to the Free Software
 **   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 **   MA  02111-1307  USA
 **
 **  Contact:
 **    Diether Knof dknof@posteo.de
 **/

#pragma once

#include <algorithm>

namespace std {
// standard algorithms for containers
template<typename T, typename P>
  auto find(T& container, P p)
  {
    return std::find(begin(container), end(container), p);
  }
template<typename T, typename P>
  auto contains(T& container, P p)
  {
    return (std::find(begin(container), end(container), p) != end(container));
  }
template<typename T, typename P>
  auto for_each(T& container, P p)
  {
    return std::for_each(begin(container), end(container), p);
  }
template<typename T, typename P>
  auto count(T const& container, P const& p)
  {
    return std::count(begin(container), end(container), p);
  }
template<typename T, typename P>
  auto count_if(T const& container, P p)
  {
    return std::count_if(begin(container), end(container), p);
  }
template<typename T, typename P>
  auto find_if(T& container, P p)
  {
    return std::find_if(begin(container), end(container), p);
  }
template<typename T, typename P>
  auto contains_if(T& container, P p)
  {
    return (std::find_if(begin(container), end(container), p) != end(container));
  }
template<typename T>
  auto unique(T& container)
  {
    auto p = std::unique(begin(container), end(container));
    container.erase(p, end(container));
  }
template<typename T, typename P>
  auto max_element(T const& container, P p)
  {
    return std::max_element(begin(container), end(container), p);
  }
template<typename T, typename P>
  auto min_element(T const& container, P p)
  {
    return std::min_element(begin(container), end(container), p);
  }
} // namespace std
