/***************************************************************************
 *
 *   Copyright (C) 2001 – 2018 by Diether Knof
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@posteo.de
 *
 **************************************************************************/

#include "parser.h"
#include "string.h"

namespace Parser {

/** Gibt zurück, ob text leer ist
 ** Abschlussroutine für die parse-Templates
 **/
bool
parse(string& text)
{
  return text.empty();
} // bool parse(string& text)

/** Liest n aus text, entfernt den gelesenen Text und die folgenden Leerzeichen
 ** Ändert text nur, wenn er mit einem unsigned beginnt.
 **
 ** @return   true, wenn text mit einem unsigned beginnt, sonst false
 **/
bool
parse_(string& text, unsigned& n)
{
  try {
    n = std::stou(text);
    while (!text.empty() && ::isdigit(text.front()))
      text.erase(0, 1);
    String::remove_leading_blanks(text);
    return true;
  } catch (...) {
    return false;
  }
} // bool parse_(string& text, unsigned& n)

/** Speichert den Text in remaining_text
 **
 ** @return   true, wenn text nicht leer ist
 **/
bool
parse_(string& text, string& remaining_text)
{
  if (text.empty())
    return false;
  remaining_text = text;
  text.clear();
  return true;
} // bool parse_(string& text, string& remaining_text)

/** Liest fixtext aus text, entfernt ihn und die folgenden Leerzeichen
 ** Ändert text nur, wenn er mit fixtext beginnt.
 **
 ** @return   true, wenn text mit fixtext beginnt, sonst false
 **/
bool
parse_(string& text, char const* fixtext)
{
  auto const n = strlen(fixtext);
  if (text.compare(0, n, fixtext))
    return false;
  text.erase(0, n);
  String::remove_leading_blanks(text);
  return true;
} // bool parse_(string& text, char const* fixtext)

}; // namespace Parser
