/**
 **   Copyright (C) 2001 – 2018 by Diether Knof
 **
 **   This program is free software; you can redistribute it and/or 
 **   modify it under the terms of the GNU General Public License as 
 **   published by the Free Software Foundation; either version 2 of 
 **   the License, or (at your option) any later version.
 **
 **   This program is distributed in the hope that it will be useful,
 **   but WITHOUT ANY WARRANTY; without even the implied warranty of
 **   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 **   GNU General Public License for more details. 
 **   You can find this license in the file 'gpl.txt'.
 **
 **   You should have received a copy of the GNU General Public License
 **   along with this program; if not, write to the Free Software
 **   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 **   MA  02111-1307  USA
 **
 **  Contact:
 **    Diether Knof dknof@posteo.de
 **/

#pragma once

#include "date.h"

#include <memory>
#include <random>

/**
 ** Creates an integer random value
 **/
class RandomBase {
  public:
    using Type = std::uint_fast32_t;

    static void test();

  public:
    RandomBase() noexcept = default;
    virtual ~RandomBase() = default;

    virtual void set_random_seed() = 0;
    virtual void set_seed(Type seed) = 0;
    virtual Type seed() const = 0;

    // returns the next random number
    virtual Type get() = 0;
    virtual int operator()(int end) = 0;
}; // class RandomBase

/**
 ** current random number generator (since 0.7.17)
 **/
class RandomMT19937 : public RandomBase {
  public:
  RandomMT19937() noexcept;
  explicit RandomMT19937(Type seed) noexcept;
  ~RandomMT19937() override;

  static Type max_seed();

  void set_random_seed() override;
  void set_seed(Type seed) override;
  Type seed() const override;

  // returns the next random number
  Type get() override;
  int operator()(int end) override;

  private:
  std::mt19937 engine_;
  Type seed_;
}; // class RandomMT19937 : public RandomBase

/**
 ** old random number generator (pre 0.7.17)
 **/
class RandomPre_0_7_17 : public RandomBase {
  static constexpr int a_ = 7200;
  static constexpr int c_ = 100;
  static constexpr int m_ = 279841;

  public:
  RandomPre_0_7_17() noexcept;
  explicit RandomPre_0_7_17(Type seed) noexcept;
  ~RandomPre_0_7_17() override;

  static Type max_seed();

  void set_random_seed() override;
  void set_seed(Type seed) override;
  Type seed() const override;

  // returns the next random number
  Type get() override;
  int operator()(int end) override;

  private:
  Type seed_;
}; // class RandomPre_0_7_17 : public RandomBase

using Random = RandomMT19937;

extern Random random_value;
